#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Control/HyperLink.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: 

package SAPDB::Install::Control::HyperLink;

use Wx::Event qw (
	EVT_LEFT_DOWN EVT_LEFT_UP EVT_KEY_DOWN EVT_PAINT
	EVT_ENTER_WINDOW EVT_LEAVE_WINDOW EVT_SIZE);

use Wx qw(
	wxDECORATIVE wxNORMAL wxICON_ERROR wxHORIZONTAL
	wxEVT_COMMAND_BUTTON_CLICKED wxWANTS_CHARS
	wxALIGN_CENTER wxLEFT wxRIGHT wxNORMAL wxSTANDARD_CURSOR
	wxNORMAL wxBOLD wxBLUE wxSOLID wxCURSOR_HAND);

use base SAPDB::Install::Helpers::PassiveWin;

sub new{
	my $self = shift->SUPER::new(@_);
	$self->{'mdc'} = Wx::MemoryDC->new();
	$self->{'fmdc'} = Wx::MemoryDC->new();
	$self->{'font'} = Wx::Font->new(10,$^O =~ /mswin/i ? wxNORMAL : wxDECORATIVE, wxNORMAL, wxBOLD, 0 );
	$self->{'font'}->SetUnderlined(1);
	$self->{'brush'} = Wx::Brush->new($self->GetParent()->GetBackgroundColour(),wxSOLID);
	$self->{'mdc'}->SetBackground($self->{'brush'});
	$self->{'mdc'}->SetFont($self->{'font'});
	$self->{'mdc'}->SetTextForeground(wxBLUE);
	$self->{'mdc'}->SelectObject(Wx::Bitmap->new(1,1));
	$self->{'fmdc'}->SetBackground($self->{'brush'});
	$self->{'fmdc'}->SetFont($self->{'font'});
	$self->{'fmdc'}->SetTextForeground(wxBLUE);
	EVT_SIZE($self,\&OnSize);
	EVT_ENTER_WINDOW($self,\&OnEnter);
	EVT_LEAVE_WINDOW($self,\&OnLeave);
	EVT_LEFT_DOWN($self,\&OnLeftDown);
	EVT_PAINT($self,\&OnPaint);
	$self->SetCursor(Wx::Cursor->new(wxCURSOR_HAND));
	$self->SetNextHandler($self->GetParent);
	return $self;
}

sub select{
	my ($self) = @_;
	return if $self->{'selected'}; 
	unless(defined $self->{'command_event'}){
		$self->{'command_event'} = Wx::CommandEvent->new(wxEVT_COMMAND_BUTTON_CLICKED,$self->GetId);
	}
	$self->GetParent->AddPendingEvent($self->{'command_event'});
	$self->{'selected'} = 1;
	$self->Refresh(1);
	my $fwin = Wx::Window::FindFocus(); 
	if(defined $fwin and Wx::GetTopLevelParent($fwin)->GetId == Wx::GetTopLevelParent($self)->GetId){
		$self->SetFocus;
	}
}

sub unselect{
	my ($self) = @_;
	return unless $self->{'selected'};
	$self->{'selected'} = 0;
	$self->Refresh(1);
}


sub Render{
	my ($self) =@_;
	return undef unless defined $self->{'label'}; 
	my ($x,$y) = $self->GetSizeWH;
	$self->{'mdc'}->SelectObject(Wx::Bitmap->new($x,$y));
	$self->{'mdc'}->BeginDrawing() if $^O =~ /mswin/i;
  	$self->{'mdc'}->Clear;
	$self->{'mdc'}->SetTextForeground(wxBLUE);
	$self->{'font'}->SetUnderlined(1);
	$self->{'mdc'}->DrawText($self->{'label'},0,0);
	$self->{'mdc'}->EndDrawing() if $^O =~ /mswin/i; 
	$self->{'fmdc'}->SelectObject(Wx::Bitmap->new($x,$y));
	$self->{'fmdc'}->BeginDrawing() if $^O =~ /mswin/i;
  	$self->{'fmdc'}->Clear;
	$self->{'fmdc'}->SetTextForeground(Wx::Colour->new(150,0,150));
	$self->{'fmdc'}->DrawText($self->{'label'},0,0);
	$self->{'fmdc'}->EndDrawing() if $^O =~ /mswin/i; 
}

sub OnPaint{
	my ($self,$evt) = @_;
	return undef unless defined $self->{'label'};
	return unless defined $self->{'mdc'};
	my $pdc = Wx::PaintDC->new($self);
	if($self->{'selected'}){
		$pdc->Blit(0,0,$self->GetSizeWH,$self->{'fmdc'},0,0);
	}
	else{
		$pdc->Blit(0,0,$self->GetSizeWH,$self->{'mdc'},0,0);
	}
}


sub OnSize{
	my ($self,$evt) = @_;
	#$self->Render; 
}

sub OnLeave{
	my ($self,$evt) = @_;
	$self->unselect;
}

sub OnEnter{
	my ($self,$evt) = @_;
	$self->select;
}

sub OnLeftDown{
	my ($self,$evt) = @_;
	$self->Call;
}


sub SetLabel{
	my ($self,$label) = @_;
	$self->{'label'} = $label;
	$self->{'mdc'}->SetFont($self->{'font'});
	my ($x,$y,$z,$a) = $self->{'mdc'}->GetTextExtent($label);
	$self->SetSize($x + 5 ,$y);
	$self->Render;
}

sub GetLabel{
	my ($self) = @_;
	return $self->{'label'};
}

sub SetFunc{
	my ($self,$func) = @_;
	return undef unless (ref($func) eq 'CODE');
	$self->{'func'} = $func;
}


sub Call{
	my ($self) = @_;
	return undef unless defined $self->{'func'};
	return &{$self->{'func'}}();
}

sub Destroy{
	my ($self) = @_;
	$self->SUPER::Destroy;

}

sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;