#!/usr/bin/perl
#
# $Header: //sapdb/V76/develop/sys/src/install/perl/SAPDB/Install/App.pm#1 $
# $DateTime: 2004/03/11 10:29:37 $
# $Change: 66919 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade, called by SDBINST/SDBUPD

package SAPDB::Install::Frame::Setup;

use Wx::Event qw (EVT_SIZE EVT_BUTTON EVT_CHECKBOX);
use Wx qw (
	wxEXPAND wxALL wxALIGN_RIGHT wxBOTTOM
	wxRA_SPECIFY_ROWS wxVERTICAL wxICON_ERROR
	wxCENTRE wxHORIZONTAL wxLEFT wxRIGHT);

use SAPDB::Install::Frame::Base;
use SAPDB::Install::App::Installer;
use SAPDB::Install::App::CreateInstance;

our @ISA = qw (SAPDB::Install::Frame::Base);


sub new {
	my $id = 1;
 	my( $class ) = shift;
	my( $this ) = $class->SUPER::new($_[0], $_[1], $_[2] );
	my $panel = Wx::Panel->new($this,$id++);
	my $top_sizer =  Wx::BoxSizer->new(wxVERTICAL);
	my $panel_sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $group_sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $cb_sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $rbox_sizer = Wx::BoxSizer->new(wxVERTICAL);
		
	$this->{'components'} = {
			'server' => {'label' => 'Server','seq' => 0},
			'client' => {'label' => 'Client','seq' => 1},
			'conn' => {'label' => 'Connectivity','seq' => 2},
			'devel' => {'label' => 'Development','seq' => 3},
		};
	my $group_box = Wx::StaticBox->new($panel,$id++,'Software Components',[0,0],[10,10]);
	
	my $rbox = Wx::RadioBox->new($group_box,$id++,'',[0,0],[10,10],
								['Create New Database','Update Existing Database','Software Installation Only'],
								3,wxRA_SPECIFY_ROWS );
	$rbox->SetSize($rbox->GetBestSize);
	$rbox_sizer->Add($rbox,0,wxLEFT|wxBOTTOM,10);
	$rbox->SetSelection(2);
	$this->{'rbox'} = $rbox;
	foreach my $comp (sort {$this->{'components'}->{$a}->{'seq'} <=> $this->{'components'}->{$b}->{'seq'}} keys(%{$this->{'components'}})){
		$this->{'components'}->{$comp}->{'cb'} =  Wx::CheckBox->new($group_box,$id++,$this->{'components'}->{$comp}->{'label'});
		$this->{'components'}->{$comp}->{'cb'}->SetSize($this->{'components'}->{$comp}->{'cb'}->GetBestSize);
		$cb_sizer->Add($this->{'components'}->{$comp}->{'cb'},1,wxEXPAND);
		if($comp eq 'server'){
			EVT_CHECKBOX($group_box,$this->{'components'}->{$comp}->{'cb'},\&OnServerClick);
			$this->{'components'}->{$comp}->{'cb'}->SetValue(1);
			$cb_sizer->Add($rbox_sizer,0);		
		}
		else{
			EVT_CHECKBOX($group_box,$this->{'components'}->{$comp}->{'cb'},\&OnCheckBoxClick);
		}
		
	}
	my $ID_OK = $id;
	my $ok_button = Wx::Button->new($panel,$id++,'&OK');
	$this->{'ok_button'} = $ok_button;
	my $ID_CANCEL = $id;
	my $cancel_button = Wx::Button->new($panel,$id++,'&Cancel');
	my $button_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$button_sizer->Add($ok_button,1, wxRIGHT | wxBOTTOM,20);
	$button_sizer->Add($cancel_button,1,wxRIGHT | wxBOTTOM,20);
	EVT_BUTTON($panel,$ID_OK,\&OnOK);
	EVT_BUTTON($panel,$ID_CANCEL,\&OnCancel);
	$group_sizer->Add($cb_sizer,1,wxEXPAND | wxALL, 20);
	$group_box->SetSizer($group_sizer);
	$group_box->SetAutoLayout(1);
	EVT_SIZE($group_box,\&OnGroupBoxSize);
	$top_sizer->Add($panel,1, wxEXPAND);
	$panel_sizer->Add($group_box,1,wxEXPAND | wxALL , 20);
	$panel_sizer->Add($button_sizer,0,wxALIGN_RIGHT);
	$panel->SetSizer($panel_sizer);
	$panel->SetAutoLayout(1);
	$this->SetSizer($top_sizer);
	$this->SetAutoLayout(1);
	$this->Layout();
	$this->Show(1);
	$this;
}

sub OnServerClick{
	my ($group_box,$evt) = @_;
	my $self = $group_box->GetParent()->GetParent();
	if($evt->IsChecked){
		$self->{'rbox'}->Enable(1);
	}
	else{
		$self->{'rbox'}->Enable(0);
	}
	$self->CheckOKButton;		
}

sub CheckOKButton{
	my ($self) = @_;
	my @checked;
	foreach my $comp (keys(%{$self->{'components'}})){
		if($self->{'components'}->{$comp}->{'cb'}->IsChecked){
			push @checked,$comp;
		}
	}
	if($#checked == -1){
		$self->{'ok_button'}->Enable(0);
	}
	else{
		$self->{'ok_button'}->Enable(1);
	}
}



sub OnCheckBoxClick{
	my ($group_box,$evt) = @_;
	my $self = $group_box->GetParent()->GetParent();
	$self->CheckOKButton;
}

sub OnGroupBoxSize{
	my($self,$evt) = @_;
	$self->Layout();
	$evt->Skip();	
}

sub OnCancel{
	my ($panel,$evt) = @_;
	$panel->GetParent->Close(1);
}

sub OnOK{
	my ($panel,$evt) = @_;
	my $self = $panel->GetParent;
	my @result;
	foreach my $comp  (keys(%{$self->{'components'}})){
		if($self->{'components'}->{$comp}->{'cb'}->IsChecked){
			push @result,$comp;
		}
	}
	
	if($#result > -1){
		if($self->{'components'}->{'server'}->{'cb'}->IsChecked && not ($self->{'rbox'}->GetSelection == 2)){
			if($self->{'rbox'}->GetSelection){
				my $msgbox = Wx::MessageDialog->new($self,"Database Update not yet implemented!\n",
							"Missing Feature", wxICON_ERROR | wxCENTRE);
				$msgbox->ShowModal();
				return;
			}
			else{
					my $instapp = SAPDB::Install::App::CreateInstance::main({'components' => \@result});
			}
		}
		else{
			my $instapp = SAPDB::Install::App::Installer::main({'components' => \@result});
		}	
	}
	else{
		return;
	}
	
	$self->Destroy;
		
}



1;
