#!/usr/bin/perl
package SAPDB::Install::Templates::Simple;

#ifdef SCRIPTING
#use strict;
#use SAPDB::Install::Templates::Base;
#use SAPDB::Install::Templates::Util;
#endif
  #ifdef PERL5.6.1
#  require Exporter;
#  our @ISA = qw ( SAPDB::Install::Templates::Base Exporter );
#  our @EXPORT=qw(  );
  #endif
  #ifdef PERL5.0.0
#  require Exporter;
#  @ISA = qw ( SAPDB::Install::Templates::Base  Exporter );
#  @EXPORT=qw( );
  #endif

#ifdef doc
#<h1>SAPDB::Install::Templates::Simple </h1>
#<h2><code>Extends <a href="Base.html">Base</a> </code></h2>
#<p>
#Template for simple database  instances.
#</p>
#<h2>Usage</h2>
#<pre>
#  my $mi = SAPDB::Install::Templates::Simple::new();
#  my $mi->set...();
#  my $mi->createInstance();
#</pre>
#endif
#because SDBINST

use SAPDB::Install::Templates::Base;
use SAPDB::Install::Templates::Util;

our @ISA = qw(SAPDB::Install::Templates::Base);

#endbecause





sub getDBVolumes(){
my ($this)=@_;
$this->initializeVolumeDefinitions();
return $this->SUPER::getDBVolumes();
}



sub setDefaults{
  my ($this)=@_;
  $this->SUPER::setDefaults();

  $this->addAfterAdvice("DBMemoryKey",sub{
                          my ($this) = @_;
                          my $mem = $this->getDBMemory();
                          if(defined $mem && $mem =~ /\d+/ ){
                            $this->addDBParam("CACHE_SIZE", $mem);
                          }
                        }
                       );

  $this->addAfterAdvice("CPUCountKey",sub{
                          my ($this) = @_;
                          my $cpu = $this->getCPUCount();
                          if(defined $cpu && $cpu =~ /\d+/ ){
                            $this->addDBParam("MAXCPU", $cpu);
                          }
                        }
                       );

  $this->addAfterAdvice("DBSizeKey",sub{
                          my ($this) = @_;
                          my $size = $this->getDBSize();
                          my $templateCode = $this->getTemplateCode();

                           if("templ_server_advanced" eq $templateCode){
                           }
                           else{
                             $this->setDataSize($size);
                             $this->setDataVolCount(1);
                           }
                        }
                       );

#   $this->addAfterAdvice("DataDriveKey",sub{
#                           my ($this) = @_;
#                           my $drive = $this->getDataDrive();
#                           my $name = $this->getDBName();
#                           my $dataPath = catfile($drive . ":", "sdb",$name,"data");
#                           $this->setDataPath($dataPath);
#                         }
#                        );
#   $this->addAfterAdvice("LogDriveKey",sub{
#                           my ($this) = @_;
#                           my $drive = $this->getLogDrive();
#                           my $name = $this->getDBName();
#                           my $logPath = catfile($drive . ":", "sdb",$name,"log");
#                           $this->setLogPath($logPath);
#                         }
#                        );

  $this->addAfterAdvice("TemplateCodeKey",sub{
                          my ($this) = @_;
                          my $code = $this->getTemplateCode();

                          if( "templ_desktop" eq $code){
                            $this->initDesktop();
                          }
                          elsif( "templ_server_small" eq $code){
                            $this->initServerSmall();
                          }
                          elsif( "templ_server_medium" eq $code){
                            $this->initServerMedium();
                          }
                          elsif( "templ_server_advanced" eq $code){
                            $this->initServerAdvanced();
                          }
                        }
                       );


  #DB defaults for users and passwords
  $this->setDBA("DBADMIN");
  $this->setDBName("MAXDB1");
  $this->setDBAPass("dba");
  $this->setDBM("DBM");
  $this->setDBMPass("dbm");
#  $this->setSQLU("GUEST");
#  $this->setSQLPass("guest");
  $this->setDomainPass("domain");
  $this->setCPUCount(1);
  #set Autoextend and log overwrite on
  $this->setAdvicedAutoExtend("YES");
  $this->setAutoOverWriteLog("YES");
  #DB-Defaults: database parameters
  $this->setDBParams({});
  $this->addDBParam("CACHE_SIZE","2500");
  $this->addDBParam("MAXDATAVOLUMES","64");
  $this->addDBParam("MAXUSERTASKS","20");
  $this->addDBParam("_UNICODE","YES");
  #Funzt noch nicht wegen Kern-Problem (Mit _UNICODE zusammen)
  $this->addDBParam("_SERVERDB_FOR_SAP","NO");
  $this->addDBParam("SESSION_TIMEOUT","0");
  $this->setDBMemory($this->proposeMemory());
  $this->setCPUCount($this->proposeCPU());
}





sub configure{
  my ($this)=@_;
  my @configResult =   $this->SUPER::configure();
  if($configResult[0]){
  }
  return @configResult;
}


sub initDesktop{
  my ($this)=@_;
  my $dbsize = 256 * (1024 / 8);
  my $logsize = $dbsize / 2;
  $this->setDataSize($dbsize);
  $this->setLogSize($logsize);
  $this->setDataVolCount(1);
  $this->setLogVolCount(1);
  $this->setRecoveryMode("Simple");
  $this->setAutoRestart("yes");
  if($^O =~ /mswin/i){
    $this->setAdvicedDataDrive("C:");
    $this->setAdvicedLogDrive("C:");
  }
  $this->setAdvicedParallelConnections(5);
  $this->setLoadSampleSchema(1);
  $this->addDBParam("XP_MAXPAGER","3");
  $this->configure();
}

sub initServerSmall{
  my ($this)=@_;
  $this->setRecoveryMode("Normal");
  my $dbsize = (1024 * 1) * (1024 / 8);
  my $logsize = 800 * (1024 / 8);
  $this->setDataSize($dbsize);
  $this->setLogSize($logsize);
  $this->setDataVolCount(1);
  $this->setLogVolCount(1);
  if($^O =~ /mswin/i){
    $this->setAdvicedDataDrive("C:");
    $this->setAdvicedLogDrive("C:");
  }
  $this->configure();
}

sub initServerMedium{
  my ($this)=@_;
  $this->setRecoveryMode("Normal");
  my $dbsize = (1024 * 1) * (1024 / 8);
  my $logsize = 1000 * (1024 / 8);
  $this->setDataSize($dbsize);
  $this->setLogSize($logsize);
  $this->setDataVolCount(1);
  $this->setLogVolCount(1);
  if($^O =~ /mswin/i){
    $this->setAdvicedDataDrive("C:");
    $this->setAdvicedLogDrive("C:");
  }
  $this->configure();
}


sub initServerAdvanced{
  my ($this)=@_;
  $this->setRecoveryMode("Normal");
  my $dbsize = (1024 * 1) * (1024 / 8);
  my $logsize = 500 * (1024 / 8);
  $this->setDBSize($dbsize);
  $this->setDataSize($dbsize);
  $this->setLogSize($logsize);
  $this->setDBVolumes(undef);
  $this->setAdvicedParallelConnections(20);
  $this->setAdvicedAutoSaveLog(1);
  $this->configure();
}

sub initializeVolumeDefinitions($){
  my ($this ) =  @_;
  my $templateCode = $this->getTemplateCode();
  
  if(defined $templateCode and "templ_server_advanced" eq $templateCode){
  }
  elsif (defined $templateCode){
    $this->SUPER::initializeVolumeDefinitions();
  }
}
#initializeVolumeDefinitions

#Konstruktor fr Simple-Instanzen
#+sub():Simple:Default-Konstruktor
sub new(){
  my $this = createThis();
  bless $this;
  $this->setDefaults();
  $this->setType('simple');
  return $this;
}
#new

1;
# Local Variables: ***
# perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
# sdbinst-defines: "-define SCRIPTING=YES -define PERL5.6.1=YES" ***
# sdbinst-testfile: "TemplateTest.pl" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***



