package  SAPDB::Install::Templates2::test::test;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::Util;
use strict;
our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
  require Getopt::Long;
  import Getopt::Long;
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
  require SDB::Common::Getopt::Long;
  import SDB::Common::Getopt::Long;
}

my @tsts;

my @ALL_KNOWN_UNIT_TESTS =
  qw(
     InstUnit
     PackageDBM
     InstTemplate
     Properties
     DatabaseInstance
     InstanceCustom
     InstanceDesktop
     InstanceSapServer
 );

my @ALL_KNOWN_INST_TESTS =
  $^O =~ /mswin/i ? 
  qw(
     PackageDBM
     DatabaseInstance
     InstanceCustom
     InstanceDesktop
    )
  :
  qw(
     DatabaseInstance
     InstanceCustom
     InstanceDesktop
    )
  ;

sub usage{
  print "USAGE:\n";
  print "<OPTIONS>  <TEST1,TEST2,...>\n";
  print "Options are:\n";
  print "-h               Show this help\n";
  print "-a               Do all tests\n";
  print "-i               Do installation tests, instead of unit tests\n";
  print "-templ <TEMPL>   Template configuration file\n";
  print "-log <FILE>      Output to file, instead of STDOUT\n";
  print "\nPOSSIBLE UNIT TESTS:\n";
  foreach(@ALL_KNOWN_UNIT_TESTS){
    print $_,"\n";
  }
  print "\nPOSSIBLE INSTALLATION TESTS:\n";
  foreach(@ALL_KNOWN_INST_TESTS){
    print $_,"\n";
  }
}


sub main{
  my $retVal;
  my ($configDir,$inst,$all,$help,$log);
  GetOptions('dir=s'=>\$configDir,'b' => undef,
             'i'=>\$inst,
             'a'=>\$all,'h'=>\$help,'log' =>\$log);
  $help and return usage();
  if($configDir){
    -d $configDir or die "Cannot find configuration directory '$configDir'\n";
  }
  else{
    $configDir = '.';
  }
  my @tests = @ARGV;
  $all and $inst and @tests = @ALL_KNOWN_INST_TESTS;
  $all and not $inst and @tests = @ALL_KNOWN_UNIT_TESTS;
  @tests or die "No tests specified\n";
  my @pkgs;
  foreach(@tests){
    push @pkgs, ($inst ? $_.'TestInstallation' : $_.'Test');
  }

  $retVal = starttests(\@pkgs,1,0,$configDir,$log) and die $retVal;
}

sub test{
  my $configFile='';
  GetOptions('config=s'=>\$configFile,'b' => undef);
  my $retVal;
  -f $configFile or die "Cannot find configuration file $configFile\n";
  my $configDir;
  my $fh;
  my ($verbose,$debug_level,$log);
  open($fh,'<',$configFile) or die "Cannot open configuration file: $!\n";
  foreach(<$fh>){
    /^\s*TESTS\s+(.*)\s*$/ and @tsts=split(/,/,$1);
    /^\s*VERBOSE\s+(.*)\s*$/ and $verbose = $1;
    /^\s*DEBUG\s+(.*)\s*$/ and $debug_level= $1;
    /^\s*CONFIGDIR\s+(.*)\s*$/ and $configDir= $1;
    /^\s*TESTLOG\s+(.*)\s*$/ and $log = $1;
  }
  close $fh;
  $retVal = starttests(\@tsts,$verbose,$debug_level,$configDir,$log) and die $retVal;
}
sub starttests{
  my ($atst,$verbose, $dblevel,$configDir,$log) = @_;
  $VERBOSE=$verbose;
  $DEBUG_LEVEL=$dblevel;
  my $fh;
  if($log){
    $|=1;
    open $fh, ">$log" or return "Cannot open log '$log'\n";
    myout($fh,datePrefix() . "Starting tests\n");
  }
  foreach(@$atst){
    my $smth = $_;
    my $pkg = "SAPDB::Install::Templates2::test::$smth";
    my $script = "SAPDB/Install/Templates2/test/$_.pm";
    myout ($fh,datePrefix() . "Loading $script\n");
    require $script;
    my $func = $pkg.'::allTests';
    no strict 'refs';
    my @retVal = &$func(catfile($configDir,$smth.'.config'));
    showErrors($smth,\@retVal,$fh);
  }
  $fh and close $fh;
  return 0;
}



sub myout{
  my ($fh,$txt) = @_;
  $fh or $fh = *STDOUT;
  print $fh($txt);
}

sub showErrors{
  my ($atst,$lines,$fh) = @_;
  if(@$lines){
    myout ($fh,"ERRORS in $atst:\n");
    myout ($fh, join("\n",@$lines),"\n");
  }
  else{
  }
}


1;

# Local Variables: ***
# eval:  (setenv "MYOWN" (sdb-get-current-own)) ***
# eval: (setq my-sdbrun (format "%s\\sys\\src\\install\\perl\\sdbrun" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-script (format "%s\\sys\\wrk\\script" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-src (format "%s\\sys\\src\\install\\perl" (sdb-get-current-own))) ***
# eval: (setq perl-dbg-flags (format "-w -I%s -I%s\\readkey -I%s -I%s" my-sdbrun my-sdbrun my-script my-src )) ***
# End: ***
