#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Untgz.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: 

package SAPDB::Install::Untgz;

use SAPDB::Install::BuildInfo;
use SAPDB::Install::FSUsage;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::StdIO;
use SAPDB::Install::System;
require SAPDB::Install::System::Win32::VersionInfo if ($^O =~ /mswin/i);

  bootstrap SAPDB::Install::Untgz;
sub AUTOLOAD {
  die "cannot load symbol ".$AUTOLOAD."\n"
    unless ($AUTOLOAD =~ /bootstrap$/);
  
  require XSLoader;
  XSLoader::load (__PACKAGE__);
}

##########simulating environment of installer:
#######################################################
my $packageDir = "c:/SapDevelop/DEV/develop/sys/wrk/fast/install";
$packageDir="C:/Documents and Settings/d022195/Local Settings/Temp/maxdb_7.6.00_COR";
$packageDir='C:\Documents and Settings\d022195\Local Settings\Temp\maxdb_7_6_00_27_WIN32';


my $destDir = "c:/sdb/programs";
$destDir = "c:/sapdb/indep_prog";
my $indepProg = "c:/sapdb/indep_prog";
my $package= "SDBBAS.TGZ";
my $testrun=0;
my $testtgz;

sub simulate{
  my $retVal = 0;
  $testtgz = SAPDB::Install::Untgz::new();
  $testtgz->SetReadOnly($testrun);
  my $fullPackage = $packageDir."/".$package;
  $retVal = openTgz($testtgz,$fullPackage) and return $retVal;
  $retVal = setDestDir($testtgz,$destDir) and return $retVal;
  return $retVal;
}

sub openTgz{
  my ($atgz,$fullPackage) = @_;
  $atgz->Open($fullPackage);
  return errorMessages($atgz);
}

sub setDestDir{
  my ($atgz,$destDir) = @_;
  my $retVal=0;
  if ( $atgz->SetDestDir($destDir)!=0 ) {
    $retVal = "Cannot set destination directory to $destDir\n";
  }
  return$retVal;
}

sub unpackit{
  my ($atgz,$adestination,$aIndepProg) = @_;
  my $retVal =0;
  while ( my $next = $atgz->Next()) {
    extractOverwrite($atgz,$adestination,$next,$aIndepProg);
  }
  return $retVal;
}#unpackit

#######################################################

sub debug{
  print shift;;
}
sub deleteOnReboot{
  my (@files) = @_;
  my $retVal=0;
  foreach my $file (@files){
    $file =~ s/\//\\/g;
    my ($lrc,$errtxt) = SAPDB::Install::System::Win32::API::MoveFileAfterReboot($file);	
    unless($lrc == 0){
      $retVal = "Cannot mark \"".$file."\" for deletion after reboot: $errtxt";
      last;
    }
  }	
  return $retVal;
}

sub getTmpDir2{
  my ($drive,$tempDir) = @_;
  my $retVal=0;
  foreach my $dir ('TEMP','TMP'){
    if( -d "$drive/$dir"){
      $$tempDir = "$drive/$dir";
      last;
    }
  }
  unless($$tempDir){
    $$tempDir = $drive.'/TEMP';
    mkdir($$tempDir,0777) or
      return "Cannot create temporary directory '$tempDir'"; 
  }
  return $retVal;
}

sub forceDeleteWin{
  my $retVal=0;
  my $tempDir;
  my ($aIndepProg,@files) = @_;
  my @renamedFiles;
  foreach my $file (@files) {
    $file =~ s/\\/\//g;
    -f $file or $retVal="File \"$file\" not found" and return $retVal;	
    my ($drive,$filename) = ( $file =~ /^(..).*\/([^\/]+)$/) ;
    $filename =~ /\S/ or next;
    $drive =~ tr/[A-Z]/[a-z]/;
    unless($drive =~ /^[a-z]:$/){
      $retVal = "drive not found for $file";
      return $retVal;
    }
    $retVal = getTmpDir2($drive,\$tempDir) and return $retVal;
    my $destination = "$tempDir/$filename.del";
    if (-f $destination) {
      my $count = 0;
      while (-f "$destination$count") {
        $count++;
      }
      $destination = "$destination$count";
    }
    rename($file,$destination) or $retVal= "cannot rename used file \"$file\": $!\n" and return $retVal; 	
    $destination =~ s/\//\\/g; 
    push @renamedFiles,$destination;
  }
  $retVal = deleteOnReboot(@renamedFiles) and return $retVal;
  return $retVal;
}


sub forceDeleteUx{
  my ($aIndepProg,@files) = @_;
  my $retVal=0;
  foreach my $file (@files){
    unless(unlink $file){
      $retVal="Cannot remove '$file'";
      last;
    }
  }
  return $retVal;
}#forceDeleteUx



sub forceDeleteHP{
  my ($aIndepProg,@files) = @_;
  my $rmfile = "$aIndepProg/pgm/rmfile";
  my $retVal=0;
  -x $rmfile or return "$rmfile not found";
  foreach my $file (@files){
    my $rc = callsystem("$rmfile $file",undef,1);
    if ($rc != 0){
      $retVal = "Error deleting $file: $rc";
      last;
    }
  }
  return $retVal;
}




#Kills used files
sub forceDelete{
  my ($aIndepProg,$files) = @_;
  my $retVal=0;
  if($^O =~ /hpux/i){
    $retVal= forceDeleteHP($aIndepProg,@$files);
  }
  elsif($^O =~ /mswin/i){
    $retVal=forceDeleteWin($aIndepProg,@$files);
  }
  else{
    $retVal = forceDeleteUx($aIndepProg,@$files);
  }
  return $retVal;
}







sub doProcState{
  my $ps = SAPDB::Install::ProcState::new();
  foreach my $proc (@{$ps->{'procs'}}) {
    my $cmd = $proc->{'cmd'};
    my $ppid = $proc->{'ppid'};
    my $command = $ps->GetCmd($ppid);
    my $pid = $proc->{'pid'};
    my $uid=   $proc->{'uid'}; 		
    debug ("$cmd $ppid $uid\n");
  }
  my $pids = $ps->WhoUsesModule('dbmcli.exe');
  if (defined $pids) {
    foreach my $pid (@$pids) {
      foreach my $proc (@{$ps->{'procs'}}) {
        if ($proc->{'pid'} == $pid) {
#          debug("Used by pid $pid\n");
        }
      }
    }
  }
}                               #doProcState


sub extractCurrentFile{
  my ($atgz,$aIndepProg) = @_;
  $atgz->ExtractFile();
  return errorMessages($atgz);
}

#sub ExtractFile{
#  die "Scheisse!!!\n";
#  }




sub busyCheck{
  my ($currentFile) = @_;
  my $retVal = 0;
  my $ps = SAPDB::Install::ProcState::new();
  my $pids = $ps->WhoUsesModule($currentFile);
  if (defined $pids) {
    foreach my $pid (@$pids) {
      foreach my $proc (@{$ps->{'procs'}}) {
        if ($proc->{'pid'} == $pid) {
          $retVal .= "Used by pid $pid\n";
        }
      }
    }
  }
  return $retVal;
} 

sub errorMessages{
  my ($atgz) = @_;
  my $retVal=0;
  my @errorMessages =  $atgz->GetErr();
  if (defined $errorMessages[0]) {
    $retVal = join (";", @errorMessages);
    $retVal .= "\n";
  }
  return $retVal;
}



sub renameExtractBusyFile{
  my ($atgz,$currentFile,$previousError,$aIndepProg) = @_;
  my $retVal=0;
  my $busyFiles = busyCheck($currentFile);
  #first case: the file was busy!
  if ($busyFiles) {
#    debug("We found busy file $currentFile\n");
    $retVal = forceDelete($aIndepProg,[$currentFile]) and return $retVal;
    if (not -f $currentFile) {
      $retVal = extractCurrentFile($atgz,$aIndepProg) and return $retVal;
    } else {
      $retVal = "Error moving $currentFile" and return $retVal;
    }
  }
  #second case: the file was not busy, so the unpack
  #error must have another reason, return the unpack error
  else {
#    debug("File was not busy:  $currentFile\n");
    $retVal = $previousError;
  }
  return $retVal;
}



sub extractOverwrite{
  my ($atgz,$adestination,$next,$aIndepProg) = @_;
  my $retVal =0;
  my @s = split(/\s+/,$next);
  my $name = $s[$#s];
  my $currentFile = catfile($adestination,$name);
  if ( canOverwrite($adestination,$next,$aIndepProg)) {
    $retVal = extractCurrentFile($atgz,$aIndepProg);
    if ($retVal) {
      $retVal = renameExtractBusyFile($atgz,$currentFile,$retVal,$aIndepProg) and return $retVal;
      print2stdout("'$currentFile' was busy (Forced extraction)\n");
    } else {
    }
  } 
  else{
    $retVal = extractCurrentFile($atgz,$aIndepProg)and return $retVal;
  }
  return $retVal;
}

sub canOverwrite{
  my ($adestination,$afile,$aIndepProg) = @_;
  my $opt_force = $SAPDB::Install::StartInstall::FORCE_EXTRACT;
  my $regexp = $aIndepProg;
  $regexp =~ s/\\/\//g;
  my $forceRegexp;
  if(defined $opt_force){
    unless("" eq $opt_force){
      $forceRegexp = $opt_force;
    }
  }
  else{
    return 0;
  }
  if($forceRegexp){
    my $fullPath = $adestination .$afile;
    $fullPath =~ s/\\/\//g;
    return $fullPath =~ /$forceRegexp/;
  }
  else{
    return $adestination =~ "^$regexp"; 
  }
#  return $afile =~  /dbmcli\.exe\s*$/i;
#  return 1;
}



sub ExtractFileForce{
  my ($this,$info,$destination,$testrun)= @_;
  my $indepProg = $SAPDB::Install::Values::indep_prog_path;
  if($testrun){
    if ( canOverwrite($destination,$info,$indepProg)) {
      return 0;
    }
    else{
      return $this->ExtractFile;
    }
  }
  else{
    return $this->internalExtractFileForce($info,$destination,$testrun,$indepProg);
  }
}#ExtractFileForce

sub internalExtractFileForce{
  my ($this,$info,$destination,$testrun,$indepProg)= @_;
  my $retVal=0;
  my @s = split(/\s+/,$info);
  my $file = $s[$#s];
  my $path = catfile($destination,$file);
  $retVal = $this->extractOverwrite($destination,$info,$indepProg);
  if($retVal){
    print2stderr("Error extracting '$file': $retVal\n");
  }
  return $retVal;
}#ExtractFileForce

sub testExtractFileForce{
  my $retVal=0;
  my ($this,$adestination,$aIndepProg) = @_;
  while ( my $next = $this->Next()) {
    $this->ExtractFileForce($next,$adestination);
  }
  return $retVal;
}#unpackit


#my $retVal =0;
#$retVal = simulate() and die $retVal;
#$retVal = unpackit($testtgz,$destDir,$indepProg) and die $retVal;
#$retVal = testExtractFileForce($testtgz,$destDir,$indepProg) and die $retVal;



1;

# Local Variables: ***
# eval:   (setq my-own  (sdb-get-current-own)) ***
# eval:   (setq my-sdbrun (format "%s\\sys\\wrk\\fast\\install\\tmp\\sdbrun" my-own)) ***
# eval:   (setq my-src (format "%s\\sys\\src\\install\\perl" my-own)) ***
# eval:   (setq perl-dbg-flags (format "-w -Ic:\\aw\\patch -I%s -I%s\\readkey -I%s" my-sdbrun my-sdbrun  my-src))***
# End: ***
