package SAPDB::Install::Window::Uninstall;

use Wx::Event qw(EVT_SET_FOCUS) ;

use Wx qw(wxVERTICAL wxEXPAND wxTAB_TRAVERSAL) ;

use SAPDB::Install::Resources;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::Registry;
use SAPDB::Install::Log;
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Wizard::Dialog::UninstProgress;
use SAPDB::Install::Wizard::Dialog::UninstallSelect;
use SAPDB::Install::Wizard::Dialog::UninstSummary;
use SAPDB::Install::ObjMngr;
use SAPDB::Install::System;
use SAPDB::Install::Window::Wizard;
use SAPDB::Install::Window::Banner;
use SAPDB::Install::Dialog::About;
use SAPDB::Install::Values;	
use SAPDB::Install::Wizard::Dialog::Finalize;

@ISA = qw(Wx::Panel);

sub new{
	my($this ) = shift->SUPER::new(@_);
	#$this->SetWindowStyleFlag($this->GetWindowStyleFlag | wxTAB_TRAVERSAL);
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $container = SAPDB::Install::ObjMngr->new();
	my ($indep_data,$indep_prog) = readIndepPath;
	unless(-f "$indep_data/config/install/INSTREG"){
		return undef;
	}
	$this->{'banner'} = SAPDB::Install::Window::Banner->new($this,-1,[0,0],[20,80]);
	$this->{'banner'}->setTitle("Installation Manager");
	$this->{'banner'}->setSubTitle(" ");
    $this->{'banner'}->Fit;
	$sizer->Add($this->{'banner'},0,wxEXPAND);
	$container->Log(SAPDB::Install::Log->new("$indep_data/wrk"));
	$container->Log->LogName("MaxDBUninstall");
	$SAPDB::Install::Values::log = $container->Log;
	$container->SetRefLog(\$SAPDB::Install::Values::log);
	$container->InstReg(SAPDB::Install::InstallRegistry->new("$indep_data/config/install",1,$container->Log));
	my $collector = SAPDB::Install::PackageCollector->new();
	$collector->GenRegPackageTree($container->InstReg);
	$collector->log($container->Log);
	$this->{'wizard'} = SAPDB::Install::Window::Wizard->new($this,-1,[0,0],[-1,-1]);
	$sizer->Add($this->{'wizard'},1,wxEXPAND);	
	$this->{'wizard'}->AddStep({'dlg'=> SAPDB::Install::Wizard::Dialog::UninstallSelect->new($this->{'wizard'},-1,[0,0],[-1,-1]),
								'name' => 'Select software somponents to be uninstalled'});
	$this->{'wizard'}->AddStep({'dlg'=> SAPDB::Install::Wizard::Dialog::UninstSummary->new($this->{'wizard'}
									,-1,[0,0],[-1,-1]),'name' => 'Uninstallation Summary'});
	$this->{'wizard'}->AddStep({'dlg'=> SAPDB::Install::Wizard::Dialog::UninstProgress->new($this->{'wizard'}
									,-1,[0,0],[-1,-1]),'name' => 'Uninstallation'});
	my $finalize_dlg = SAPDB::Install::Wizard::Dialog::Finalize->new($this->{'wizard'},-1,[0,0],[-1,-1]);
	$finalize_dlg->{'uninst_mode'} = 1;
	$this->{'wizard'}->AddStep({'dlg'=> $finalize_dlg ,'name' => 'Uninstallation finished'});
	$this->{'wizard'}->SetConfig({'collector' => $collector,'container' => $container});
	EVT_SET_FOCUS($this,sub {$this->{'wizard'}->{'nbutton'}->SetFocus});
	$this->SetSizer($sizer);
	$this->SetAutoLayout(1);
	$this->Layout;
	return $this;
}

sub GetBestSize{
	my ($self) = @_;
	my ($width,$height) = (0,0);
	my $banner_size = $self->{'banner'}->GetBestSize;
	$height += $banner_size->GetHeight;
	$width =  $banner_size->GetWidth;
	my $wizard_size = $self->{'wizard'}->GetBestSize;
	$height += $wizard_size->GetHeight;	
	$width =  $wizard_size->GetWidth if $wizard_size->GetWidth > $width;
	return Wx::Size->new($width,$height);
}

sub init{
	my ($self) = @_;
	$self->{'wizard'}->init;
	$self->Layout;
	return 1;
}



sub Destroy{
	my ($self) = @_;
	$self->{'banner'}->Destroy;
	$self->{'wizard'}->Destroy;
	$self->SUPER::Destroy;

}


sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}
1;