#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/UninstallSelect.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: Advanced/Detailed software package selection
#       
#

package SAPDB::Install::Wizard::Dialog::UninstallSelect;

use Wx::Event qw (EVT_TREE_SEL_CHANGED EVT_TOGGLEBUTTON EVT_BUTTON);
use Wx qw ( 
	wxLEFT wxRIGHT wxYES_NO wxICON_QUESTION wxID_YES
	wxDefaultSize wxHORIZONTAL wxEXPAND wxTOP wxBLUE
	wxALL wxLC_REPORT wxVERTICAL wxALIGN_CENTER_HORIZONTAL
	wxBOTTOM wxTE_MULTILINE wxTE_READONLY wxVSCROLL wxNO_BORDER);

use SAPDB::Install::Control::UninstTreeCtrl;
use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Resources;
use base SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Tools;
use SAPDB::Install::Registry;
use SAPDB::Install::System;
use SAPDB::Install::Dialog::DBDrop;

our $images;

our $tree_desc = 'The software component tree depicts the dependencies between the software components. If you select a component, all dependent components are automatically selected as well.';

sub new{
	my($this ) = shift->SUPER::new(@_);
	unless( defined $images){
		$images = SAPDB::Install::Resources->new('RESOURCES.TGZ')->getImages('icon/cb_checked.png',
															'icon/cb_unchecked.png',
															'icon/package.png',
															'icon/button_select_all.png',
															'icon/button_deselect_all.png',
															'icon/db.png');
	}
	my $sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	my $left_sizer = Wx::BoxSizer->new(wxVERTICAL);
	$this->{'treectrl'} = SAPDB::Install::Control::UninstTreeCtrl->new($this,-1,[0,0],[-1,-1]);
	$this->{'list'} = Wx::ListCtrl->new( $this, -1, [0,0],[-1,-1],
                                wxLC_REPORT);
	$this->{'list'}->InsertColumn( 1, "Parameter" );
	$this->{'list'}->InsertColumn( 2, "Value" );
	$this->{'list'}->Show(0);
	#$sizer->Add($this->{'treectrl'},1,wxEXPAND | wxALL , 10);
	my $text = Wx::StaticText->new($this,-1,'Select or deselect software components',[0,0],[-1,-1]);
	$left_sizer->Add($text,0, wxEXPAND | wxBOTTOM , 10  );
	$left_sizer->Add($this->{'treectrl'},1,wxEXPAND);
	my $sbutton = Wx::BitmapButton->new($this,-1,Wx::Bitmap->new($images->{'icon/button_select_all.png'}));
	my $ubutton = Wx::BitmapButton->new($this,-1,Wx::Bitmap->new($images->{'icon/button_deselect_all.png'}));
	EVT_BUTTON($this,$sbutton->GetId,
		sub{
			my ($self,$evt) = @_;
			$self->{'treectrl'}->CheckChildren($self->{'treectrl'}->GetRootItem);
		}
	);
	EVT_BUTTON($this,$ubutton->GetId,
		sub{
			my ($self,$evt) = @_;
			$self->{'treectrl'}->UncheckAllChildren($self->{'treectrl'}->GetRootItem);
		}
	);

	my $button_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$button_sizer->Add($sbutton,0,wxLEFT | wxRIGHT,20);
	$button_sizer->Add($ubutton,0,wxRIGHT,20);	
	unless(Wx::wxX11){
		$sbutton->SetToolTip('Select All');
		$ubutton->SetToolTip('Deselect All');
	}
	$left_sizer->Add($button_sizer,0,wxTOP, 10);
	
	$sizer->Add($left_sizer,1,wxEXPAND | wxALL , 10);
	$this->{'desc_box'} = Wx::StaticBox->new($this,-1,'Description');
	$this->{'box_sizer'} = Wx::StaticBoxSizer->new($this->{'desc_box'},wxVERTICAL);	
	if($^O =~ /mswin/i){
		$this->{'desc_text'} = Wx::StaticText->new($this,-1,'',[0,0],[-1,-1]);	
	}
	else{
		$this->{'desc_text'} = SAPDB::Install::Control::StaticText->new($this,-1,[0,0],[-1,-1]);
		$this->{'desc_text'}->{'auto_split'} = 1;
		$this->{'desc_text'}->{'auto_resize'} = 0;
		$this->{'desc_text'}->{'auto_vresize'} = 1;
			
	}
	
	$this->{'box_sizer'}->Add($this->{'desc_text'},1,wxEXPAND | wxALL ,10);
	$this->{'desc_sizer'} = Wx::BoxSizer->new(wxVERTICAL);
	$this->{'desc_sizer'}->Add($this->{'box_sizer'},1,wxEXPAND | wxALL,10);
	$this->{'tb'} = Wx::ToggleButton->new($this,-1,'&Show Details',[0,0],[-1,-1]);
	unless(Wx::wxX11){
		$this->{'tb'}->SetToolTip('Show details of selected component');
	}
	EVT_TOGGLEBUTTON($this,$this->{'tb'}->GetId,\&OnDetails);
	$this->{'desc_sizer'}->Add($this->{'tb'},0,(wxALL) & (~wxTOP) | wxALIGN_CENTER_HORIZONTAL, 10);	
	$sizer->Add($this->{'desc_sizer'},1,wxEXPAND);	
	#$sizer->Add($this->{'list'},0,wxEXPAND | wxALL | wxALIGN_CENTER, 10);
	EVT_TREE_SEL_CHANGED($this,$this->{'treectrl'},\&OnItemSelect);
	$this->SetSizer($sizer);
	$this->SetAutoLayout(1);
	$this->{'details_on'} = 0;
	$this->{'item_index'} = {};
	$this->Layout();
	return $this;
}

sub OnDetails{
	my ($self,$evt) = @_;
	if($self->{'details_on'}){
		$self->{'desc_sizer'}->Remove($self->{'list'});
		$self->{'desc_box'}->Show(1);
		$self->{'desc_text'}->Show(1);
		$self->{'list'}->Show(0);
		$self->{'box_sizer'} = Wx::StaticBoxSizer->new($self->{'desc_box'},wxVERTICAL);
		$self->{'box_sizer'}->Add($self->{'desc_text'},1,wxEXPAND | wxALL ,10);
		$self->{'desc_sizer'}->Prepend($self->{'box_sizer'},1,wxEXPAND | wxALL,10);
		$self->{'details_on'} = 0;
	}
	else{
		$self->{'desc_sizer'}->Remove($self->{'box_sizer'});
		delete $self->{'box_sizer'};
		$self->{'desc_box'}->Show(0);
		$self->{'desc_text'}->Show(0);
		$self->{'list'}->Show(1);
		$self->{'desc_sizer'}->Prepend($self->{'list'},1,wxEXPAND | wxALL , 10);
		$self->{'details_on'} = 1;
	}
	$self->Layout;
	return 1;
}

sub AddPackage{
	my ($self,$node) = @_;
	my $save_cur_node = $self->{'cur_node'}; 
	
	my $img_id = 2;
	if($node->{'selected'}){
		$img_id = 1;
	}

	my $disp_name = $node->{'name'};
	
	if($node->{'type'} == 0){
		$disp_name = $self->{'collector'}->package_index->{$node->{'name'}}->{'DISPLAY_NAME'}
			if $self->{'collector'}->package_index->{$node->{'name'}}->{'DISPLAY_NAME'} =~ /\S/;
	}
	else{
		$disp_name = $self->{'collector'}->regpackage_index->{$node->{'name'}}->{$node->{'instkey'}}->{'DISPLAY_NAME'}
			if $self->{'collector'}->regpackage_index->{$node->{'name'}}->{$node->{'instkey'}}->{'DISPLAY_NAME'} =~ /\S/;
	}
	
	$self->{'cur_node'} = $self->{'treectrl'}->AppendItem(
				$self->{'cur_node'},$disp_name,$img_id,$img_id);
	if($node->{'selected'}){
		$self->{'treectrl'}->CheckParent($self->{'cur_node'});
	}
	if(exists $self->{'item_index'}->{$node}){
		$self->{'item_index'}->{$node} = [@{$self->{'item_index'}->{$node}},$disp_name];
	}
	else{
		$self->{'item_index'}->{$node} = [$disp_name];
	} 		
	$self->{'treectrl'}->SetPlData($self->{'cur_node'},$node);
	foreach my $child (sort {$a->{'name'} cmp $b->{'name'}} @{$node->{'children'}}){
		$self->AddPackage($child);
	}
	
	if($node->{'name'} eq 'Database Kernel'){
		if(defined $self->{'config'}->{'collector'}->instance_index){
			my $dbindex = $self->{'config'}->{'collector'}->instance_index;
			my $found = 0;
			foreach my $db (keys(%$dbindex)){
				if(normalizePath($dbindex->{$db}->{'instroot'}) eq normalizePath($node->{'instkey'})){
					my $t_node = $self->{'treectrl'}->AppendItem($self->{'cur_node'},$db,3,3);
					$self->{'treectrl'}->SetPlData($t_node,{'name' => $db,'type' => 2});
					$found = 1;
				}
			}
			$self->{'treectrl'}->Expand($self->{'cur_node'}) if $found;
		}
	}
	
	$self->{'cur_node'} = $save_cur_node; 
	$self->{'treectrl'}->Expand($self->{'cur_node'});
	return 1;
}

sub init{
	my ($self,$config) = @_;
	#return undef;
	$self->{'config'} = $config;
	unless(defined $config->{'collector'}->instance_index){
		$config->{'collector'}->GenInstanceIndex($config->{'container'}->InstReg,1);
	}
	$self->{'image_list'} = Wx::ImageList->new(16,16,1);
	foreach my $key ('icon/package.png','icon/cb_checked.png','icon/cb_unchecked.png','icon/db.png'){
		$images->{$key}->Rescale(16,16);
		$self->{'image_list'}->Add(Wx::Bitmap->new($images->{$key}));
	}
	$self->{'treectrl'}->SetImageList($self->{'image_list'});
	$self->{'collector'} = $config->{'collector'};
	$self->{'cur_node'} =
		$self->{'treectrl'}->AddRoot('MaxDB Components',0,0);
	
	$self->AddPackage($config->{'collector'}->regpackage_tree);
	foreach my $node (keys(%{$self->{'item_index'}})){
		local *ids = $self->{'item_index'}->{$node};
		if($#ids > 0){
			foreach my $idx (@ids){
				foreach my $idy (@ids){
					if(exists $self->{'treectrl'}->{'dependent_nodes'}->{$idx}){
					$self->{'treectrl'}->{'dependent_nodes'}->{$idx} =
						[@{$self->{'treectrl'}->{'dependent_nodes'}->{$idx}},$idy]; 	
					}
					else{
						$self->{'treectrl'}->{'dependent_nodes'}->{$idx} = [$idy];
					}					
				}
			}		
		} 
	}
	$self->{'treectrl'}->SelectItem($self->{'treectrl'}->GetRootItem);
	if($^O !~ /mswin/i){
		$self->{'desc_box'}->SetLabel('Description of Component Tree');
		$self->Fit;
		$self->{'desc_text'}->SetLabel($tree_desc);
		$self->{'box_sizer'}->Remove($self->{'desc_text'});
		$self->{'box_sizer'}->Add($self->{'desc_text'},1,wxEXPAND | wxALL ,10);
	}
	return {}; 
}

sub OnItemSelect{
	my ($self,$event) = @_;
	my @packmappings = (
		{'Name' => 'PACKAGE_NAME'},
		{'Version' => 'SOFTWARE_VERSION'},
		{'Object Mode' => 'MODE'},
		{'Package Type' => 'TYPE'},
		{'Minimal Needed Verison' => 'MIN_VERSION'},
		{'Archive Name' => 'ARCHIVE_NAME'},
		{'Decompressed Size (Bytes)' => 'SIZE'},
		{'Require' => 'REQUIRE'},
		#{'Build String' => 'BUILD_STRING'},
		{'Make ID' => 'MAKE_ID'},
		{'Purpose' => 'PURPOSE'},
		{'QA Level' => 'QA_LEVEL'},		
		{'Make Type' => 'MAKE_TYPE'},
		{'Changelist ID' => 'CHANGELIST'}
	);
	
	my @regpackmappings = (
		{'Name' => 'PACKAGE_NAME'},
		{'Version' => 'SOFTWARE_VERSION'},
		{'Object Mode' => 'MODE'},
		{'Installation Path' => '&instkey'},
		{'Is Valid' => 'VALID'},
		{'Installed Size (Bytes)' => 'SIZE'},
		{'Modification Date' => 'DATE'},
		{'Require' => 'REQUIRE'},
		#{'Build String' => 'BUILD_STRING'},
		{'Make ID' => 'MAKE_ID'},
		{'Purpose' => 'PURPOSE'},
		{'QA Level' => 'QA_LEVEL'},		
		{'Make Type' => 'MAKE_TYPE'},		
		{'Changelist ID' => 'CHANGELIST'}
	);
	
	my @databasemappings = (
		{'Name' => '&name'},
		{'Version' => 'version'},
		{'Software Path' => 'instroot'},
	);
	
	
	
	my $item = $event->GetItem;
	
	if($self->{'treectrl'}->GetItemText($item) eq 'MaxDB Components'){
		$self->{'tb'}->Show(0);
		$self->{'desc_sizer'}->Remove($self->{'tb'});
		if($self->{'details_on'} == 1){
			$self->OnDetails;
		}
		$self->{'desc_box'}->SetLabel('Description of Component Tree');
		$self->{'desc_text'}->SetLabel($tree_desc);
		$self->Layout;
		return 1;
	}
	unless($self->{'tb'}->IsShown){
		$self->{'desc_sizer'}->Add($self->{'tb'},0,(wxALL) & (~wxTOP) | wxALIGN_CENTER_HORIZONTAL, 10);	
		$self->{'tb'}->Show(1);
	}
	if( $self->{'tb'}->GetValue and $self->{'details_on'} == 0 or
	    not $self->{'tb'}->GetValue and $self->{'details_on'} == 1) {
		$self->OnDetails;
	}

	my $data = $self->{'treectrl'}->GetPlData($item);
	

	$self->{'list'}->DeleteAllItems;
	return unless defined $data; 
	my $count = 0;
	
	
	if($data->{'type'} == 2){
		#
		#	database type 
		#
		my $db_data = $self->{'collector'}->instance_index->{$data->{'name'}};
		foreach my $map (@databasemappings){
			foreach my $param_name (keys(%$map)){
				my $item = $self->{'list'}->InsertStringItem($count++,$param_name);
				my $value = $db_data->{$map->{$param_name}};
				if($map->{$param_name} eq '&name'){
					$value = $data->{'name'};
				}
				
				if($map->{$param_name} eq 'instroot' and $^O =~ /mswin/i){
					$value = getRealPathName($value) if ($^O =~ /mswin/i); 				
				}
				$self->{'list'}->SetItem($item,1,$value);
			}
		}
		$self->{'desc_text'}->SetLabel('Database Instance '.$data->{'name'});
		$self->{'desc_box'}->SetLabel('Existing Database');
	}
	else{
		my $package_data;	
		$package_data = $self->{'collector'}->regpackage_index->{$data->{'name'}}->{$data->{'instkey'}};
		foreach my $map (@regpackmappings){
			foreach my $param_name (keys(%$map)){
				my $item = $self->{'list'}->InsertStringItem($count++,$param_name);
				my $value = $package_data->{$map->{$param_name}};
				if($map->{$param_name} eq '&instkey'){
					$value = $data->{'instkey'};
					$value = getRealPathName($value) if ($^O =~ /mswin/i); 
				}
				elsif($param_name eq 'Name'){
					$value = $package_data->{'DISPLAY_NAME'}
						if $package_data->{'DISPLAY_NAME'} =~ /\S/; 
				} 
				elsif($param_name eq 'Object Mode'){
					$value = $value.' bit' if $value =~ /\S/;
				} 
				$self->{'list'}->SetItem($item,1,$value);
			}
		}
		my $name = $package_data->{'DISPLAY_NAME'};
		$name = $package_data->{'PACKAGE_NAME'} unless $name =~ /\S/;
		$self->{'desc_box'}->SetLabel('Description of Component '.$name);
		if (defined $package_data->{'DESC'}){
			$self->{'desc_text'}->SetLabel($package_data->{'DESC'});
		}
		else{
			$self->{'desc_text'}->SetLabel('<no description found>');
		}
	
	}
	$self->{'list'}->SetColumnWidth(0,-1);
	$self->{'list'}->SetColumnWidth(1,-1);
	
	$self->Layout();
	return 1;
}



sub Process{
	my ($self,$config) = @_;

	my $packagelist = $config->{'collector'}->GetRegPackageList;
	
	my @packagelist;
	
	@packagelist = @$packagelist if defined $packagelist;
	
	if($#packagelist == -1){
		$self->SetError("No Component Selected","Please select at least one component!");
		return undef;
	}

	my %instances;
	my @databases;
    if(exists $config->{'collector'}->regpackage_index->{'Database Kernel'}){
		%instances = getInstances();    
		foreach my $dbroot (keys(%{$config->{'collector'}->regpackage_index->{'Database Kernel'}})){
			next unless $config->{'collector'}->regpackage_index->{'Database Kernel'}->{$dbroot}->{'tree_ref'}->{'selected'};
			foreach my $instance (keys(%instances)){
				if(normalizePath($dbroot) eq normalizePath($instances{$instance})){
					push @databases,$instance;
				}
			}				
		}
	} 
	
	if($#databases > -1){
		if (Wx::wxGTK){
			if (exists $self->GetWizard()->{'busy_cursor'}){
				delete $self->GetWizard()->{'busy_cursor'};
			}
		}
		$all_dropped = 1;
		foreach $db (@databases){
			my $msgBox = Wx::MessageDialog->new($self,"Do you want to drop database \"$db\" now?" ,"Found Existing Database", wxYES_NO | wxICON_QUESTION ,[-1,-1]);
			my $dropped = 0;
			if($msgBox->ShowModal() == wxID_YES){
					my $dlg = SAPDB::Install::Dialog::DBDrop->new({'dbname' => $db, 'log' => $config->{'container'}->Log},$self,-1);
					$dropped = $dlg->ShowModal();
					$dlg->Destroy;
			}
			
			unless($dropped){
				my $path = $instances{$db};
				$path = getRealPathName($path) if $^O =~ /mswin/i;
				$self->SetError("Found Existing Database \"$db\"","Cannot remove component Database Kernel in $path");
				$all_dropped = 0;
			}
			$msgBox->Destroy;
		}
		unless($all_dropped){
			return undef;
		}
	}
		
	return {};
}

sub BackInit{
	my ($self,$config) = @_;
	foreach my $package (@{$config->{'packages'}}){
		$SAPDB::Install::PackageObj::SELFOBJ[$package->ID] = undef;
	}
	delete $config->{'packages'};
	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'treectrl'}){
		$self->{'treectrl'}->Destroy;
		delete $self->{'treectrl'};
	}
	if(defined $self->{'list'}){
		$self->{'list'}->Destroy;
		delete $self->{'list'};
	}
	if(defined $self->{'desc_box'}){
		$self->{'desc_box'}->Destroy;
		delete $self->{'desc_box'};
	}
	if(defined $self->{'desc_text'}){
		$self->{'desc_text'}->Destroy;
		delete $self->{'desc_text'};
	}
	if(defined $self->{'tb'}){
		$self->{'tb'}->Destroy;
		delete $self->{'tb'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
