package SAPDB::Install::Templates2::InstUnit;
use SAPDB::Install::Tools;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Values;
use SAPDB::Install::MD5Sum;
use SAPDB::Install::Trace;
use SAPDB::Install::Registry;
use SAPDB::Install::System;
use strict;

our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}
push  @ISA, qw( SAPDB::Install::Templates2::Properties);
our @EXPORT = qw (createObjectFromTGZ);



sub readAFileInTGZ{
  my($tgzhandle,$readfile)=@_;
  my $returnvalue;
  my $info = "start";
  $tgzhandle->Rewind(); 
  my $retVal = 0;
  my $file=undef;
  while ($info) {
    $info = $tgzhandle->Next();
    $retVal = $tgzhandle->GetErr() and return ($retVal,undef);
    unless($info){
      next;
    }
    $info =~ /(\S+)$/;
    my $aFileName = $1;
    unless($aFileName){
      next;
    }
    if ($aFileName eq $readfile) {
      $retVal =  $tgzhandle->ExtractScalar($file) and return ($retVal,undef);
      last;
    }
  }
  $file or $retVal = "Could not find $readfile\n";
  return ($retVal,$file);
}

sub createObjectFromTGZ{
  my ($untgz,$instConfig) = @_;
  my ($retVal,$rawData); 
  ($retVal,$rawData) = readAFileInTGZ($untgz,'PACKAGEDATA');
  $retVal and return ($retVal,undef);
  my %data = %SAPDB::Install::Values::package_interface;
  $rawData or return ("No package data found!",undef);
  my @filedata=split("\n",$rawData);
  @filedata or return ("Package data file is empty!",undef);
  foreach my $line (@filedata){
    foreach my $key (keys(%data)){
      if($line =~ /^\s*$key\s*=\s*"(.*)"\s*$/){
        $data{$key} = $1;
        last;
      }
    }
  }
  keys %data or return ("Package data file contains no valid data!",undef);
  $data{'UNTGZ'}=$untgz;
  my $newpackage = SAPDB::Install::Templates2::InstUnit::new(\%data,$instConfig);
  return ($retVal,$newpackage);
}

sub createObjectFromRegistry{
  my ($reg,$instConfig) = @_;
  my %data  ;
  my ($retVal);
  $data{'PACKAGE_NAME'}=$reg->Name;
  $data{'SOFTWARE_VERSION'}=$reg->Version;
  $data{'DESC'}=$reg->Desc;
  my $packobj = SAPDB::Install::Templates2::InstUnit::new(\%data,$instConfig);
  $packobj->setMainPath($reg->Path);
  $packobj->setFileList($reg->FileList);
  return ($retVal,$packobj);
}

# delete a directory tree
sub delTree{
  my ($this,$dir) = @_;
  -d $dir || return 0;
  opendir(DH,$dir) || return 0;
  my @content = readdir(DH);
  closedir(DH);
  my @files = grep { -f "$dir/$_" } @content;
  my @dirs = grep { -d "$dir/$_" && ! /^\.{1,2}$/} @content;
  unless($^O =~ /mswin/i){
    if ($#files > -1 || $#dirs > -1) {
      my @statbuf = stat($dir);
      my $mask;
      if ($> == $statbuf[4]) {
        $mask = 0200;
      } else {
        my $found;
        foreach my $gid (split(' ',$) )) {
          if ($gid == $statbuf[5]) {
            $found = 1;
            last;
          } 
        }
        if ($found) {
          $mask = 020;
        } else {
          $mask = 002;
        }
      }
      if (($statbuf[2] & $mask) == 0) {
        chmod($statbuf[2] | $mask,$dir) || $this->warnit("cannot enable write permissions for $dir\n");
      }
    }
  }

  foreach (@dirs) {
    -l "$dir/$_" and next;
    delTree("$dir/$_") || return 0;
  }
  foreach (@files) {
    -l "$dir/$_" and next;
    -f "$dir/$_" and (unlink("$dir/$_") or $this->warnit("cannot delete $dir/$_\n") and return 0);
  }
  rmdir($dir) or $this->warnit("cannot delete $dir\n") and return 0;
  return 1;
}



sub testaccess{
  my ($this,$node,$username,$access) = @_;
  $^O =~ /mswin/i and return 0;
  -e $node or return "Access failed: no such file or directory: $node";
  my @passwd_buf = getpwnam($username);
  my $uid = $passwd_buf[2];
  defined $uid or return "unknown user $username"; 
  $access =~ /^[rwx]{1,3}$/i or return "wrong access mode format '$access'";  
  my @statbuf = stat ($node) or return "Cannot stat $node: $!"; 
  my $mode = $statbuf[2];
  if ($statbuf[4] == $uid) { # user is owner of node:check owner permissions
    if ($access =~ /r/i) {
      ($mode & 0400) == 0 and return 'owner has no r bit'; 
    }
    if ($access =~ /w/i) {
      ($mode & 0200) == 0 and return 'owner has no w bit';
    }
    if ($access =~ /x/i) {
      ($mode & 0100) == 0 and return 'owner has no x bit';
    } 
    return 0;
  }
  # try to find username in group of node 
  my $found = 0;
  my $prim_gid == $statbuf[5] and $found = 1; # group is primary group of user 
  foreach my $member (split(' ',(getgrgid($statbuf[5]))[3])) {
    $found and last;            # group is primary group of user
    $member eq $username and $found = 1 and last; 
  } 
  if($found){
    if ($access =~ /r/i) {
      ($mode & 040) == 0 and return 'group has no r bit';
    }
    if ($access =~ /w/i) {
      ($mode & 020) == 0 and return 'group has no w bit' ;
    }
    if ($access =~ /x/i) {
      ($mode & 010) == 0 and return 'group has no x bit' ;
    } 
    return 0;
  } else { # user is not owner and not member of group: check permissions for others
    if ($access =~ /r/i) {
      ($mode & 04) == 0 and return 'others have no r bit';
    }
    if ($access =~ /w/i) {
      ($mode & 02) == 0 and return 'others have no w bit'; 
    }
    if ($access =~ /x/i) {
      ($mode & 01) == 0 and return 'others have no x bit'; 
    } 
    return 0;
  }
  return "Not found $node"; 
}


sub crelease2num{
  my $this = shift;
  $_=shift; 
  my $cor_digit_num = 5;
  unless(/^(\d)+\.(\d+)\.(\d+)\.(\d+)$/ || /^(\d)\.(\d)([A-Z|\d]+)\.(\d+)$/){
    return 0;
  } 
  my ($major, $minor,$cor_lev,$pat_lev) = ($1,$2,$3,$4);
  if ($cor_lev =~ /\D/) {
    $cor_lev = ord($cor_lev) - ord('A');
  }
  if ($cor_digit_num < length($cor_lev)) {
    return 0;
  }
  length($minor)==1 and $minor="0$minor";
  $cor_lev = ('0' x ($cor_digit_num - length($cor_lev))) . $cor_lev;
  if (length($pat_lev) == 1) {
    $pat_lev="00$pat_lev";
  } elsif (length($pat_lev) == 2) {
    $pat_lev="0$pat_lev";
  } elsif (length($pat_lev) > 3) {
    return 0;
  }
  my $num="$major$minor$cor_lev$pat_lev";
  return $num;
}




#XSERVER CALLS:
my $x_server_results = {
               '19710' => "INFO: Service 'XServer' removed successfully" , 
               '19709' => "INFO: Service 'XServer' installed successfully" , 
               '19712' => "INFO: Service 'XServer' is already installed" , 
               '18640' => "ERROR: XSERVER not active" , 
               '19720' => "INFO: XSERVER started" , 
               '18641' => "ERROR: XSERVER is already running!" ,
               '18581' => "ERROR: Service 'XServer' not installed!",
               '11779' =>  "NISERVER Error during startup:'NI: listen failed,Service in use"
               };
sub call_x_server{
  my ($this,$arg,$retVals) = @_;
  my $retVal=0;
  my $exec  = catfile($this->getIndepProg(), "bin","x_server") ;
  my $command = $exec . " $arg";
  my $instConfig = $this->getInstTemplate();
  $this->logit("[".$command."]");
  callSystem($command,
             sub{
               my $line = $_[0];
               $this->logit($line);
               if ( $line =~ /(^\d+)\s.*$/
                    or $line =~ /^ERR\s+(\d+)/
                  ) {
                 my $id = $1;
                 my $error = 1;
                 foreach (@$retVals) {
                   if ($id eq $_ ) {
                     $error = 0;
                   }
                 }
                 if ($error) {
                   $retVal = $line;
                 }
               }
               return 1;
             });
  return $retVal;
}

#DBMCLI CALLS:

sub call_dbmcli{
  my ($this,$command,$result) = @_;
  $command = $this->GetDbmcliExe() . $command;
  my $retVal=0;
  my $error=1;
  my $output="";
  my $instConfig = $this->getInstTemplate();
  my $displayCommand = $this->removePasswordFromCommand($command);
  $this->logit("[".$displayCommand."]");
  callSystem($command, sub{
               my $line = $_[0];
               unless(defined $line){
                 return ;
               }
               $output .= $line;
               $result and push @$result,($line);
               if ( $line =~ /OK/) {
                 $error = 0;
               }
               return 1;
             }
            );
  if ($error) {
    $retVal = "ERROR in '$displayCommand'\n" . $output;
  }
  return $retVal;
}


sub call_dbmcli_script{
  my ($this,$script,$user,$db) = @_;
  my $retVal = 0;
  $user = $user ? "-u $user" : '';
  $db = $db ? "-d $db" : '';
  my $tmpDir = $ENV{'TMP'} ? $ENV{'TMP'} : $ENV{'TEMP'};
  -d $tmpDir or return "Cannot write to temporary directory $tmpDir";
  my $tmpFile = catfile($tmpDir,"tmpscript.dbm");
  open TMPSCRIPT , ">$tmpFile" or return "Cannot open $tmpFile: $!";
  print TMPSCRIPT $script;
  close TMPSCRIPT;
  my $cmd = $this->GetDbmcliExe() . " $user $db -i \"$tmpFile\"";
  my $displayCommand = $this->removePasswordFromCommand($cmd);
  $this->logit("[".$displayCommand."]");
  $this->logit("\nBEGIN SCRIPTCONTENTS\n${script}END SCRIPTCONTENTS");
  #The first error of the script is reported:
  my ($scriptError,$errorCode,$output) = (0,0,'');

  $retVal = callSystem($cmd, sub{
               my ($line) = @_;
               $line or return;
               $output .= $line;
               $line =~ /^\s*-?\d+,ERR/ and  not $errorCode and $errorCode = $line;
               $line =~ /^\s*ERR/ and  $scriptError = 1;
             }
            ); 
  $scriptError and $this->logit("\nBEGIN SCRIPTOUTPUT\n${output}END SCRIPTOUTPUT");
  $errorCode and return $errorCode;
  $scriptError and return "Unknown error in script occured.";
  return $retVal;
}

sub GetDbmcliExe{
  my $this = shift;
  my $dbmcli ; 
  if($^O =~ /mswin/i){
    $dbmcli = catfile($this->getIndepProg(),"pgm","dbmcli");
  }
  else{
    $dbmcli = catfile($this->getIndepProg(),"bin","dbmcli");
  }
  return $dbmcli;
}



#DBMCMD CALLS:

sub dbmcmd{
  my ($this,$dbm,$cmd,$result) = @_;
  my $displayCommand = $this->removePasswordFromCommand($cmd);
  $this->logit("[".$displayCommand."]");
#  print("[$cmd]\n");
  my $output = $dbm->exec($cmd);
  defined $result and $$result = $output;
  return $this->handleDBMOutput($dbm,$output);
}


sub async_dbmcmd{
  my ($this, $dbm,$cmd,$waitTime) = @_;
  my $displayCommand = $this->removePasswordFromCommand($cmd);
  $this->logit("[".$displayCommand."]");
  defined $waitTime or $waitTime = 1;
  my $fatalError = 0;
  my $go=1;
  my $output  = undef;
  my $id = $dbm->send ($cmd);
  my $startTime = time();
  while ($go) {
    #Get 0 if job still running, or the id of the job
    my $sel = $dbm->select($waitTime);
    my $newTime = time()-$startTime;
    unless(defined $sel){#select failed for an unknown reason
      $go=0;
      $fatalError = "Fatal error in async_dbm of  [$displayCommand]\n";
      last;
    }
    if ($sel == $id ) {#Job has finished: Handle the output and break the loop.
      $go=0;
      $output  = $dbm->recv ($id);
      $this->async_status($newTime,".[FINISHED]\n");
      return $this->handleDBMOutput($dbm,$output);
      last;
    }
    elsif ($sel == 0) {#Job still running: Print status and loop again
      $this->async_status($newTime,".");
    }
  }
  return $fatalError;
}                               #asyncDbmcliHandler

sub handleDBMOutput{
  my ($this,$dbm,$output) = @_;
  if($output){
    my $result =  (split('\n', $output))[0];
    $result =~ /^OK/ and return 0;
    #OK not found means error, Return the output:
    return $output;
  }
  else{
    #No output means error, return last dbm command:
    return $this->lastDBMError($dbm);
  }
}

sub lastDBMError{
  my ($this,$dbm) = @_;
    my $lastmsg = $dbm->lastmsg();
    my $lasterr = $dbm->lasterr();
    my $errorReturn = "$lastmsg\n$lasterr\n";
    if($errorReturn =~ /^\s*$/){
      $errorReturn = "NO ERROR MESSAGE AVAILABLE";
    }
    return $errorReturn;
}




#our $keys->{'MappedFiles'} = {} if ($^O =~ /mswin/i);

sub GetUID{
  $^O =~ /mswin/i and return 0;
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $uid = getpwnam($instConfig->getOwnerName());
  return $uid;
}
sub GetGRID{
  $^O =~ /mswin/i and return 0;
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $grid = getgrnam($instConfig->getOwnerGroup());
  return $grid;
}

sub GetRelease{
  my $this = shift;
  my ($release) = ($this->getVersion() =~ /(\d+\.\d+)/); 
  return $release;
}

sub fireMessageEvent{
  my ($this,$type)=  (shift,shift);
  my @margs = @_;
  my $lt; 
  $lt = $this->internalGetListeners($type) or return 1;
  foreach(@$lt){ &$_(@margs);}
  return 0;
}
sub warnit{
  my ($this,$message)  = @_;
  return  $this->fireMessageEvent('warning',fixmsg($message)); 
}
sub logit{
  my ($this,$message)  = @_;
  return  $this->fireMessageEvent('log',fixmsg($message)); 
}

sub async_status{
  my ($this,$time,$message)  = @_;
  return  $this->fireMessageEvent('async_status',$time,$message); 
}
sub status_callback{
  my ($this,$lst)  = @_;
  return  $this->fireMessageEvent('status_callback',$lst); 
}

sub debugit{
  my ($this,$level,$message)  = @_;
  return  $this->fireMessageEvent('debug',$level,fixmsg($message)); 
}

sub fixmsg{
  my $msg = shift;
  $SCRIPTING and return $msg;
  $msg =~ s/(\s+)$//;
  $msg .= "\n";
  return $msg;
}


sub internalGetListeners{
  my ($this,$type) = @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->GetListeners($type);
}


sub SetInstTemplate{
  my ($this,$value) = @_;
  return $this->setInstTemplate($value);
}



sub getGroup{
  my $this =  shift;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->getOwnerGroup();
}
sub setGroup{
  my ($this,$val)  =  @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->setOwnerGroup($val);
}



sub getUser{
  my $this =  shift;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->getOwnerName();
}
sub setUser{
  my ($this,$val)  =  @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->setOwnerName($val);
}



sub getSystemProgPath{
  my $this =  shift;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->getSystemProgPath();
}
sub setSystemProgPath{
  my ($this,$val)  =  @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->setSystemProgPath($val);
}

sub getIndepProg{
  my $this =  shift;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->getIndepProg();
}
sub setIndepProg{
  my ($this,$val)  =  @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->setIndepProg($val);
}

sub getIndepData{
  my $this =  shift;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->getIndepData();
}
sub setIndepData{
  my ($this,$val)  =  @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->setIndepData($val);
}

	
sub initData{
  my ($this,$rhData)=@_;
  $this->SUPER::initData();
  my %data = %$rhData;
  $this->setInterfaceVersion($data{'INTERFACE_VERSION'});
  $this->setName($data{'PACKAGE_NAME'});
  $this->setVersion($data{'SOFTWARE_VERSION'});
  $this->setPackageVersion($data{'PACKAGE_VERSION'});
  $this->setMode($data{'MODE'});
  $this->setType($data{'TYPE'});
  if($data{'PART_OF'}){
    my @profiles=split(',',$data{'PART_OF'});
    $this->setProfiles(\@profiles);
  }
  if ($data{'REQUIRE'}) {
    my @require=split(',',$data{'REQUIRE'});
    $this->setRequire(\@require);
  } else {
    $this->setRequire([]);
  }
  $this->setArchiveName($data{'ARCHIVE_NAME'});
  $this->setArchiveDir($data{'ARCHIVE_DIR'});
  $this->setMinVersion($data{'MIN_VERSION'});
  $this->setAllowSkip($data{'ALLOW_SKIP'});
  $this->setTopOf($data{'IS_TOP_OF'});
  $this->setIsSubPackage($data{'IS_SUBPACKAGE'});
  $this->setSubDir($data{'SUB_DIR'});
  $this->setMagicMode($data{'MAGIC_MODE'});
  $this->setDesc($data{'DESC'});
  $this->setSize($data{'SIZE'});
  $this->setBuildString($data{'BUILD_STRING'});
  if ($data{'BUILD_STRING'}) {
    my ($make_state,$qa_level,$purpose,$change) = ($data{'BUILD_STRING'} =~ /-(\d)(\d)(\d)-(\d{3}-\d{3})$/);
    $change =~ s/-//g;
    $change =~ s/^0*//;
    $this->setChangeList($change);
    if ($make_state == 1) {
      $make_state = 'productive';
    } else {
      $make_state = 'test/development';
    }
    $this->setMakeState($make_state);
    if ($qa_level == 0) {
      $qa_level = 'DEV';
    } elsif ($qa_level == 1) {
      $qa_level = 'COR';
    } elsif ($qa_level == 2) {
      $qa_level = 'RAMP';
    } elsif ($qa_level == 3) {
      $qa_level = 'HOT';
    }
    $this->setQALevel($qa_level);
    my @purpose;
    if ($purpose == 0) {
      push @purpose,'no classification';
    } else {
      if (($purpose & 1) != 0) {
        push @purpose,'OLTP';
      }
      if (($purpose & 2) != 0) {
        push @purpose,'liveCache';
      }		
    }
    $this->setPurpose(join(' & ',@purpose));
  }
	
	
  $this->setMakeId($data{'MAKE_ID'});
  unless($^O=~/mswin/i){
    $data{'DEFAULT_USER'} and $this->setUser($data{'DEFAULT_USER'});
    $data{'DEFAULT_GROUP'} and $this->setGroup($data{'DEFAULT_GROUP'});
  }
  $this->setUntgz($data{'UNTGZ'});

  if ($data{'TEST_FILE'}) {
    $this->setTestFile($data{'TEST_FILE'});
    $this->setMagicString($data{'MAGIC'});
  }
  if (defined $data{'DISPLAY_NAME'}) {
    $this->setDispName($data{'DISPLAY_NAME'});
  } else {
    $this->setDispName($data{'PACKAGE_NAME'});
  }

  my $scriptName = $data{'SCRIPT'};
  if ($scriptName) {
    my $text = readFileInTGZ($data{'UNTGZ'},$scriptName);
    $this->setScript(\$text);
  }
	
  my $archiveName = $this->getArchiveName();
  my $archiveDir = $this->getArchiveDir();
  if($archiveDir and $archiveName){
    my $archive = catfile($archiveDir,$archiveName);
    my $md5sum = MD5Sum($archive);
    if (defined $md5sum && $md5sum =~ /^[0-9a-f]{32}$/) {
      $this->setCheckSum($md5sum);
    }
    my %md5sums;
    my $fileListName = $data{'FILELIST'};
    if ($fileListName) {
      $this->debugit(4,"Reading file list $fileListName");
      my ($retVal,$filelist) = readAFileInTGZ($data{'UNTGZ'},$fileListName);
      $retVal and $this->logit("Error reading $archiveName: $retVal");
      
      foreach my $file (split("\n",$filelist)) {
        if( $file=~/^\s*"(\S.*\S)"\s*([0-9a-z]{32})\s*$/){
          $md5sums{$1}=$2;  
        }
        elsif($file =~/^\s*#/){
        }
        elsif($file =~/\S/){
          $this->debugit(1,"Invalid entry in file list: $file");
        }
      }
    }
    $this->setFileList(\%md5sums);
  }
  $this->setPaths([]);
  $this->setOpts([]);
}



sub preinstall {return 0;}
sub preuninstall {return 0;}
sub preunpack {return 0;}
sub postprepare {return 0;}
sub postinstall {return 0;}
sub postuninstall {return 0;}
sub register {return 0;}
sub unregister {return 0;}
sub framestart {return 0;}
sub framestop {return 0;}


sub uninstall{
  my $this = shift;
  my @undeleted;
  my $regmd5sums = $this->getFileList();
  foreach my $file (keys(%$regmd5sums)) {
    my $fullpath = catfile($this->getMainPath(),$file);
    -f $fullpath or $this->warnit("Cannot delete file \"$fullpath\": no such file\n") and next; 
    if ($regmd5sums->{$file} eq MD5Sum($fullpath)){
      if(unlink($fullpath)){
      } else {
        $this->warnit("Not deleted: \"$fullpath\": $!\n");
      }
    } else {
      $this->warnit("Modified:      \"$fullpath\"\n");
    }
  }
}

my %packageClasses =
  (
   '^Base$' => "SAPDB::Install::Templates2::PackageBase",
   '^Database Manager$' => "SAPDB::Install::Templates2::PackageDBM",
   '^Server Utilities$' => "SAPDB::Install::Templates2::PackageServerUtil",
   '^Database Kernel$' => "SAPDB::Install::Templates2::PackageDBKernel",
   '^SQL Studio$' => "SAPDB::Install::Templates2::PackageSQLS",
   '^CPC Base$' => "SAPDB::Install::Templates2::PackageCPCBase",
   '^DB Analyzer$' => "SAPDB::Install::Templates2::PackageDBAnalyzer",
   '^Database Connectivity$' => "SAPDB::Install::Templates2::PackageDBConnect",
   '^JDBC$' => "SAPDB::Install::Templates2::PackageJDBC",
   '^LC SDK$' => "SAPDB::Install::Templates2::PackageLCSDK",
   '^PCR \d\d\d\d$' => "SAPDB::Install::Templates2::PackagePCR",
   '^CPC SDK \d\d\d\d$' => "SAPDB::Install::Templates2::PackageCPC",
   '^DB Analyzer$' => "SAPDB::Install::Templates2::PackageDBAna",
   '^Redist Python$' => "SAPDB::Install::Templates2::PackagePython",
   '^Webtools$' => "SAPDB::Install::Templates2::PackageWebtools",
   '^XML Indexing Engine$' => "SAPDB::Install::Templates2::PackageXML",
   '^APO COM$' => "SAPDB::Install::Templates2::PackageAPO"
  );


sub classForName{
  my ($name)  = @_;
  my $retVal;
  foreach my $key (keys %packageClasses) {
    if ($name =~ /$key/) {
      $retVal = $packageClasses{$key};
      last;
    }
  }
  return $retVal;
}



sub new{
  my ($data,$instConfig) = @_;
  my $keys={
            'iD' => {},
            'interfaceVersion' => {},
            'name' => {},       # unique package type identifier
            'version' => {},
            'packageVersion' => {},
            'mode' => {},
            'type' => {},
            'minVersion' => {},
            'profiles' => {},
            'topOf' => {},
            'require' => {},
            'required' => {},
            'fileList' => {},
            'script' => {},
            'testFile' => {},
            'magicMode' => {},
            'allowSkip' => {},
            'paths' => {},
            'mainPath' => { 'DEFAULT' => '${indepProg}'},
            'archiveName' => {},
            'archiveDir' => {},
            'opts' => {},
            'regData'=> {},
            'skip' => {},
            'user' => {},
            'uID' => {},
            'group' => {},
            'gID' => {},
            'magicString' => {},
            'untgz' => {},
            'isSubPackage' => {}, 
            'subDir' => {},     
            'subPackages' => {}, 
            'parentPackage' => {}, 
            'flushIntoReg' => {}, 
            'checkSum' => {},   
            'withParent' => {}, 
            'subPackObjs' => {}, 
            'dispName' => {},   
            'ignoreBusyFiles' => {}, 
            'size' => {},
            'buildString' => {},
            'makeId' => {},
            'qALevel' => {},
            'purpose' => {},
            'changeList' => {},
            'makeState' => {},
            'indepData' => {},
            'indepProg' => {},
            'systemProgPath' => {},
            'desc' => {},
            'fSUsage' => {},
            'instTemplate' => {},
            'date' => {}
           };
  my $this = {'KEYS' => $keys};
  my $aName =  $data->{'PACKAGE_NAME'}; 
  my $aClass = classForName($aName);
  $aClass or $aClass =  $data->{'CLASS'}; 
  if ($aClass) {
    bless $this,$aClass;
  }
  else {
    bless $this;
  }
#  print "$this\n";
  $instConfig and $this->SetInstTemplate($instConfig);
  $this->initData($data);
  return  $this;
}



sub prepare{
  my ($this) = @_;
  my $result = $this->preinstall(); 
  if($result){return $result;}
  $result  = $this->preprepare();
  if($result){return $result;}
  $result = $this->postprepare();
  return $result;
}

sub preprepare{
  my $this = shift;
  my $result = $this->handleTestExtraction();
  unless($result) {
    my $sp = $this->handleSpaceCheck();
    if ($sp) {
      $result =  $sp;
    }
  }
  return $result;
}

sub handleTestExtraction{
  my $this = shift;
  my $retVal=0;
  my $mainPath= $this->getMainPath();
  my $mk = $this->pmakedir($mainPath,0555);
  if ($mk) {
    $retVal =    "\nError creating directory '$mainPath': $mk\n"; 
  } else {
    my $testResult = $this->unpackIt(1);
    if ($testResult) {
      $retVal = "Error in test extraction:\n$testResult\n";
    }
  }
  return $retVal;
}


sub handleSpaceCheck{
  my $this = shift;
  my $fsusage =  $this->getFSUsage();
  my $retVal = 0;
  my %fsall;
  foreach my $fs (@$fsusage) {
    my $name = $fs->{'FSNAME'};
    my $afs = $fsall{$name};
    unless(defined $afs){
      $afs = { 'AVAIL' => 0, 'NEEDED' => 0, 'FSNAME' => $name};
      $fsall{$name}=$afs;
    }
    my $avail = $fs->{'AVAIL'};
    my $needed = $fs->{'NEEDED'};
    my $allNeeded =  $afs->{'NEEDED'}+$needed;
    $afs->{'NEEDED'} = $allNeeded;
    $afs->{'AVAIL'} = $avail;
  }
  foreach my $fs (values %fsall) {
    my $avail = $fs->{'AVAIL'};
    my $needed = $fs->{'NEEDED'};
    my $name = $fs->{'FSNAME'};
    if ($needed > $avail) {
      $retVal =  "Not enough space on $name  available $avail, needed $needed\n";
    } else {
    }
  }
  return $retVal;
}




sub displayString{
  my $this = shift;
  my $aName = $this->getDispName ? $this->getDispName : $this->getName;
#  my $aVersion = $this->getVersion ? $this->getVersion : "";
  my $tmpstring=$aName;
  my $aPath = $this->getMainPath();
  if($this->getMode){
    $tmpstring .= ' ('.$this->getMode.' BIT)' if $this->getMode =~ /32|64/;
  }
#  $tmpstring .= " $aPath";
  return  $tmpstring;
}





sub GetAbsoluteFileList{
  my ($this)=@_;
  my %fileList = %{$this->getFileList}; # md5sums of files in tgz archive
  my $mainPath =  $this->getMainPath();
  my @retVal;
  foreach(keys %fileList){
    my $aPath = normalizeAPath(catfile($mainPath,$_));
    push @retVal,($aPath);
  }
  return @retVal;
}

sub backupFile{
  my $file = shift;
  my $backupFile = $file;
  my $counter = 1;
  while(-f $backupFile){
    $backupFile = $file . ".bak." . $counter;
    $counter++;
  }
#  rename ($file,$backupFile);
  my $result = copyFile($file,$backupFile);
  return $result;
}



  # $tgzname: full parth to archive
  # $destdir: destination directory
  # $testrun: if true no files will be extracted, only try to open for writing
  # @list: empty -> all files of archive will be extracted; set -> only files in @list will be extracted
sub unpackIt{
  #sub extract archive
  my ($this,$testrun)=@_;
  my @list = keys (%{$this->getFileList()});
  my $listCount = @list;
  my $name = $this->getName();
  $this->debugit(1,"Start extracting files for $name ($listCount files).");
  my $destdir = $this->getMainPath();
  my $cwd;
  my $tgzhandle = $this->getUntgz;
  my $action="";
  $tgzhandle->SetOwner($this->getUser,$this->getGroup) unless($^O =~ /mswin/i);
  $tgzhandle->Rewind();
  if ($testrun) {
    $tgzhandle->SetReadOnly(1);
    $action = "Testextracting";
  } else {
    $tgzhandle->SetReadOnly(0);
    $action = "Extracting";
  }
  unless(defined $destdir and -d $destdir){
    my $displ = defined $destdir ? $destdir : "<UNDEFINED>";
    return "Destination Directory \"$displ\" not found";
  }
  unless($tgzhandle->SetDestDir($destdir)==0){
    return "Cannot set destination directory \"$destdir\": ".join(': ',$tgzhandle->GetErr);
  }
  $this->debugit(1,"Destination directory for unzipping is '$destdir'\n");
  my $info = "start"; 
  while ($info) {
    $info = $tgzhandle->Next();
    unless ($info) {
      last;
    }
    $_=$info;
    if (/^d/) {
      $tgzhandle->ExtractFile;
      next;
    }
    /([^\ ]*)$/	and my $file=$1;
    $file=~s/^\.\///;           #remove "./" at begin of filename
    my ($rc,$errtext);
    if (@list) {
      foreach my $extfile (@list) { 
        if ($extfile eq $file) {
          $tgzhandle->ExtractFile;
          $this->debugit(4,"$action '$file'\n");
        }
      }
    } else {
      $tgzhandle->ExtractFile($testrun);	
      $this->debugit(1,"$action '$file'\n");
    }
    if ($tgzhandle->GetErr) {
      return "Extraction error: ".join(': ',$tgzhandle->GetErr);
    }
  }
  my @fsusage = $tgzhandle->GetFSUsage; 
  $this->setFSUsage(\@fsusage); 
  return 0;
}

sub install{
  my $this = shift;
  my $result = $this->doUpdate();
  return $result;
}
sub doUpdate{
  my ($this,$oldPackage)  = @_;
  my $result = 0;
  if($oldPackage){
    $result = $oldPackage->unregister();
    if ($result) {$this->warnit("Unregister failed:\n" . $result);}
  }
  $result = $this->update($oldPackage) and return "Error in update: \n" . $result;
  $result =   $this->postinstall() and return "Error in postinstall:\n" .$this->getDispName . "\n". $result;
  $result =   $this->register() and return "Error in register:\n".$result;
  return $result;
}
sub update{
  my ($this,$oldPackage)=@_;
  my $result = 0;
  my $oldFileList = {};
  if($oldPackage){
    $oldFileList = $oldPackage->getFileList();
  }
  my $newFileList = $this->getFileList; 
  my $mainPath = $this->getMainPath();
  #Backup changed files:
  foreach my $aKey (keys %$oldFileList) {
    my $aFile = catfile($mainPath,$aKey);
    if (-f $aFile) {
      my $newSum=MD5Sum($aFile);
      my $fileInfo = $oldFileList->{$aKey};
      my $oldSum ; 
      if(ref $fileInfo eq 'HASH'){
        $oldSum = $fileInfo->[0];
      }
      else{
        $oldSum = $fileInfo;
      }
      if ( $oldSum ne $newSum) {
        $result = backupFile($aFile);
        if ($result) {
          return $result;
        }
      }
    }
  }
  #Delete old files
   foreach my $aKey (keys %$oldFileList) {
     unless( exists $newFileList->{$aKey}){
       my $aFile = catfile($mainPath,$aKey);
       if(-f $aFile){
         unlink ($aFile);
       }
     }
   }
  $result =  $this->preunpack();
  if ($result) { return $result; }
  $result = $this->unpackIt(0);
  if ($result) { return $result; }
  return $result;
}



sub pmakedir{
  my ($this,$dir,$perm) = @_;
  my $retVal=0;
  $perm = 0777 unless $perm;
  unless($dir){
    return "Directory not specified";
  }
  makedir($dir,$perm,$this->GetUID(),$this->GetGRID());
  unless(-d $dir){
    $retVal = $!;
  }
  return $retVal;
}

sub GetPaths{
  return ();
}





sub removePasswordFromCommand{
  my ($this,$command) = @_;
  $command =~ s/\s+(\w+),(\w+)\s*/ $1,*** /g;
  return $command;
}












package SAPDB::Install::Templates2::PackageBase;
use SAPDB::Install::Registry;
use SAPDB::Install::System;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::System::Win32::Registry;
use SAPDB::Install::DBMCmd;
unless($^O =~ /mswin/i){
  use SAPDB::Install::System::Unix;
  use SAPDB::Install::System::Unix::Dir;
}

our @ISA = qw (SAPDB::Install::Templates2::InstUnit);


my %datapath;
my %progpath;
$datapath{'name'} = 'independent data path';
$datapath{'mode'}=0775;
$datapath{'opt'}='indep_data=s';
$progpath{'name'} = 'independent program path';
$progpath{'opt'} = 'indep_prog=s';
$progpath{'update'}=0;

my  $initPath = '/usr/spool/sql/ini/SAP_DBTech.ini';
my $globPath = '/etc/opt/sdb';


sub initData{
  my ($this,$rhData)=@_;
  $this->SUPER::initData($rhData);
  $datapath{'default'} = catfile(getAppDataPath(),'sdb/data');
  $progpath{'default'}=catfile(getSysProgPath(),'sdb/programs');
}


sub postuninstall{
  my $this = shift;
  $this->SUPER::postuninstall();
  if($^O =~ /mswin/i){
    my $key;
    my $tech =  'SOFTWARE\SAP\SAP DBTech';
    my $sap = 'Software\SAP';
    my $sapKey = 'SAP DBTech';
    my $warn = 0;
    my $res  = RegOpenKeyEx(&HKEY_LOCAL_MACHINE,$tech, 0, &KEY_ALL_ACCESS , $key );  
    if($res){ $this->warnit(  "WARNING: Could not  open registry key $tech.\n"); }
    $res = RegDeleteValue($key,'IndepData'); 
    if($res){ $this->warnit(  "WARNING: Could not  delete registry value 'IndepData'.\n"); }
    $res = RegDeleteValue($key,'IndepPrograms'); 
    if($res){ $this->warnit(  "WARNING: Could not  delete registry value 'IndepPrograms'.\n"); }
    $res = RegCloseKey($key); 
    if($res){ $this->warnit(  "WARNING: Could not  closing registry key $tech.\n"); }
    $res = RegOpenKeyEx(&HKEY_LOCAL_MACHINE, $sap, 0, &KEY_ALL_ACCESS , $key); 
    if($res){ $this->warnit(  "WARNING: Could not  open registry key $sap.\n"); }
    $res = RegDeleteKey($key,$sapKey); 
    if($res){ $this->warnit(  "WARNING: Could not  delete registry key $sapKey.\n"); }
    $res = RegCloseKey($key); 
    if($res){ $this->warnit(  "WARNING: Could not close registry key.\n"); }
  }
  else{
    -f $initPath and unlink($initPath); 
    -f $initPath and $this->warnit("WARNING: Cannot delete '$initPath'\n");
    -f $globPath and unlink($globPath); 
    -f $globPath and  $this->warnit("WARNING: Cannot delete '$globPath'\n");
  }
  return 0;
}


sub GetPaths{
  return (\%datapath,\%progpath);
}


sub GetServicePath{
  my $service_path;
  if ($^O=~/.*mswin.*/i) {
    my %hash = readValues(0,"SYSTEM\\CurrentControlSet\\Control\\Windows");
    if ($hash{'SystemDirectory'}) {
      $service_path="$hash{'SystemDirectory'}/drivers/etc/services";
      $service_path=~s/\\/\//g;			
    }
  } else {
    $service_path="/etc/services";
  }
  return $service_path;
}

sub unregister{
  my $this = shift;
  $this->SUPER::register();
#   unless($^O =~ /mswin/i){
#     -f '/etc/opt/sdb' and unlink '/etc/opt/sdb'; 
#     -f '/etc/opt/sdb' and $this->warnit("Could not delete /etc/opt/sdb: $!");
#   }
  return 0;
}

sub register{
  my $this = shift;
  $this->SUPER::register();
  if($^O =~ /mswin/i){
    return $this->registerWin();
  }
  else{
    return $this->registerUx();
  }
}
sub registerWin{
  my $this = shift;
  my $retVal = 0;
  my $command = " -s dbm_setpath IndepProgPath \\\"". $this->getIndepProg() . "\\\"";
  $retVal = $this->call_dbmcli($command);
  if($retVal){return $retVal;}
  $command = " -s dbm_setpath IndepDataPath \\\"" . $this->getIndepData() . "\\\"";
  $retVal = $this->call_dbmcli($command);
  return $retVal;
}

sub registerUx{
  my $this = shift;
  my $retVal =0;
  my $conf = {};
  my $indepProg = $this->getIndepProg();
  my $indepData = $this->getIndepData();
  my $sdbconfig = catfile($indepProg,"bin","sdbconfig");
  my $user = $this->getUser();
  my $group = $this->getGroup();
  if (-f $globPath && open (CONFIG, $globPath)) {
    while (my $line = <CONFIG>) {
      chomp $line;
      foreach my $key 
        ('IndepData', 'IndepPrograms', 'SdbOwner', 'SdbGroup') {
          if ($line =~ /^$key=/) {
            $line =~ s/^$key=//;
            $conf->{$key} = $line;
          }
        }
    }
    close (CONFIG);
  }
  unless (
          exists $conf->{IndepData} && $conf->{IndepData} eq $indepData &&
          exists $conf->{IndepPrograms} && $conf->{IndepPrograms} eq $indepProg &&
          exists $conf->{SdbOwner} && $conf->{SdbOwner} eq $user &&
          exists $conf->{SdbGroup} && $conf->{SdbGroup} eq $group) {
    my $path = '/etc/opt';
    makedir($path,0555) unless -d $path;
    my $res ='';
    callSystem("$sdbconfig IndepData=$indepData",sub{$res.=$_[0];}) == 0
      or return "ERROR: Failed command was '$sdbconfig IndepData=$indepData'\n$res";
    $res='';
    callSystem("$sdbconfig IndepPrograms=$indepProg",sub{$res.=$_[0];}) == 0
      or return "ERROR: Failed command was '$sdbconfig IndepPrograms=$indepProg'\n$res";
    $res='';
    callSystem("$sdbconfig SdbOwner=$user",sub{$res.=$_[0];})== 0
      or return "ERROR: Failed command was '$sdbconfig SdbOwner=$user'\n$res";
    $res='';
    callSystem("$sdbconfig SdbGroup=$group",sub{$res.=$_[0];})== 0
      or return "ERROR: Failed command was '$sdbconfig SdbGroup=$group'\n$res";
    $res='';
  }
  return $retVal;
}

sub writeServices{
  my $service_path = GetServicePath;
  open(SRV,$service_path) or return $!;
  my %port=(
            'sql6' => '7210/tcp',
            'sapdbni72' => '7269/tcp'
           );
  my %service_exist;
  while (<SRV>) {
    foreach my $service (keys(%port)) {
      if (/^$service\s/) {
        $service_exist{$service}=1;
      }
    }
  }
  close(SRV);
  # append SAP DB services to etc/services
  open(SRV,">>$service_path") or return $!;
  foreach my $service (keys(%port)) {
    unless ($service_exist{$service}==1) {
      print SRV "$service\t\t$port{$service}\n";
    }
  }
  close(SRV);
  return 0;
}

sub preinstall(){
  my $this = shift;
  my $retVal =0;
  $retVal = writeServices;
  $datapath{'value'}=$this->getIndepData;
  $progpath{'value'}=$this->getIndepProg;
  if($retVal){return $retVal;}
  return $retVal;
}


#Base package: postinstall
sub postinstall{
  my $this = shift;
  my $retVal=0;
  $retVal = $this->SUPER::postinstall(@_) and return $retVal;
  my $instConfig = $this->getInstTemplate();
  my $indepData = $instConfig->getIndepData();
  my $mainPath = $this->getMainPath();
  my $grid = $this->GetGRID();
  my $uid = $this->GetUID();
  my $version = $this->getVersion();
  my $aDir = catfile($indepData,'config');
  not -d $aDir and $retVal = $this->pmakedir($aDir,0770) and  return "Error creating directory $aDir:\n".$retVal;
  $aDir = catfile($indepData,'config/install');
  not -d $aDir and $retVal = $this->pmakedir($aDir,0770) and  return "Error creating directory $aDir:\n".$retVal;
  $aDir = catfile($indepData,'config/wrk');
  not -d $aDir and $retVal = $this->pmakedir($aDir,0770) and  return "Error creating directory $aDir:\n".$retVal;
  unless ($^O =~ /mswin/i){
    my $aFile = catfile($mainPath,'bin/starter');
    chmod (0500, $aFile) or return "Cannot chmod directory '$aFile' to 0500:\n$!";
    $aFile = catfile($mainPath,'bin/starter/sdbstarter');
    chown (0, $grid, $aFile) or return "Cannot chown file '$aFile' to 0,$grid:\n$!";
    chmod (04510, $aFile) or return "Cannot chmod file '$aFile' to 04510:\n$!";
    # create directory tree <data>/...
    foreach my $dir ('config','ipc','fifo','diag','dbspeed','pipe','pid','ppid','pid/serverpids') {
      my $mode = $dir eq 'config' ? 0775 :
        $dir eq 'ipc'    ? 0775 :
          $dir eq 'fifo'   ? 0775 :
            0770;
      my $longdir = catfile($indepData,$dir);
      if (-d $longdir) {
        chmod($mode,$longdir) or return "Cannot chmod '$longdir' to ".sprintf ("%04o", $mode).":\n$!";    
        chown($uid,$grid,$longdir) or return "Cannot chown '$longdir' to $uid,$grid:\n$!";
      } else {
        $this->pmakedir($longdir,$mode,$uid,$grid) and return "Cannot create directory '$longdir':\n$!" ;
      }
    }
    my $longdir = "/usr/spool/sql";
    if (-d $longdir) {
      chmod(0775,$longdir) or return "Cannot chmod '$longdir' to 775\n";
      chown($uid,$grid,$longdir) or return "Cannot chown '$longdir' to $uid,$grid:\n$!";    
    } else {
      $this->pmakedir ($longdir,0775,$uid,$grid) and return "Cannot create directory '$longdir'\n$!"; 
    }
    my $db_ini = catfile($indepData,'config/Databases.ini');
    unless(-f $db_ini){
      open(DBR,'>'.$db_ini) or return "cannot create '$db_ini':\n$!";
      close(DBR);
    }
    chmod(0660,$db_ini)or return "Cannot chmod '$db_ini' to 0660:\n$!";
    chown($uid,$grid,$db_ini) or return "Cannot chown '$db_ini' to $uid,$grid:\n$!";
    foreach my $dir ('ipc','fifo','diag','dbspeed','pid','ppid','pipe') {
      my $fullname = catfile("/usr/spool/sql",$dir);
      my $ais = catfile($indepData,$dir);
      if (-l $fullname) {
        unlink($fullname);
      } else {
        $this->delTree($fullname);
      }
      symlink ($ais, $fullname) or return "Cannot create symlink '$fullname' -> '$ais'.\n$!"; 
      lchown ($uid,$grid,$fullname) or return "Cannot chown of symlink '$fullname' to $uid,$grid.\n"; 
    }
    $longdir = "/usr/spool/sql/ini";
    if (-d $longdir) {
      chmod(0777,$longdir) or return "Cannot chmod '$longdir' to 0777:\n$!";
      chown($uid,$grid,$longdir) or return "Cannot chown '$longdir' to $uid,$grid.\n$!";
    } else {
      $this->pmakedir ($longdir,0777) and return "Cannot create directory '$longdir'\n"; 
    }

    my $inifile = "/usr/spool/sql/ini/SAP_DBTech.ini";
    unless (-f $inifile) {
      open (INI, ">$inifile") or return "cannot create SAP_DBTech.ini:\n$!";
      print INI "[Globals]\n";
      print INI "IndepData=$indepData\n";
      print INI "IndepPrograms=$mainPath\n";
      close INI;
    }

    chown ($uid,$grid,$inifile) or return "Cannot chown '$inifile' to $uid,$grid.\n" ;
    chmod (0666,$inifile) or return "Cannot chmod '$inifile' to 0666\n"; 
    if ($this->crelease2num($version) < $this->crelease2num('7.5.00.00')) {
      #  setServerRootPermissions ($packobj->User) or return "Cannot change permissions of server software root directories\n";
    }
  }
  return 0;
}


package   SAPDB::Install::Templates2::PackageDBM;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Registry ;
use SAPDB::Install::System::Win32::Registry ;

our @ISA = qw (SAPDB::Install::Templates2::InstUnit);



sub myKeys{
my $instScript = <<"INSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{38E77B11-839B-4138-B002-598026C48116\}-DlgOrder]
Dlg0=\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcome-0
Count=5
Dlg1=\{38E77B11-839B-4138-B002-598026C48116\}-SdAskDestPath-0
Dlg2=\{38E77B11-839B-4138-B002-598026C48116\}-SdSelectFolder-0
Dlg3=\{38E77B11-839B-4138-B002-598026C48116\}-SdStartCopy-0
Dlg4=\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0
[\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcome-0]
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdAskDestPath-0]
szDir=\${winInstPath}
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdSelectFolder-0]
szFolder=MySQL MaxDB
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdStartCopy-0]
Result=1
[Application]
Name=Database Manager
Version=\${version}
Company=SAP AG
Lang=0009
[\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{765537B8-DA4C-480E-917E-E25CAA26CE99\}-DlgOrder]
Count=0
[\{B464AF4C-EFBF-4488-82DA-E1CF26531D34\}-DlgOrder]
Count=0
[\{9829D65A-1914-418C-98B6-48008C42FFD2\}-DlgOrder]
Count=0
[\{4E709CE6-A288-462A-AE8B-E76CB9AD0342\}-DlgOrder]
Count=0
INSTFILE
  my $uninstScript = <<"UNINSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{38E77B11-839B-4138-B002-598026C48116\}-DlgOrder]
Dlg0=\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcomeMaint-0
Count=3
Dlg1=\{38E77B11-839B-4138-B002-598026C48116\}-SprintfBox-0
Dlg2=\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0
[\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcomeMaint-0]
Result=303
[\{38E77B11-839B-4138-B002-598026C48116\}-SprintfBox-0]
Result=1
[Application]
Name=Database Manager
Version=\${version}
Company=SAP AG
Lang=0009
[\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{765537B8-DA4C-480E-917E-E25CAA26CE99\}-DlgOrder]
Count=0
[\{B464AF4C-EFBF-4488-82DA-E1CF26531D34\}-DlgOrder]
Count=0
[\{9829D65A-1914-418C-98B6-48008C42FFD2\}-DlgOrder]
Count=0
[\{4E709CE6-A288-462A-AE8B-E76CB9AD0342\}-DlgOrder]
Count=0
UNINSTFILE

  my $updateScript = <<"UPDATEFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[{38E77B11-839B-4138-B002-598026C48116}-DlgOrder]
Dlg0={38E77B11-839B-4138-B002-598026C48116}-SdWelcome-0
Count=3
Dlg1={38E77B11-839B-4138-B002-598026C48116}-SdStartCopy-0
Dlg2={38E77B11-839B-4138-B002-598026C48116}-SdFinish-0
[{38E77B11-839B-4138-B002-598026C48116}-SdWelcome-0]
Result=1
[{38E77B11-839B-4138-B002-598026C48116}-SdStartCopy-0]
Result=1
[Application]
Name=Database Manager
Version=\${version}
Company=SAP AG
Lang=0009
[{38E77B11-839B-4138-B002-598026C48116}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[{AAE16ED0-5751-4059-A3AC-C8CEEA9EC206}-DlgOrder]
Count=0
[{CC2821B5-2907-4B12-AA15-73F545F171D2}-DlgOrder]
Count=0
[{21D3C994-574E-4D5C-8B5A-13A8CB209AAB}-DlgOrder]
Count=0
[{38C6A71B-4176-49C2-814E-D078808CCF5B}-DlgOrder]
Count=0
UPDATEFILE
  return {
          winInstPath =>{ 'DEFAULT' => '${indepProg}\DBM' },
          winExePath =>{ 'DEFAULT' => '${indepProg}\misc' },
          mainPath =>{ 'DEFAULT' => '${indepProg}' },
          instScript =>{ 'DEFAULT' => $instScript },
          uninstScript =>{ 'DEFAULT' => $uninstScript },
          updateScript =>{ 'DEFAULT' => $updateScript },
          exePrefix =>{ 'DEFAULT' => 'DBM' },
          versionKey => { 'DEFAULT' => 'SOFTWARE\SAP AG\Database Manager\7.6'},
          classID => {},# must be read from registry
          uninstKey => { 'DEFAULT' => 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\${classID}'}
         };
}






sub getInstalledVersionForRegistryKey(){
  my ($this,$aRegistryKey) = @_;
  my  %aValueHash=readValues(0,$aRegistryKey);
  my $installedVersion = $aValueHash{'Version'};
  return $installedVersion;
}

sub getInstalledVersionViaReg(){
  my ($this) = @_;
  my $aVersionKey = $this->getVersionKey();
  my $installedVersion = $this->getInstalledVersionForRegistryKey($aVersionKey);
  if(not $installedVersion){
    my $r74=$aVersionKey;
    $r74 =~ s/\d\.\d$/7\.4/;
    my $r75 = $aVersionKey;
    $r75 =~ s/\d\.\d$/7\.5/;
    my $r76 = $aVersionKey ;
    $r76 =~ s/\d\.\d$/7\.6/;
    foreach my $key ($r74,$r75,$r76){
      unless($key eq $aVersionKey){
        $installedVersion = $this->getInstalledVersionForRegistryKey($key);
        if($installedVersion){
          last;
        }
      }
    }
  }
  return $installedVersion;
}


sub willSkip{
  my ($this) = @_;
  my $installedVersion=$this->getInstalledVersionViaReg();
  my $aVersion= $this->getVersion();
  my $retVal = 0;
  if ( $installedVersion and $aVersion le $installedVersion) {
      $retVal=1;
  }
  return $retVal;
}




sub addLogs{
  my $instance = shift;
  my $instConfig = $instance->getInstTemplate();
  push @{$instConfig->GetListeners('log')},(sub {Logit($_[0]);});
  push @{$instConfig->GetListeners('warning')},(sub {Logit("WARNING: " . $_[0]);});
  push @{$instConfig->GetListeners('debug')},(sub { Debugit(@_);});
}

sub createExternalInstallerInstanceFromPackobj{
  my ($packobj,$instConfig) = @_;
  my ($retVal,$instance);
  ($retVal,$instance) = SAPDB::Install::Templates2::InstUnit::createObjectFromTGZ($packobj->Untgz(),$instConfig);
  my $aPathStruct = $packobj->MainPath();
  my $aPath ; 
  $aPathStruct and 
    $aPath = $aPathStruct->{'value'} and $instance->setMainPath($aPath);
  $retVal and return ($retVal,undef);
  addLogs($instance);
  my $mainPath = $instance->getMainPath();
  $instance->debugit(1,"Installation path is '$mainPath'");
  return ($retVal,$instance);
}

sub createExternalInstallerInstanceFromReg{
  my ($reg,$instConfig) =@_;
  my ($retVal,$instance);
  ($retVal,$instance) = SAPDB::Install::Templates2::InstUnit::createObjectFromRegistry($reg,$instConfig);
  $retVal and return ($retVal,undef);
  addLogs($instance);
  return ($retVal,$instance);
}





sub preuninstall{
  my ($this,$retVal) = (shift,0);
  $retVal = $this->SUPER::preuninstall(@_) and return $retVal;
  #Get parameters
  my ($mainPath , $version , $exePrefix , $responseFile , $logFile , $aExe , $name ) = $this->getMyParams();
  #determine already installed version
  my $installedVersion;
  ($retVal,$installedVersion) = $this->getInstalledVersion();
  $retVal and return $retVal;
  #Check if same version is installed:
  #if not: return
  my $aScript;  
  if (not defined $installedVersion){
    $this->logit("$name is not installed (Skipping)");
    return 0;
  }
  elsif($installedVersion ne $version){
    return "$name is installed in Version $installedVersion, cannot uninstall Version $version";
  }
  else{
    $aScript = $this->getUninstScript();
  }
  #find executable  uninstaller executable

#   my $uninstKey = $this->getUninstKey();
#   my %values = readValues(&HKEY_LOCAL_MACHINE,$uninstKey);
#   print("$uninstKey =  '" . join(",",keys %values)."'");
#   exists $values{'(Default)'} or return "Cannot get uninstaller for $name ";
#   my $uninstValue = $values{'(Default)'};
#   ($aExe) = ($uninstValue =~ /^(\S)\s/);
#   $this->logit("uninstaller executable is '$aExe'");

  #Execute the external uninstaller
  $retVal = $this->callExternalExe($responseFile,$aExe,$aScript,$logFile,$name) and return $retVal;
  $this->logit("Execution of external uninstaller for  $name was successfull");
  return $retVal;
}#PackageDBM::preuninstall
sub staticpreuninstall{
  my ($regpackobj,$instConfig) = @_;
  my ($retVal,$obj)=(0);
  ($retVal,$obj) = createExternalInstallerInstanceFromReg($regpackobj,$instConfig);
  $retVal and return $retVal;
  $retVal = $obj->preuninstall() and return $retVal;
  return $retVal;
}

sub postinstall{
  my ($this,$retVal) = (shift,0);
  $retVal = $this->SUPER::postinstall(@_) and return $retVal;
  #Get parameters
  my ($mainPath , $version , $exePrefix , $responseFile , $logFile , $aExe , $name ) = $this->getMyParams();
  #determine already installed version
  my $installedVersion;
  ($retVal,$installedVersion) = $this->getInstalledVersion();
  $retVal and return $retVal;
  #Determine response file depending on installed version:
  my $aScript;  
  if (not defined $installedVersion){
    $aScript =  $this->getInstScript();
  }
  elsif($installedVersion lt $version){
    $aScript =  $this->getUpdateScript();
    $this->logit("$name already installed in version $installedVersion. Upgrading to $version.");
  }
  elsif($installedVersion eq $version){
    $this->logit("$name already installed in version $installedVersion, skipping.");
    return 0;
  }
  elsif($installedVersion gt $version){
    return "Cannot downgrade $name $installedVersion -> $version";
  }
  #Execute the external installer with the response file
  $retVal = $this->callExternalExe($responseFile,$aExe,$aScript,$logFile,$name) and return $retVal;
  $this->logit("Execution of external installer for  $name was successfull");
  return $retVal;
}



sub staticwillskip{
  my ($packobj,$instConfig) = @_;
  my ($retVal,$obj)=(0);
  ($retVal,$obj) = createExternalInstallerInstanceFromPackobj($packobj,$instConfig);
  $retVal and return ($retVal,undef);
  return (0,$obj->willSkip());
}


sub staticpostinstall{
  my ($packobj,$instConfig) = @_;
  my ($retVal,$obj)=(0);
  ($retVal,$obj) = createExternalInstallerInstanceFromPackobj($packobj,$instConfig);
  $retVal and return $retVal;
  $retVal = $obj->postinstall() and return $retVal;
  return $retVal;
}






#0  Success. 
#-1  General error. 
#-2  Invalid mode. 
#-3  Required data not found in the Setup.iss file. 
#-4  Not enough memory available. 
#-5  File does not exist. 
#-6  Cannot write to the response file. 
#-7  Unable to write to the log file. 
#-8  Invalid path to the InstallShield Silent response file. 
#-9  Not a valid list type (string or number). 
#-10  Data type is invalid. 
#-11  Unknown error during setup. 
#-12  Dialog boxes are out of order. 
#-51  Cannot create the specified folder. 
#-52  Cannot access the specified file or folder. 
#-53  Invalid option selected. 










sub callExternalExe{
  my $retVal =0;
  my ($this,$responseFile,$aExe,$aScript,$logFile,$name) = @_;
  open FH, ">$responseFile" or return "Cannot open install response file '$responseFile': $!";
  print FH ($aScript);
  close FH;
  my $command = "\"$aExe\"" . " -s -a -s  -f1\"" . $responseFile . "\" -f2\"" . $logFile . "\"";
  $this->logit("[".$command."]"); 
  $retVal = callSystem($command, sub{}) and return "Error installing $name: $retVal";
  open FH, "<$logFile" or return "Cannot read log file '$logFile': $!";
  my $resultCode;
  foreach(<FH>){
    if(/^\s*ResultCode\s*=\s*([-0-9]+)\s*$/){
      $resultCode = $1;
      last;
    }
  }
  close FH;
  defined $resultCode or return "Cannot get resultcode in '$logFile'";
  $this->logit("ResultCode from installer log: $resultCode");
  $resultCode eq '0' or return "Error executing installer of $name";
  #unlink($aScript);
  #unlink($logFile);
  return $retVal;
}


sub getInstalledVersion{
  my ($this) = @_;
  my ($retVal,$version)=(0,'');
  $version = $this->getInstalledVersionViaReg();
  return ($retVal,$version);
}

sub getMyParams{
  my $this = shift;
  my $mainPath = $this->getMainPath();
  my $exePrefix =  $this->getExePrefix();
  my $version =  $this->getVersion();

  return ( $mainPath,
           , $version
           , $exePrefix
           , catfile($mainPath,"misc","inst$exePrefix.iss")
           , catfile($mainPath,"misc","inst$exePrefix.log")
           , catfile($mainPath,"misc",join( "" , ($exePrefix, ($version =~ /^(\d+)\.(\d+)\./), ".exe")))
           ,$this->getName());
}



package   SAPDB::Install::Templates2::PackageServerUtil;
use SAPDB::Install::System;
use SAPDB::Install::Templates2::Util;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);


my $progpath;
$progpath{'name'} = 'independent program path';
$progpath{'list'} = 'utils.lst';
$progpath{'opt'} = 'indep_prog=s';
$progpath{'mode'}=0775;
$progpath{'update'}=0;
sub GetPaths{ return (\%progpath); }
sub initData{
  my ($this,$rhData)=@_;
  $this->SUPER::initData($rhData);
  $progpath{'default'}=catfile(getSysProgPath(),'sdb/programs');
}

sub preinstall{
  my $this = shift;
  my $result = $this->SUPER::preinstall();
  if($result ){
    return $result;
  }
  $progpath{'value'}=$this->getIndepProg;
  return $result;
}


sub register{
  my $this = shift;
  $this->SUPER::register();
  unless($^O =~ /mswin/i){ return 0; }
  my $retVal=$this->call_x_server("install",['19712','19709']);
  return $retVal;
}

sub unregister{
  my $this = shift;
  $this->SUPER::unregister();
  my $retVal=$this->call_x_server("stop",['18640']);
  unless($^O =~ /mswin/i){ return $retVal; }
  $retVal=$this->call_x_server("remove",['19710','18581']);
  return $retVal;
}


package SAPDB::Install::Templates2::DatabaseInstance;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::InstInfo;
use SAPDB::Install::Registry;
use SAPDB::Install::FSUsage;
use SAPDB::Install::DBMCmd;
use SAPDB::Install::MachInfo;
use SAPDB::Install::Portable::Helper;

our @ISA = qw (SAPDB::Install::Templates2::InstUnit);


our $DEFAULTSTATUSLIST ;
if(exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
$DEFAULTSTATUSLIST = << "DEFAULTSTATUSLIST";
checkVolumes|Checking free space on filesystem|PENDING||
crInstance|Creating database instance|PENDING||
logOn|Logging in|PENDING||
dbParams|Setting database parameters |PENDING||
volumePaths|Creating paths for volumes|PENDING||
crVolumes|Adding volumes|PENDING||
goAdmin|Going to admin mode|PENDING||
activate|Activating database|PENDING||
statistics|Setting statistics configuration|PENDING||
autoExtend|\${AUTOEXTENDSTATUSMESSAGE}|PENDING||
loadSystab|Loading system tables|PENDING||
loadSampleSchema|\${LOADSAMPLESCHEMAMESSAGE}|PENDING||
logConfiguration|Setting log configuration to \${LOGSTRATEGY}|PENDING||
createUsers|Creating users|PENDING||
DEFAULTSTATUSLIST
}
else{
$DEFAULTSTATUSLIST = << "DEFAULTSTATUSLIST";
checkVolumes|Checking free space on filesystem|PENDING||
crInstance|Creating database instance|PENDING||
startxserver|Starting communication service|PENDING||
logOn|Logging in|PENDING||
dbParams|Setting database parameters |PENDING||
volumePaths|Creating paths for volumes|PENDING||
crVolumes|Adding volumes|PENDING||
goAdmin|Going to admin mode|PENDING||
activate|Activating database|PENDING||
statistics|Setting statistics configuration|PENDING||
autoExtend|\${AUTOEXTENDSTATUSMESSAGE}|PENDING||
loadSystab|Loading system tables|PENDING||
loadSampleSchema|\${LOADSAMPLESCHEMAMESSAGE}|PENDING||
logConfiguration|Setting log configuration to \${LOGSTRATEGY}|PENDING||
createUsers|Creating users|PENDING||
DEFAULTSTATUSLIST
}


my $dataAreaLabel =   "Data Area" ;
my $logAreaLabel = "Log Area";
my $userRegexp =   '^\s*\w+\s*,\s*\w+\s*$';

my $baseInstanceData = << "XMLDATA";
<ParameterDefinitions>
  <Parameter id="dBName" dependencies="dependendPath" persistent="YES">
    <Type>STRING</Type>
    <Default>MAXDB1</Default>
    <Description>Database Name</Description>
  </Parameter>
  <Parameter id="dBHost" >
    <Type>STRING</Type>
    <Default>localhost</Default>
    <Description>DB Name</Description>
  </Parameter>
  <Parameter id="oSUser" >
    <Type>STRING</Type>
    <Description>OS User for db create</Description>
  </Parameter>
  <Parameter id="dBAUser" >
    <Type>STRING</Type>
  </Parameter>
  <Parameter id="dBAdmin" >
    <Type>USER</Type>
    <Validator>$userRegexp</Validator>
    <Default>DBADMIN,DBM</Default>
  </Parameter>
  <Parameter id="dBVolume" >
  </Parameter>
  <Parameter id="dBAPass" >
    <Type>PASSWORD</Type>
  </Parameter>
  <Parameter id="dBMUser" >
    <Type>STRING</Type>
  </Parameter>
  <Parameter id="dBManager" >
    <Type>STRING</Type>
    <Validator>$userRegexp</Validator>
    <Default>DBM,DBM</Default>
  </Parameter>
  <Parameter id="dBMPass" >
    <Type>PASSWORD</Type>
  </Parameter>
  <Parameter id="domainPass" >
    <Type>PASSWORD</Type>
    <Default>domain</Default>
  </Parameter>
  <Parameter id="dBParams" >
  </Parameter>
  <Parameter id="loadSampleSchema"  >
    <Type>BOOLEAN</Type>
     <Description>Flag indicating wether sample schema should be loaded</Description>
  </Parameter>
  <Parameter id="parallelConnections"  unit="number">
    <Type>INTEGER</Type>
  </Parameter>
  <Parameter id="autoRestart" >
    <Type>BOOLEAN</Type>
  </Parameter>
  <Parameter id="autoExtend" >
    <Type>BOOLEAN</Type>
  </Parameter>
  <Parameter id="autoExtendSize" >
    <Type>STRING</Type>
     <Description>Percentage of increase for auto extend feature</Description>
  </Parameter>
  <Parameter id="autoUpdateStatistics" >
    <Type>BOOLEAN</Type>
  </Parameter>
  <Parameter id="autoSaveLog" >
    <Type>BOOLEAN</Type>
  </Parameter>
  <Parameter id="autoOverWriteLog" >
    <Type>BOOLEAN</Type>
  </Parameter>
  <Parameter id="initialBackupDir" >
    <Type>PATH</Type>
    <Default>\${DATAPATHPREFIX}/\${DBNAME}/backup/data</Default>
  </Parameter>
  <Parameter id="initialBackupFile" >
    <Type>PATH</Type>
    <Default>\${INITIALBACKUPDIR}/\${DBNAME}.dat</Default>
  </Parameter>
  <Parameter id="logSaveDir" >
    <Type>PATH</Type>
    <Default>\${DATAPATHPREFIX}/\${DBNAME}/backup/log</Default>
  </Parameter>
  <Parameter id="dataPathPrefix" >
    <Type>PATH</Type>
    <Default>\${INDEPDATA}</Default>
  </Parameter>
  <Parameter id="dataPathPostfix" >
    <Type>PATH</Type>
    <Default>\${DBNAME}/data</Default>
  </Parameter>
  <Parameter id="logPathPrefix" >
    <Type>PATH</Type>
    <Default>\${INDEPDATA}</Default>
  </Parameter>
  <Parameter id="logPathPostfix" >
    <Type>PATH</Type>
    <Default>\${DBNAME}/log</Default>
  </Parameter>
  <Parameter id="dependendPath" persistent="YES" >
    <Type>PATH</Type>
     <Description>Path  of database kernel installation</Description>
  </Parameter>
  <Parameter id="parallelConnections" unit="number">
    <Default>20</Default>
  </Parameter>
  <Parameter id="loggingStruct" >
    <Type>LOGGING.COMPOUND</Type>
  </Parameter>
  <Parameter id="logStrategy" >
    <Type>STRING</Type>
  </Parameter>
  <Parameter id="logStrategyString" >
    <Type>STRING</Type>
  </Parameter>
  <Parameter id="isAutoLogStrategy" />
  <Parameter id="cPUCount" unit="number">
    <Type>INTEGER</Type>
  </Parameter>
  <Parameter id="dBMemory"  persistent="YES" unit="pages">
    <Type>DATASIZE</Type>
    <Description>Memory consumption in 8K pages.(Calculated dynamically for local host!)</Description>
  </Parameter>
  <Parameter id="statusList" />
  <Parameter id="autoExtendStatusMessage" />
  <Parameter id="loadSampleSchemaMessage" />
  <Parameter id="sQLUser"  persistent="NO" >
    <Description>
Database users to be created. Comma seperated list of length two or three:
USER,PASSWORD(,ASCII|UNICODE)?
Last parameter can be omitted and defaults to UNICODE.
    </Description>
  </Parameter>
  <Parameter id="xUserKey"  persistent="NO" >
    <Description>
XUser Key to be set the format is
USER,PASSWORD,KEYVALUE
    </Description>
  </Parameter>
</ParameterDefinitions>
<ParameterData>
  <ParameterValue id="statusList">
$DEFAULTSTATUSLIST
  </ParameterValue>
</ParameterData>
XMLDATA
my @baseXmlInstanceData = split(/\n/,$baseInstanceData);




#OVERWRITTEN METHODS:



#get data definitions:
sub GetXmlData{ return \@baseXmlInstanceData; }


my $REGEXP1='^\s*DBPARAM\s*[\s=]?\s*$';
my $REGEXP2='^\s*DBPARAM\s*[\s=]\s*(.*)$';
my $REGEXP3='^\s*([\w\d]+)\s+(.+)\s*$';
my $REGEXP4='^\s*DBVOLUME\s*[\s=]\s*$';
my $REGEXP5='^\s*DBVOLUME\s*[\s=]\s*(.*)$';
my $REGEXP6='^\s*SQLUSER\s*[\s=]\s*$';
my $REGEXP7='^\s*SQLUSER\s*[\s=]\s*(.*)$';
my $REGEXP8='^\s*XUSERKEY\s*[\s=]\s*$';
my $REGEXP9='^\s*XUSERKEY\s*[\s=]\s*(.*)$';

my $SQLUSERREGEXP = '^(\w+)\s*,\s*(\w+)\s*(?:,\s*(ASCII|UNICODE)\s*(?:,(.*))?)?$';
my $XUSERKEYREGEXP = '^(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*(?:,(\w+))?\s*$';



#overwrite standard line parsing for DBPARAM and DBVOLUME-Parameters
sub parseLine{
  my ($this,$aLine) = @_;
  if ( $aLine =~ /$REGEXP1/i) {
      $this->setDBParams({});
  }
  elsif ( $aLine =~ /$REGEXP2/i) {
    my $rest = $1;
    if ( $rest =~ m/$REGEXP3/) {
      my $val = $2;
      my $key = $1;
      my $dbps = $this->getDBParams();
      $dbps = $dbps ?  $dbps : {};
      $dbps->{$key}=$val;
      $this->setDBParams($dbps);
    }
  }
   elsif ( $aLine =~ /$REGEXP4/i) {
     $this->setDBVolume([]);
   }
   elsif ( $aLine =~ /$REGEXP5/i) {
     my $vol = $1;
     my $vols = $this->Get('dBVolume',"no can","no subst");
     $vols or $vols =  [];
     push @$vols,($vol);
     $this->setDBVolume($vols);
   }
   elsif ( $aLine =~ /$REGEXP6/i) {
     $this->setSQLUser([]);
   }
   elsif ( $aLine =~ /$REGEXP7/i) {
     my $user = $1;
     my $userArray = $this->getSQLUser();
     $userArray or $userArray =  [];
     push @$userArray,($user);
     $this->setSQLUser($userArray);
   }
   elsif ( $aLine =~ /$REGEXP8/i) {
     $this->setXUserKey([]);
   }
   elsif ( $aLine =~ /$REGEXP9/i) {
     my $xuserKey = $1;
     my $xuserKeyArray = $this->getXUserKey();
     $xuserKeyArray or $xuserKeyArray =  [];
     push @$xuserKeyArray,($xuserKey);
     $this->setXUserKey($xuserKeyArray);
   }
  else {
    $this->SUPER::parseLine($aLine);
  }
}#parseLine



sub ValidateLine{
  my ($this,$aLine) = @_;
  if ( $aLine =~ /$REGEXP1|$REGEXP2|$REGEXP4|$REGEXP5|$REGEXP6|$REGEXP7|$REGEXP8|$REGEXP9/i) {
    return 0;
   }
  else{
    return $this->SUPER::ValidateLine($aLine);
  }
}

sub ExistsKey{
  my ($this,$aKey) = @_;
  if ( $aKey =~ /^\s*DBPARAM\s*$||^\s*DBVOLUME\s*$/i) {
    return 1;
  }
  elsif ( $aKey =~ /^\s*INDEPDATA\s*$||^\s*INDEPPROG\s*$/i) {
    return 0;
  }
  else{
    return $this->SUPER::ExistsKey($aKey);
  }
}




sub createParameterDescriptionArray{
  my ($this,$array) = @_;
  $array or return Logit("Array not specified in DatabanseInstance:createParameterDescriptionArray!",undef);
  if($array eq 'dataVolumes'){
    return $this->createVolumeParameterDescriptionArray('DATA');
  }
  elsif($array eq 'logVolumes'){
    return $this->createVolumeParameterDescriptionArray('LOG');
  }
  else{
   return Logit("Unknown array '$array'  in DatabanseInstance:createParameterDescriptionArray!",undef);
  }
}
#INSTANCE METHODS:


sub changeDBState{
  my ($this,$dbm,$state)=@_;
  my $retVal;
  $retVal = $this->dbmcmd($dbm,"$state") and return $retVal;
  my $result;
  $retVal = $this->dbmcmd($dbm,'db_state',\$result) and return $retVal;
  $result or return "Error after $state: 'db_state' gave me no output";
  my @result =  split('\n', $result);
  my $newState = $result[2];
  $newState or return "Error after '$state'. Could not verify database state:\n$result";
  $this->logit("Database state ist now $newState");
  return 0;
}
#changeState


sub instanceExists{
  my $this = shift;
  return utilInstanceExists($this->getDBName());
}
sub instanceWithNameExists{
  my ($this,$dbname) = @_;
  return utilInstanceExists($dbname);
}


sub modifyStatusList{
  my ($this,$list,$id,$newStatus,$startTime,$endTime,$newText) = @_;
  my $newList='';
  foreach(split(/\n/,$list)){
    my ($aid,$atxt,$astate,$astart,$aend) = split(/\|/);
    $aid =~ s/\s//g;
    if($aid eq $id){
      $startTime and $astart = $startTime;
      $endTime and $aend = $endTime;
      defined $newText and $atxt = $newText;
      $newStatus and $astate=$newStatus;
      $newList .= join ("|",($aid,$atxt,$astate,$astart,$aend)) . "\n";
    }
    else{
      $newList .= $_ . "\n";
    }
  }
  return $newList;
}


# sub skipUntil{
#   my ($this,$list,$id,$txt) = @_;
#   my $newList='';
#   foreach(split(/\n/,$list)){
#     my ($aid,$atxt,$astate,$astart,$aend) = split(/\|/);
#     $aid =~ s/\s//g;
#     if($aid eq $id){
#       $newList .= $_ . "\n";

#     }
#     else{
#        $astart = time();
#        $aend = time();
#        $astate='FINISHED_SUCCESS';
#        $atxt = $txt;
#        $newList .= join ("|",($aid,$atxt,$astate,$astart,$aend)) . "\n";
#     }
#   }
#   return $newList;
# }

# sub skiptToStatus{
#   my ($this,$id,$txt) = @_;
#   my $list = $this->getStatusList();
#   $list = $this->skipUntil($list,$id,$txt);
#   $this->setStatusList($list);
#   $this->status_callback($this->GetStatusArray());
# }





sub GetStatusArray{
  my ($this) = @_;
  my $st = $this->getStatusList();
  my $retVal = [];
  foreach(split(/\n/,$st)){
    my ($aid,$atxt,$astate,$astart,$aend) = split(/\|/);
    push @$retVal , (
                     {
                      'Id'=>$aid,'Label'=>$atxt,'Status'=>$astate,'StartTime'=>$astart,'EndTime'=>$aend
                     });
  }
  return $retVal;
}

sub startStatus{
  my ($this,$id) = @_;
  my $list = $this->getStatusList();
  $list = $this->modifyStatusList($list,$id,'RUNNING',time);
  $this->setStatusList($list);
  $this->status_callback($this->GetStatusArray());
}

sub endStatusError{
  my ($this,$id) = @_;
  my $list = $this->getStatusList();
  $list = $this->modifyStatusList($list,$id,'FINISHED_ERROR',undef,time);
  $this->setStatusList($list);
  $this->status_callback($this->GetStatusArray());
}

sub endStatusSuccess{
  my ($this,$id) = @_;
  my $list = $this->getStatusList();
  $list = $this->modifyStatusList($list,$id,'FINISHED_SUCCESS',undef,time);
  $this->setStatusList($list);
  $this->status_callback($this->GetStatusArray());
}

sub createVolumeParameterDescriptionArray{
  my ($this,$art) = @_;
  my @a;
  my $v = $this->getDBVolume($art);
  foreach (@$v) {
    my ($num,$art,$path,$type,$size )=split(/;/);
    my $osPath = makePathOSDependend($path);
    push @a, ($this->createParameterDescriptionByLabel('label:'.$path,$osPath));
  }
  return \@a;
}



sub createVolumeArray{
  my ($this,$art) = @_;
  my @a;
  my $v = $this->getDBVolume($art);
  foreach (@$v) {
    my ($num,$art,$path,$type,$size )=split(/;/);
    push @a, (makePathOSDependend($path));
  }
  return \@a;
}



sub deleteInstance{
  my $this = shift;
  my $retVal=0;
  my ($dbmuser,$dbmpass) = ($this->getDBMUser(),$this->getDBMPass());
  my $dbm = SAPDB::Install::DBMCmd->new ('localhost', $this->getDBName()) or return "DBMCmd not initialized!";
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmuser,$dbmpass") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_offline") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_drop") and return $retVal;
  return $retVal;
}



sub createInstance{
  my ($this) = @_;
  my ($result,$retVal,$dbm);
  #Before anything is done, check if there is enough space for the volumes.
  #if this check fails no half ready database instance will remain, because
  #we exit at once.
  $this->startStatus('checkVolumes');
  $retVal = $this->DoValidateDBVolume() and return $this->creationError('checkVolumes',$retVal);
  $this->endStatusSuccess('checkVolumes');

  #do db_create
  $this->startStatus('crInstance');
  $retVal = $this->DoDBCreate() and return $this->creationError('crInstance', $retVal);
  $this->endStatusSuccess('crInstance');



  #Start xserver
  if(not exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
    $this->startStatus('startxserver');
    ($retVal,$dbm) = $this->DoStartXServer();
    $retVal and return  $this->creationError('startxserver', $retVal);
    $this->endStatusSuccess('startxserver');
  }


  # Create DBMCmd-Object and use it to login as DBM
  $this->startStatus('logOn');
  ($retVal,$dbm) = $this->CreateDBMCmdObject();
  $retVal and return  $this->creationError('logOn', $retVal);
  $this->endStatusSuccess('logOn');

  #Initialize database parameters:
  $this->startStatus('dbParams');
  $retVal = $this->DoInitializeDBParams($dbm) and return  $this->creationError('dbParams',$retVal);
  $this->endStatusSuccess('dbParams');

  #Create volume paths:
  $this->startStatus('volumePaths');
  $retVal = $this->DoCreateVolumeDirectories() and return  $this->creationError('volumePaths',$retVal);
  $this->endStatusSuccess('volumePaths');

  #Add the volumes:
  $this->startStatus('crVolumes');
  $retVal = $this->DoParamAddVolumes($dbm) and return  $this->creationError('crVolumes',$retVal);
  $this->endStatusSuccess('crVolumes');

  #Go admin:
  $this->startStatus('goAdmin');
  $retVal = $this->DoChangeState($dbm,'db_admin') and return  $this->creationError('goAdmin',$retVal);
  $this->endStatusSuccess('goAdmin');

  #Do dbActivate
  $this->startStatus('activate');
  $retVal = $this->DoDBActivate($dbm) and return  $this->creationError('activate',$retVal);
  $this->endStatusSuccess('activate');

  #auto update statistics option:
  $this->startStatus('statistics');
  $this->DoAutoUpdateStatistics($dbm);
  $this->endStatusSuccess('statistics');

  #auto extend option:
  $this->startStatus('autoExtend');
  $retVal = $this->DoAutoExtend($dbm) and return  $this->creationError('autoExtend',$retVal);
  $this->endStatusSuccess('autoExtend');

  #load system tables:
  $this->startStatus('loadSystab');
  $retVal = $this->DoLoadSystab($dbm) and return  $this->creationError('loadSystab',$retVal);
  $this->endStatusSuccess('loadSystab');

  #load sample schema:
  $this->startStatus('loadSampleSchema');
  $retVal = $this->DoLoadSampleSchema($dbm) and return  $this->creationError('loadSampleSchema',$retVal);
  $this->endStatusSuccess('loadSampleSchema');

  #Logging options
  $this->startStatus('logConfiguration');
  $retVal = $this->DoLogStrategy($dbm) and return  $this->creationError('logConfiguration',$retVal);
  $this->endStatusSuccess('logConfiguration');

  #Create users:
  $this->startStatus('createUsers');
  $retVal = $this->DoInitializeUsers($dbm) and return  $this->creationError('createUsers',$retVal);
  $this->endStatusSuccess('createUsers');




  #DBM logs off from the new database 
  $this->logit("Closing DBM-Connection.");
  $dbm->release ();

  
  undef $dbm;
  return $retVal;
}#createInstance

sub creationError{
  my ($this,$status,$retVal) = @_;
  my $msg = $retVal;
  $msg =~ s/^\s*//;
  $msg =~ s/\s*$//;
  $this->logit("DB creation error in status '$status', message is:\n\"$msg\"\n");
  $this->endStatusError($status);
  return $retVal;
}





sub DoStartXServer{
  my($this) =@_;
  return $this->call_x_server("start",['18641','19720','11779']); 
}


sub DoLogStrategy{
  my($this,$dbm) =@_;
  my $retVal=0;
  my $logStrategy = $this->getLogStrategy();
  $logStrategy or return 0;
  if('AUTO' eq $logStrategy){
    return $this->DoAutoSaveLog($dbm);
  }
  elsif('NONE' eq $logStrategy){
    return $this->DoAutoOverwriteLog($dbm);
  }
  elsif('MANUAL' eq $logStrategy){
  }
  return $retVal;
}


sub DoAutoSaveLog{
  my($this,$dbm) =@_;
  my $retVal=0;
  $retVal = $this->DoPerformInitialBackup($dbm) and return $retVal;
  $retVal = $this->DoSwitchOnAutoSaveLog($dbm) and return $retVal;
  return $retVal;
}

sub DoAutoOverwriteLog{
  my($this,$dbm) =@_;
  return $this->dbmcmd($dbm, "db_execute SET LOG AUTO OVERWRITE ON"); 
}








sub DoSwitchOnAutoSaveLog(){
  my($this,$dbm) =@_;
  my $retVal = 0;
  my $dbname = $this->getDBName();
  my $logSaveDir = $this->getLogSaveDir();
  my $logBackupPath= "\"".catfile($logSaveDir,$dbname . "_autosavelog")."\"";
  my $mediumName = "LOGBACKUP";

  #Create the directory, if it does not exist
  if(not -d $logSaveDir){
    $this->logit("Creating log  directory $logSaveDir");
    mkpath ($logSaveDir,1,0777,$this->GetUID(),$this->GetGRID()) or return "Error creating log save directory $logSaveDir: $!";
  }
  else{
    $this->logit("Logging directory '$logSaveDir' already exists");
  }
  #Put the medium:
  $retVal = $this->dbmcmd($dbm, "medium_put LOGBACKUP $logBackupPath  FILE LOG 0 8 YES") and return $retVal;
  #start util session:
  $retVal = $this->dbmcmd($dbm, "util_connect") and return $retVal;
  #switch on auto log
  $retVal = $this->dbmcmd($dbm, "autolog_on LOGBACKUP") and return $retVal;
  #stop util session:
  $retVal = $this->dbmcmd($dbm, "util_release") and return $retVal;
  return $retVal;
}                               #DoSwitchOnAutoSaveLog







sub DoPerformInitialBackup{
  my($this,$dbm) =@_;
  my $retVal = 0;
  my $dbname = $this->getDBName();
  my $backupFile = makePathOSDependend($this->getInitialBackupFile());
  my $backupDir = dirname($backupFile);
  my $backupPath="\"".$backupFile."\"";
  #Create the directory, if it does not exist
  if(not -d $backupDir){
    $this->logit("Creating backup directory $backupPath");
    mkpath ($backupDir,1,0777,$this->GetUID(),$this->GetGRID()) or return "Error creating backup directory $backupDir: $!";
  }
  else{
    $this->logit("Backup directory '$backupDir' already exists");
  }
  #Put the medium:
  $retVal = $this->dbmcmd($dbm, "medium_put INITIALBACKUP $backupPath  FILE DATA 0 8 YES") and return $retVal;
  #start util session:
  $retVal = $this->dbmcmd($dbm, "util_connect") and return $retVal;
  #start the backup:
  $retVal = $this->dbmcmd($dbm, "backup_start INITIALBACKUP DATA") and return $retVal;
  #stop util session:
  $retVal = $this->dbmcmd($dbm, "util_release") and return $retVal;
  return $retVal;
}                               #DoPerformInitialBackup

sub DoValidateDBVolume(){
  my ($this) = @_;
  my($result, $msg1,$msg2 ) = $this->guiValidateDBVolume($this->getDBVolume());
  $result == 1 or return $msg1."\n".$msg2;
  return 0;
}#DoValidateDBVolume


sub DoAutoUpdateStatistics(){
  my($this,$dbm) =@_;
  if( $this->getAutoUpdateStatistics()){
    return  $this->dbmcmd($dbm,"auto_update_statistics ON");
  }
  else{
    return 0;
  }
}#DoAutoUpdateStatistics
sub DoAutoExtend(){
  my($this,$dbm) =@_;
  if($this->getAutoExtend()){
    return $this->dbmcmd($dbm,"auto_extend ON"); 
  }else{
    return 0;
  }
}#DoAutoExtend

sub DoLoadSampleSchema(){
  my($this,$dbm) =@_;
  if($this->getLoadSampleSchema()){
    return $this->dbmcmd($dbm,"load_tutorial");
  }
  else{
    return 0;
  }
}#DoLoadSampleSchema



sub DoLoadSystab{
  my ($this, $dbm) = @_;
  my $retVal = 0;
  my $dbadmin = $this->getDBAdmin();
  my $aDomainPass = $this->getDomainPass();
  my $loadCommand = "load_systab -u $dbadmin -ud $aDomainPass";
  $retVal = $this->async_dbmcmd ($dbm, $loadCommand,1) and return $retVal;
  return $retVal;
}#DoLoadSystab


sub DoChangeState{
  my ($this,$dbm,$state)=@_;
  my $retVal=0;
  $retVal = $this->async_dbmcmd($dbm, "$state", 1)  and return $retVal;
  return $retVal;
}#DoChangeState



#createInstanceInternal
sub DoDBActivate{
  my ($this,$dbm) = @_;
  my $retVal = 0;
  my $admin = $this->getDBAdmin();
  $retVal = $this->async_dbmcmd($dbm,"db_activate $admin",1);
  return $retVal;
}#DoDBActivate

sub DoParamAddVolumes{
  my ($this,$dbm) = @_;
  my $retVal = 0;
  my $vols = $this->getDBVolume();
  foreach (@$vols) {
    my ($num,$type,$path,$kind,$size)= split/;/; 
    if(not file_name_is_absolute($path)){
      $path = catfile('.',$path);
    }
#    $path = $this->relativeVolumePathToAbsoluteVolumePath($path);
    $path = makePathOSDependend($path);
    $retVal = $this->dbmcmd($dbm,"param_addvolume $num $type \"$path\" $kind $size") and return $retVal;
  }
  return $retVal;
}#DoParamAddVolumes
sub DoDBCreate{
  my ($this) = @_;
  my $retVal = 0;
  #do db_create
  my $aFlag = (isTrue($this->getAutoRestart()) and $^O =~ /mswin/i) ? "-a" : "";
  my $depPath = $this->getDependendPath();
  $depPath or  return "Parameter DEPENDENDPATH missing." ;
  -d $depPath or return "Dependend path '$depPath' does not exist.";

  my $depRelPath = undef;
  if(exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
    $depRelPath = getRelPath($depPath,0);
    defined $depRelPath or return "Could not find relative path for $depPath.";
  }
  my $depPath2 = defined $depRelPath ? $depRelPath : $depPath;
  my $dbname = $this->getDBName();
  my $dbmu = $this->getDBManager();
  $retVal =  $this->call_dbmcli(" -R \"$depPath2\" db_create  $aFlag $dbname $dbmu") and return $retVal; 
  return $retVal;
}#DoDBCreate


sub DoInitializeUsers{
  my ($this,$dbm) = @_;
  my $retVal = 0;
  if(@{$this->getSQLUser()}){
    $retVal = $this->DoCreateSQLUsers($dbm)
      and return $retVal;
  }
  if(@{$this->getXUserKey()}){
    $retVal = $this->DoCreateXUserKeys($dbm)
      and return $retVal;
  }
  return $retVal;
}

sub DoCreateSQLUsers{
  my ($this,$dbm) = @_;
  my $users = $this->getSQLUser();
  my $admin = $this->getDBAdmin();
  my  $retVal = 0;
  $retVal = $this->dbmcmd($dbm,"sql_connect  $admin") and return $retVal;
  foreach(@$users){
    my ($user,$pwd,$defcode,$cmd)= /$SQLUSERREGEXP/i;
    $defcode or $defcode='UNICODE';
    $cmd or $cmd =
      'sql_execute create user #USER password #PASSWORD dba not exclusive defaultcode #DEFAULTCODE';
    $cmd =~ s/#USER/$user/g;
    $cmd =~ s/#PASSWORD/$pwd/g;
    $cmd =~ s/#DEFAULTCODE/$defcode/g;
    if( $retVal = $this->dbmcmd($dbm,$cmd)){
      $this->dbmcmd($dbm,"sql_release");
      return $retVal;
    }#if
  }#foreach()
  $retVal = $this->dbmcmd($dbm,"sql_release") and return $retVal;
  return $retVal;
}#DoCreateSQLUsers



sub DoCreateXUserKeys{
  my ($this,$dbm) = @_;
  my $exec  = catfile($this->getIndepProg(), "bin","xuser") ;
  my $userKeys = $this->getXUserKey();
  my $dbname = $this->getDBName();
  $userKeys or return 0;
  my $retVal=0;
  foreach(@$userKeys){
    my ($user,$pwd,$aKey,$cmd)= /$XUSERKEYREGEXP/i;
    $cmd or $cmd =
      "$exec -u #USER,#PASSWORD -d #DBNAME -n localhost -U \"#KEY\"";
    $cmd =~ s/#USER/$user/g;
    $cmd =~ s/#PASSWORD/$pwd/g;
    $cmd =~ s/#KEY/$aKey/g;
    $cmd =~ s/#DBNAME/$dbname/g;
    my $logcmd =$cmd;
    $logcmd =~ s/(\w+),(\w+)/$1,***/g;
    $this->logit("[".$logcmd."]");
    foreach((1,2)){
      $retVal = callSystem($cmd,
                           sub{ my $line = $_[0];
                                $this->logit($line);
                              }) and return $retVal;
    }#foreach
  }#foreach()
  return $retVal;
}#DoCreateXUserKeys















sub CreateDBMCmdObject{
  my ($this) = @_;
  my $retVal = 0;
  # Create DBMCmd-Object for dbmcli-calls:
  my $dbname = $this->getDBName();
  my $dbmu = $this->getDBManager();
  my $host = exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}
    ? undef : 'localhost';
  my $dbm = SAPDB::Install::DBMCmd->new ($host, $dbname) or return ("DBMCmd not initialized!",undef);
  #Logon to the new database as DBM
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmu") and return ($retVal,undef);
  return ($retVal,$dbm);
}#CreateDBMCmdObject

#does param init and sets parameters from configuration:
sub DoInitializeDBParams{
  my ($this,$dbm) = @_;
  my $retVal = 0;
  #do a param  init first:
  $retVal = $this->dbmcmd($dbm,"param_startsession") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_init") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_commitsession") and return $retVal;
  #Set database parameters from configuration:
  my $params = $this->getDBParams();
  foreach (keys %$params) {
    $retVal = $this->dbmcmd($dbm,"param_directput " . $_ . " " . $params->{$_}) and return $retVal;
  }

  if(exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
    $retVal = $this->dbmcmd
      ($dbm,"param_directput RUNDIRECTORY wrk" .
       ($^O =~ /mswin/i ? '\\' : '/') .
       $this->getDBName())
        and return $retVal;
  }




  #Do param_check again!!
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  my $autoExtSize = $this->getAutoExtendSize();
  if($autoExtSize){
    my $cmd =  "dbm_configset  AutoExtSize $autoExtSize";
    $retVal = $this->dbmcmd($dbm,$cmd)
      and return $retVal;
  }
  return $retVal;
}#DoInitializeDBParams






#creates volume directories AND sets permission and owner on linux
sub DoCreateVolumeDirectories{
  my ($this) = @_;
  my $retVal = 0;
  my $isWin = $^O =~ /mswin/i ? 1 : 0;
  #Create volume paths:
  my $uid = $this->GetUID();
  my $gid = $this->GetGRID();
  my $vols = $this->getDBVolume();
  foreach (@$vols) {
    my $aVol = $_;
    my ($num,$type,$path,$kind,$size)= split(/;/,$aVol); 
    $path = makePathOSDependend($path);
    if( not file_name_is_absolute($path)){
      my $relativePath=$path;
      $path = $this->relativeVolumePathToAbsoluteVolumePath($path);
      $this->logit("Found relative volume '$relativePath'");
      $this->logit("Assuming full volume path '$path'");
    }
    my $volDir = dirname($path);
    $this->DoCreateOneVolumeDir($isWin,$uid,$gid,$volDir);
 }
  return $retVal;
}#DoCreateVolumeDirectories



sub relativeVolumePathToAbsoluteVolumePath{
  my ($this,$apath) = @_;
  if( file_name_is_absolute($apath)){
    return  $apath;
  }
  my $indepData = $this->getIndepData();
  my $dbname = $this->getDBName();
  my $absolutePath = catfile($indepData,'wrk',$dbname,$apath);
  return $absolutePath;
}

sub DoCreateOneVolumeDir{
  my ($this,$isWin,$uid,$gid,$volDir) = @_;
  my $retVal = 0;
  my $parentDir = dirname($volDir);
  if (not -d $volDir) {
    $this->logit("[mkpath  $volDir]");
    mkpath ($volDir) or return "Error creating directory $volDir: $!";
    unless ($isWin){
      $this->logit("[chown $uid:$gid $volDir]");
      chown ($uid, $gid, $volDir) == 1 or return "Cannot chown $uid:$gid '$volDir'";
      $this->logit("[chmod 0750 $volDir]");
      chmod (0750, $volDir) == 1 or return "Cannot chmod 0750 '$volDir";
    }
    if ($parentDir and not $isWin) {
      $this->logit("[chown $uid:$gid $parentDir]");
      chown ($uid, $gid, $parentDir) == 1 or return "Cannot chown $uid:$gid '$parentDir'";
      $this->logit("[chmod 0551 $parentDir]");
      chmod (0551, $parentDir) == 1 or return "Cannot chmod 0551 '$parentDir";
    }            
  } elsif (not $isWin) {
    $this->logit("[chown $uid:$gid $volDir]");
    chown ($uid, $gid, $volDir) == 1 or return "Cannot chown $uid:$gid '$volDir'";
    $this->logit("[chmod 0750 $volDir]");
    chmod (0750, $volDir) == 1 or return "Cannot chmod 0750 '$volDir";
    if ($parentDir ) {
      $this->logit("[chown $uid:$gid $parentDir]");
      chown ($uid, $gid, $parentDir) == 1 or return "Cannot chown $uid:$gid '$parentDir'";
      $this->logit("[chmod 0551 $parentDir]");
      chmod (0551, $parentDir) == 1 or return "Cannot chmod 0551 '$parentDir";
    }            
  }
  return $retVal;
}



sub GetSizeforVols{
  my ($this,$vols) = @_;
  my $allSize = 0;
  foreach my $aVol (@$vols) {
    my ($num,$art,$path,$type,$size )=split(/;/,$aVol);
    $allSize += $size;
  }
  return $allSize;
}
sub GetVolumeSize{
  my ($this,$art) = @_;
  my $vols = $this->filterVolumes($art,"complement");
  return $this->GetSizeforVols($vols);
}

sub guiValidateDBVolume{
  my ($this,$vols,$minimum) = @_;
  my @retVal = (1);
  my $dataSizeVar=0;
  my %nameChecker;
  defined $vols or return (0,"Volume list undefined","Specify volumes");
  @$vols or return (0,"No volumes specified","Please specify volumes");
  foreach my $aVol (@$vols) {
#    print $aVol,"\n";
    my ($num,$art,$path,$type,$size )=split(/;/,$aVol);
    my $aName = basename($path);
    if(exists $nameChecker{$aName}){
      return (0,"Duplicate volume name","Volume name $aName is used more than once.\n".
              "Use unique volume names");
    }
    else{
      $nameChecker{$aName} = 1;
    }
    $dataSizeVar+=$size;
  }
#  print "Datasizevar: $dataSizeVar\n";
  if ( $dataSizeVar <= 0) {
    my ($dataSize,$dataSizeUnit)=readableUnit($dataSizeVar,"PAGE"); 
    return (0,"Invalid value for data volume size",
            "Data size of $dataSize $dataSizeUnit does not make sense\n" . 
            "Specify  useful sizes for your log volumes");
  }
  #optionally test if data size is less than  a mimimum
  if ( $minimum and $dataSizeVar <    $minimum ) { 
    my ($dataSize,$dataSizeUnit)=readableUnit($dataSizeVar,"PAGE"); 
    my ($minSize,$minSizeUnit)=readableUnit($minimum,"PAGE"); 
    return (0,"Value for data size too low","Data size of $dataSize $dataSizeUnit is not enough.\n".
            "Specify  enough  space for data volumes ($minSize $minSizeUnit is Minimum)");
  }
  my $fsusage = $this->fileSystemUsageForVolumes($vols);
  @retVal = $this->ValidationResultFromFsUsage($fsusage,"Provide enough space for data and log volumes");
  return @retVal; 
}#guiValidateDBVolume



sub guiValidateVolstructArray{
  my ($this,$aStruct,$art) = @_;
  my $vols = [];
  my $counter =1;
  foreach(@$aStruct){
    push @$vols, ($this->volumeFromStruct($_,$counter,$art));
    $counter++;
  }
  
  #my $minimalDataSize = 8 * 1024; #Minimal datasize is hard coded to 64 MB
  my $minimalDataSize = 128 * 50; #Minimal datasize is hard coded to 50 MB
  return $this->guiValidateDBVolume($vols,( 'DATA' eq $art ? $minimalDataSize : undef ));
}#guiValidateVolstructArray




sub ValidationResultFromFsUsage{
  my ($this,$fsresult,$advice) = @_;
  my @retVal = (1);
  foreach my $key (keys %$fsresult) {
    my $fsdata  = $fsresult->{$key}; 
    my $neededkb = $fsdata->{"needed"};
    my $availablekb = $fsdata->{"available"};
    my $totalkb = $fsdata->{"total"};
    if ($neededkb > $availablekb) {
      my ($needed,$neededUnit)=readableUnit($neededkb,"KB"); 
      my ($available,$availableUnit)=readableUnit($availablekb,"KB"); 
      my ($total,$totalUnit)=readableUnit($totalkb,"KB"); 
      my $message =  "Not enough space on \"$key\" : (Available: $available $availableUnit, " .
        "Needed $needed $neededUnit)\n" . $advice;
      @retVal = (0,"Insufficent space left on device",$message);
      last;
    }
  }
  return @retVal; 
}



sub fileSystemUsageForVolumes{
  my ($this,$volumes) = @_;
  my $retVal = {};
  foreach my $volume (@$volumes) {
    my ($num,$art,$path,$type,$size )=split(/;/,$volume);
    if(not file_name_is_absolute($path)){
      next;
    }
    my $ksize = $size *8;
    my $aFS = GetFSName($path);
    next unless($aFS);
    my $fsData=undef;
    if ( defined $retVal->{$aFS}) {
      $fsData = $retVal->{$aFS};
    } else {
      $retVal->{$aFS} = {};
      $fsData = $retVal->{$aFS};
      my ($totalkb, $availkb) = GetFSSize($path);
      $fsData->{"total"} = $totalkb;
      $fsData->{"available"} = $availkb;
    }
    $fsData->{"needed"} += $ksize;
  }
  return $retVal;
}

sub structFromVolume{
  my ($this,$volume) = @_;
  my $retVal = {};
#  print "Volume: $volume\n";
  my ($num,$art,$path,$type,$size )=split(/;/,$volume);
  $path = makePathOSDependend($path);
  my $prefix = $art eq 'DATA' ? $this->getDataPathPrefix() : $this->getLogPathPrefix();
  my $postfix = $art eq 'DATA' ? $this->getDataPathPostfix() : $this->getLogPathPostfix();
  my ($drive) = ( $path =~ /^(\w:)/);
  my $dir = dirname($path);
  my $name = basename($path);
  my $label =  $art eq  "DATA" ?  $dataAreaLabel : $logAreaLabel;
  $retVal->{'Postfix'}=$postfix;
  $retVal->{'Prefix'}=$prefix;
  $retVal->{'Dir'}=$dir;
  $retVal->{'Drive'}=$drive;
  $retVal->{'Name'}=$name;
  $retVal->{'Number'}=$num;
  $retVal->{'Art'}=$art;
  $retVal->{'Location'}=$dir;
  $retVal->{'Type'}=$type;
  $retVal->{'Size'}=$size;
  $retVal->{'Label'}=$label;
  $retVal->{'DBInstance'}=$this;
  return $retVal;
}#structFromVolume

sub volumeFromStruct{
  my ($this,$aVol,$counter,$mart) = @_;
  my $location = $aVol->{"Location"};
  my $size = $aVol->{"Size"};
  my $name = $aVol->{"Name"};
  my $volumePath ; 
  if(defined $location and $location =~ /\S/){
    $volumePath = catfile($location,$name);
  }
  else{
    $volumePath = $name;
  }
  my $retVal = join(";", ($counter,$mart,$volumePath,"F",$size));
  return $retVal;
}




sub GetVolStructs{
  my ($this,$art)=@_;
  my $vols = $this->getDBVolume($art);
  my $retVal = [];
  foreach(@$vols){
    push @$retVal, ($this->structFromVolume($_));
  }
  return $retVal;
}


sub volStructToString{
  my ($this,$aStruct)= @_;
  my $s ='';
  foreach my $sm (@$aStruct){
    foreach(keys %$sm){
      $s .= $_.'='.$sm->{$_}.';';
    }
    $s.="\n";
  }
  return $s;
}





#filter out all volumes of art $art
sub filterVolumes{
  my ($this,$art,$complement)=@_;
  my $vols = $this->getDBVolume();
  my $newVols = [];
  if ($art) {
    foreach (@$vols) {
      my ($num,$mart)=split(/;/);
      if($complement){
        $mart eq  $art and push @$newVols, ($_);
      }
      else{
        $mart eq  $art or push @$newVols, ($_);
      }
    }
  }
  return $newVols;
}



sub clearVolumes{
  my ($this,$art)=@_;
  my $newVols = $this->filterVolumes($art);
  return $this->setDBVolume($newVols);
}

sub replaceVolumesByStruct{
  my ($this,$aStruct,$typeToReplace,$typeToKeep) = @_;
  my $counter = 1;
  my $volArray = [];
  $this->clearVolumes($typeToReplace);
  foreach my $aVol (@$aStruct) {
    push @$volArray, $this->volumeFromStruct($aVol,$counter,$typeToReplace);
    $counter++;
  }
  my $vols = $this->getDBVolume();
  push (@$vols,@$volArray);
  return  $this->setDBVolume($vols);
}

sub identifyLogDataStructsByLabel{
  my ($this,$aStruct) = @_;
  my $dataStruct=undef;
  my $logStruct=undef;
  foreach my $aVol (@$aStruct) {
    my $label = $aVol->{"Label"};
    if (defined $label) {
      if ($label eq $dataAreaLabel) {
        $dataStruct=$aVol;
      } elsif ($label eq $logAreaLabel) {
        $logStruct=$aVol;
      }
    }
  }
  return ($dataStruct,$logStruct);
}

sub getLogDataStructValues{
  my ($this,$aStruct,$aKey) = @_;
  my @aShit = $this->identifyLogDataStructsByLabel($aStruct);
  return $this->getLogDataStructValuesFromShit(\@aShit,$aKey);
}

sub getLogDataStructValuesFromShit{
  my ($this,$aShit,$aKey) = @_;
  my ($dataStruct,$logStruct) = @$aShit;
  my $dataValue = defined $dataStruct ? $dataStruct->{$aKey} : undef;
  my $logValue = defined $logStruct ? $logStruct->{$aKey} : undef;
  return ($dataValue,$logValue);
}
sub dirtyFillupVolstruct{
  my ($this,$aStruct,$art) = @_;
  my $num=1;
  foreach(@$aStruct){
    $_->{'Art'}=$art;
    $_->{'Label'}=$art eq 'DATA' ? $dataAreaLabel : $logAreaLabel;
    $_->{'Number'}=$num++;
  }
}


sub dbnameValidateCharacters{
  my ($this,$dbname) = @_;
  my @retVal = (1);
  my $length = $dbname ? length($dbname): 0;
  if (not defined $dbname) {
    my $message="Database name undefined";
    my $desc = "Specifiy database name";
    @retVal =(0,$message,$desc);
  } elsif ( $length > 8) {
    my $message="Database name too long";
    my $desc = "Only 8 characters are allowed in database name, you have $length";
    @retVal =(0,$message,$desc);
  } elsif ($length <= 0) {
    my $message="No database name";
    my $desc = "No database name entered, specify a database name";
    @retVal =(0,$message,$desc);
  } else {
    foreach ( split(//,$dbname)) {
      if ( ord($_)>127) {
        my $message="Invalid database name";
        my $desc = "Use ascii7 characters only in database name (\"$_\" not allowed) ";
        @retVal = (0,$message,$desc);
        last ;
      }
    }
  }
  if ( $retVal[0]) {
    @retVal = (1,"Good database name \"$dbname\"","Go on and feel happy")
  }
  return @retVal;
}

sub dbnameValidatorViaGetInstances{
  my ($this,$val) = @_;
  my %instance = getInstances();
  my @retVal = (1);
  foreach my $aKey (keys %instance) {
    if ((uc $aKey) eq (uc $val)) {
      @retVal = (0,"DB $val already exists.","Choose another database name");
      last;
    }
  }
  return @retVal;
}


sub dbnameValidatorViaInstInfo{
  my ($this,$val) = @_;
  my @retVal = (1);

  my $info = SAPDB::Install::InstInfo::new ($val);
  unless( defined $info) {
    @retVal = (2,"DB Info not available","SAPDB::Install::InstInfo returned undef");
  }
  my $infoListRef =  undef;
  if ($retVal[0]==1) {
    $infoListRef =  $info->{'database'};
    unless(defined $infoListRef){
      @retVal = (2,"DB Info not available","SAPDB::Install::InstInfo->{'database'} return undef");
    }
  }

  if ($retVal[0]==1) {
    my $dbInfo = $infoListRef->{$val};
    if ( defined $dbInfo) {
      @retVal = (0,"DB $val already exists.","Choose another database name");
    }
  }
  return @retVal;
}



#PARAMETERS:


#AUTORESTART

sub defaultAutoRestart{
  my ($this) = @_;
  return  $^O =~ /mswin/i ? "ON" : "";
}

sub initializeAutoRestart{
  my ($this) = @_;
  return  $this->defaultAutoRestart();
}

#ISAUTOLOGSTRATEGY:

sub getIsAutoLogStrategy{
  my $this = shift;
  return $this->getLogStrategy() eq 'AUTO';
}


sub getLogStrategyString{
  my $this = shift;
  my $logStrategy = $this->getLogStrategy();
  $logStrategy or return 0;
  if('AUTO' eq $logStrategy){
    return 'Automatic log backup';
  }
  elsif('NONE' eq $logStrategy){
    return 'No log backup (overwrite mode)';
  }
  elsif('MANUAL' eq $logStrategy){
    return 'Manual log backup';
  }
}

#STATUSLIST:
sub parseStatusList{
  my($this,$aKey,$aValue) = @_;
  my @ar = split/\n/,$aValue;
  my $retVal='';
  foreach(@ar){ /^\s*$/ or $retVal .= $_ . "\n"; }
  return $retVal;
}

#LOADSAMPLESCHEMAMESSAGE
sub getLoadSampleSchemaMessage{
  my $this = shift;
  $this->getLoadSampleSchema() ? "Loading tutorial data" : "Skipping load of tutorial data";
}

#AUTOEXTENDSTATUSMESSAGE
sub getAutoExtendStatusMessage{
  my $this = shift;
  $this->getAutoExtend() ? "Enabling automatic volume extension" : "Disabling automatic volume extension";
}

#DBMUSER:
sub setDBMUser{
  my ($this,$val) = @_;
  $this->Set('dBMUser',$val,'no can');
  my $p = $this->getDBMPass();
  $p or $p ='';
  $val or $val = '';
  $this->Set('dBManager',$val.','.$p,'no can');
}
sub defaultDBMUser{
  my ($this) = @_;
  my $df = $this->GetDefault('dBManager'); 
  my ($user) = ($df =~ /(\w+),\w+/);
  return $user;
}

#DBMPASS:
sub setDBMPass{
  my ($this,$p) = @_;
  $this->Set('dBMPass',$p,'no can');
  my $name = $this->getDBMUser();
  $p or $p ='';
  $name or $name = '';
  $this->Set('dBManager',$name.','.$p,'no can');
}
#DBAUSER:
sub setDBAUser{
  my ($this,$val) = @_;
  $this->Set('dBAUser',$val,'no can');
  my $p = $this->getDBAPass();
  $p or $p ='';
  $val or $val = '';
  $this->Set('dBAdmin',$val.','.$p,'no can');
}

sub defaultDBAUser{
  my ($this) = @_;
  my $df = $this->GetDefault('dBAdmin'); 
  my ($user) = ($df =~ /(\w+),\w+/);
  return $user;
}

#DBAPASS:
sub setDBAPass{
  my ($this,$p) = @_;
  $this->Set('dBAPass',$p,'no can');
  my $name = $this->getDBAUser();
  $p or $p ='';
  $name or $name = '';
  $this->Set('dBAdmin',$name.','.$p,'no can');
}

#DBADMIN:
sub setDBAdmin{
  my ($this,$admin) = @_;
  $this->Set('dBAdmin',$admin,'no can');
  my ($user,$passwd) = split(/,/,$admin);
  $this->Set('dBAPass',$passwd,'no can');
  $this->Set('dBAUser',$user,'no can');
}


sub initializeDBAdmin{
  my $this = shift ;
  return  $this->GetDefault('dBAdmin');
}

#DBMANAGER:
sub setDBManager{
  my ($this,$manager) = @_;
  $this->Set('dBManager',$manager,'no can');
  my ($user,$passwd) = split(/,/,$manager);
  $this->Set('dBMPass',$passwd,'no can');
  $this->Set('dBMUser',$user,'no can');
}
sub initializeDBManager{
  my $this = shift ;
  return  $this->GetDefault('dBManager');
}





#DBPARAM:
sub formatDBParam{
  my ($this,$key,$value ) = @_;
  my $retVal = "#Clearing inherited parameters (Do not delete this line):\nDBPARAM  \n";
  if($value){
    foreach(keys %$value){
      $retVal .=  'DBPARAM  = ' . $_ . ' ' . $value->{$_}  . "\n";
    }
  }
  return  $retVal;
}


#XUSERKEY:
sub formatXUserKey{
  my ($this,$key,$value ) = @_;
  my $retVal = "#Clearing inherited xuser keys (Do not delete the following  line):\nXUSERKEY  \n";
  if($value){
    foreach(@$value){
      $retVal .=  'XUSERKEY  = '. $_ . "\n";
    }
  }
  return  $retVal;
}


sub getXUserKey{
  my ($this,$art,$noSubst)  = @_;
  my $l = $this->Get('xUserKey',"no can","no subst");
  my @outlist;
  foreach(@$l){
    my $val = $noSubst?  $_ :  $this->recursiveSubstitute($_);
    push @outlist, ($val);
  }
  return \@outlist;
}


sub validateXUserKey{
  my ($this,$val) = @_;
  my $retVal = 0;
  my ($user,$pwd,$xuserKey);
  if (not $val){
    return 0;
  }
  foreach(@$val){
    if(
       ($user,$pwd,$xuserKey)=/$XUSERKEYREGEXP/i){
      print "User: $user\n";
      print "Passwd: $pwd\n";
      print "xuserKey: $xuserKey\n";
    }
    else{
      return "Invalid value '$_', use '<USER>,<PASSWD>,<KEY>'";
    }
  }
  return 0;
}





#SQLUSER:
sub formatSQLUser{
  my ($this,$key,$value ) = @_;
  my $retVal = "#Clearing inherited users (Do not delete the following  line):\nSQLUSER  \n";
  if($value){
    foreach(@$value){
      $retVal .=  'SQLUSER  = '. $_ . "\n";
    }
  }
  return  $retVal;
}

sub getSQLUser{
  my ($this,$art,$noSubst)  = @_;
  my $l = $this->Get('sQLUser',"no can","no subst");
  my @outlist;
  foreach(@$l){
    my $val = $noSubst?  $_ :  $this->recursiveSubstitute($_);
    push @outlist, ($val);
  }
  return \@outlist;
}

sub validateSQLUser{
  my ($this,$val) = @_;
  my $retVal = 0;
  my ($user,$pwd,$defcode,$st);
  if (not $val){
    return 0;
  }
  foreach(@$val){
    if(
       ($user,$pwd,$defcode,$st)=/$SQLUSERREGEXP/i){
#      print "User: $user\n";
#      print "Passwd: $pwd\n";
#      print "defcode: $defcode\n";
#      print "Statement: $st\n";
    }
    else{
      return "Invalid value '$_', use '<USER>,<PASSWD>,ASCII|UNICODE'";
    }
  }
  return 0;
}












#DBVOLUMES:
sub formatDBVolume{
  my ($this,$key,$value ) = @_;
  my $retVal = "#Clearing inherited volumes (Do not delete the following  line):\nDBVOLUME  \n";
  if($value){
    foreach(@$value){
      $retVal .=  'DBVOLUME  = '. $_ . "\n";
    }
  }
  return  $retVal;
}
sub getDBVolume{
  my ($this,$art,$noSubst)  = @_;
  my $l = $this->Get('dBVolume',"no can","no subst");
  my @outlist;
  foreach(@$l){
    my $val = $noSubst?  $_ :  $this->recursiveSubstitute($_);
    if($art){
      my ($num,$mart)=split(/;/,$val);
      $mart eq $art and push @outlist, ($val);
    }
    else{
      push @outlist, ($val);
    }
  }
  return \@outlist;
}

#DBNAME:
sub guiValidateDBName{
  my ($this,$val) = @_;
  my @retVal = $this->dbnameValidateCharacters($val);
  if ( $retVal[0]>= 1 ) {
    my $isExist = $this->instanceWithNameExists($val); 
    $isExist and @retVal = (0,"Instance already exists","Database $val must not exist");
  }
  return @retVal;
}

sub validateDBName{
  my ($this,$val) = @_;
  my $retVal = 0;
  my @res =  $this->guiValidateDBName($val);
  my $returnCode = $res[0];
  if($returnCode == 0){
    my $sd = $res[1];
    my $ld = $res[2];
    $retVal = "$sd\n$ld";
  }
  return $retVal;
}
#PARALLELCONNECTIONS

sub setParallelConnections{
  my ($this,$val) = @_;
  $this->Set('parallelConnections',$val,"no can");
  my $dbParam = $this->getDBParams();
  $dbParam->{'MAXUSERTASKS'} = $val;
  $this->setDBParams($dbParam);
}
sub minParallelConnections{
  return 1;
  }

sub maxParallelConnections{
  return 50000;
}


#LOGGINGSTRUCT:
sub setLoggingStruct{
  my ($this, $loggingStruct) = @_;
  if (ref $loggingStruct) {
    $this->SetLoggingStructHash($loggingStruct);
  } elsif ($loggingStruct) {
    $this->SetLoggingStructString($loggingStruct);
  }
}

sub SetLoggingStructString{
  my ($this, $loggingStruct) = @_;
  if($loggingStruct =~ /^\s*AUTO\s*\[([^;]*);(.*)\]\s*$/i){
    my $initialBackupDir = $1;
    my $logSaveDir = $2;
    $this->setInitialBackupDir($initialBackupDir);
    $this->setLogSaveDir($logSaveDir);
    $this->setAutoOverWriteLog(0);
    $this->setAutoSaveLog(1);
    $this->setLogStrategy('AUTO');
  }
  elsif( $loggingStruct =~ /^\s*MANUAL\s*$/i){
    $this->setLogStrategy('MANUAL');
    $this->setAutoOverWriteLog(0);
    $this->setAutoSaveLog(0);
  }
  elsif( $loggingStruct =~ /^\s*NONE\s*$/i){
    $this->setLogStrategy('NONE');
    $this->setAutoOverWriteLog(1);
    $this->setAutoSaveLog(0);
  }
  else{
    Logit("Invalid value for loggingStruct: $loggingStruct");
  }
}
sub SetLoggingStructHash{
  my ($this, $loggingStruct) = @_;
  my $isAutoOverWriteLog = $loggingStruct->{"AutoOverWriteLog"};
  my $isAutoSaveLog = $loggingStruct->{"AutoSaveLog"};
  my $initialBackupDir = $loggingStruct->{"InitialBackupDir"};
  my $logSaveDir = $loggingStruct->{"LogSaveDir"};
  $this->setAutoOverWriteLog($isAutoOverWriteLog);
  $this->setAutoSaveLog($isAutoSaveLog);
  $this->setInitialBackupDir($initialBackupDir);
  $this->setLogSaveDir($logSaveDir);
  if($isAutoOverWriteLog){
    $this->setLogStrategy('NONE');
  }
  elsif($isAutoSaveLog){
    $this->setLogStrategy('AUTO');
  }
  else{
    $this->setLogStrategy('MANUAL');
  }
}

sub getLoggingStruct{
  my ($this ) = @_;
  my $isAutoOverWriteLog = isTrue($this->getAutoOverWriteLog());
  my $isAutoSaveLog = isTrue($this->getAutoSaveLog());
  my $initialBackupDir = $this->getInitialBackupDir();
  my $dataPrefix = $this->getDataPathPrefix();
  my $logSaveDir = $this->getLogSaveDir();
  
  my  $aLogStruct = {
                     "AutoOverWriteLog" => $isAutoOverWriteLog,
                     "AutoSaveLog" => $isAutoSaveLog,
                     "InitialBackupDir" => $initialBackupDir,
                     "LogSaveDir" => $logSaveDir
                    };
  return $aLogStruct;
}

sub formatLoggingStruct{
  my ($this,$key,$val ) = @_;
  my $overw = $val->{"AutoOverWriteLog"};
  my $savel = $val->{"AutoSaveLog"};
  my $logSaveDir = $val->{"LogSaveDir"};
  my $initial = $val->{"InitialBackupDir"};
  my $retVal;
  if($overw){
      $retVal = (uc $key) . ' =  NONE';
  }
  elsif($savel){
    $retVal = (uc $key) . ' =  AUTO['.$initial.','.$logSaveDir.']';
  }
  else{
    $retVal = (uc $key) . ' =  MANUAL';
  }
  return $retVal;
}




#DBMEMORY:
sub maxDBMemory{
  my ($this) = @_;
   my ($real,$unit) =getPhysicalMemoryTotal8KPages(); 
   my $dbsize = $this->GetVolumeSize('DATA');

   my $osFix = ($^O =~ /mswin/i) ? 128 * (1024/ 8) : 64  *  (1024/ 8)   ;
   $osFix = sprintf "%d" , $osFix;
   my $max = $real - $osFix;
   $max = $dbsize <= $max ? $dbsize: $max;
  return $max;
}
sub minDBMemory{
  return 1000;
}
sub defaultDBMemory{
  my ($this) = @_;
  my ($real,$unit) =getPhysicalMemoryTotal8KPages(); 
  my $perc =  0.7;
  my $fullData= $this->GetVolumeSize('DATA');
  my $min = $this->minDBMemory();
  my $retVal =  $this->maxDBMemory();
  $retVal *= $perc;
  if ($min< $real) {
    $retVal = $min> $retVal ? $min: $retVal;
  }
  return sprintf "%d", $retVal;
}
sub setDBMemory{
  my ($this,$val) = @_;
  $this->Set('dBMemory',$val,"no can");
  my $dbParam = $this->getDBParams();
  $dbParam->{'CACHE_SIZE'} = $val;
  $this->setDBParams($dbParam);
}

#CPUCOUNT:

sub setCPUCount{
  my ($this,$val) = @_;
  $this->Set('cPUCount',$val,"no can");
  my $dbParam = $this->getDBParams();
  $dbParam->{'MAXCPU'} = $val;
  $this->setDBParams($dbParam);
}

sub maxCPUCount{
  getNumOfCPU();
}
sub minCPUCount{
  return 1;
}

sub defaultCPUCount{
  my ($this) = @_;
  my $cpu = getNumOfCPU();
  return sprintf "%d", ($cpu + 1) /2 ; 
}


# VERSION    = 7.7.00
# BUILD      = DBMServer 7.7.00   Build 009-103-112-029
# OS         = WIN32
# INSTROOT   = c:\sapdb\depend_7.7.00
# LOGON      = True
# CODE       = ASCII
# SWAP       = full
# UNICODE    = (unknown)
# INSTANCE   = (unknown)
# SYSNAME    = Windows 2000
# MASKING    = YES
sub dbm_version{
  my ($this,$output) = @_;
  my $retVal = 0;
  my @lines;
  $retVal = $this->call_dbmcli(" dbm_version",\@lines)
    and  return  $retVal . ": ". (join "\n",@lines);
  foreach(@lines){
    my ($key, $val) =  /(\w+)\s+=\s*(.+)\s*$/;
    $val and $val =~ s/\r$//;
    if($key){
      $key and $output->{$key} = $val;
    }
  }
  return $retVal;
}


# OK
# Returncode          	0
# Date                	20051114
# Time                	00114629
# Server              	p101402.dhcp.ber.sap.corp
# Database            	TESTDB
# Kernel Version      	Kernel    7.7.00   Build 009-103-112-029
# Pages Transferred   	0
# Pages Left          	0
# Volumes             	
# Medianame           	
# Location            	C:\Documents and Settings\All Users\Application Data\sdb\data\wrk\TESTDB\complete
# Errortext           	
# Label               	DAT_000000001
# Is Consistent       	true
# First LOG Page      	301
# Last LOG Page       	
# DB Stamp 1 Date     	20051114
# DB Stamp 1 Time     	00114614
# DB Stamp 2 Date     	
# DB Stamp 2 Time     	
# Page Count          	224
# Devices Used        	1
# Database ID         	p101402.dhcp.ber.sap.corp:TESTDB_20051114_114629
# Max Used Data Page  	0
# Converter Page Count	8
sub medium_label{
  my ($this,$file,$output) = @_;
  my $retVal = 0;
  my $cmd = " medium_labeloffline \"" . $file . "\" FILE ";
  $retVal = $this->call_dbmcli($cmd,$output) and  return  $retVal;
  return $retVal;
}


sub validateBackup{
  my ($this,$file) = @_;
  return $this->checkBackup($file);
}

sub checkBackup{
  my ($this,$file,$maxSize) = @_;
  my $retVal = 0;
  my @label;
  my $version = {};
  -f $file or return "File not found: $file";
  $retVal = dbm_version($this,$version) and return $retVal;
  my $dbmVersion = $version->{'VERSION'};
  my $kernelVersion;
  my $pageCount;
  $retVal = $this->medium_label($file,\@label) and  return  $retVal;
  foreach(@label){
    if(/^Kernel Version\s*\w+\s*([\w0-9.]+)/){
      $kernelVersion = $1;
    }
    elsif(/^Page Count \s*(\d+)/){
      $pageCount = $1;
    }
  }
  if( $kernelVersion ne $dbmVersion){
    return "Version check failed: '$kernelVersion' != '$dbmVersion'";
  }
  if( $maxSize and $maxSize <= $pageCount){
    return "Size check failed: Page Count $pageCount larger than $maxSize";
  }

  return $retVal;
}


package SAPDB::Install::Templates2::InstanceDesktop;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Registry;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::DatabaseInstance);

my $desktopData = << "XMLDATA";
<ParameterDefinitions>
  <Parameter id="loadSampleSchema"  persistent="YES"/>
  <Parameter id="dBSize" unit="pages" persistent="YES">
    <Type>DATASIZE</Type>
    <Description>Size of the database in 8K-Pages</Description>
    <Default>32768</Default>
  </Parameter>
  <Parameter id="dependendPath">
  </Parameter>
  <Parameter id="dBUser"  persistent="YES">
    <Type>USER.COMPOUND</Type>
    <Description>User Name and password (comma separated). Standard password is 'SECRET'</Description>
    <Default>DBADMIN,</Default>
  </Parameter>
  <Parameter id="dBManager" >
    <Default>DBM,SECRET</Default>
  </Parameter>
  <Parameter id="dBAdmin" >
    <Default>DBADMIN,SECRET</Default>
  </Parameter>
  <Parameter id="volStructSimple"  >
    <Type>VOLUMELIST.COMPOUND</Type>
  </Parameter>
  <Parameter id="dataDrive" persistent="YES:win">
    <Type>DRIVE</Type>
    <Default>C</Default>
    <Description>Disk drive for software installation</Description>
  </Parameter>
  <Parameter id="dataPath" persistent="YES:ux">
    <Type>PATH</Type>
    <Description>Path for data volumes</Description>
    <Default>\${DATAPATHPREFIX}/\${DATAPATHPOSTFIX}</Default>
  </Parameter>
  <Parameter id="logPath" persistent="YES:ux">
    <Type>PATH</Type>
    <Description>Path for log volumes</Description>
    <Default>\${LOGPATHPREFIX}/\${LOGPATHPOSTFIX}</Default>
  </Parameter>
</ParameterDefinitions>
<ParameterData>
LOADSAMPLESCHEMA YES
#DBSIZE is 256 MB -> 32768 8K-Pages
DBSIZE 32768
DBNAME MAXDB1
PARALLELCONNECTIONS 10
LOGSTRATEGY NONE
AUTOEXTEND YES
AUTOUPDATESTATISTICS YES
#DBVOLUME 1;DATA;\${DATAPATH}/DISKD0001;F;\${DBSIZE}
#DBVOLUME 1;LOG;\${LOGPATH}/DISKL0001;F;6400
DBPARAM CACHE_SIZE 2500
DBMEMORY 2500
DBPARAM MAXDATAVOLUMES   64
DBPARAM _UNICODE   YES
DBPARAM _SERVERDB_FOR_SAP   NO
DBPARAM SESSION_TIMEOUT   0
DBPARAM XP_MAXPAGER 3
</ParameterData>
XMLDATA
my @desktopXmlData = split(/\n/,$desktopData);
#OVERWRITTEN METHODS:

sub initializeDBVolume{
  return defaultDBVolume();
}


sub defaultDBVolume{
  my $retVal;
  if(exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
  $retVal=   [
     "1;DATA;DISKD0001;F;\${DBSIZE}",
     "1;LOG;DISKL0001;F;6400"
    ];
  }
  else{
  $retVal=   [
     "1;DATA;\${DATAPATH}/DISKD0001;F;\${DBSIZE}",
     "1;LOG;\${LOGPATH}/DISKL0001;F;6400"
    ];
  }
  return $retVal;
}







sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@desktopXmlData);
  return \@newData;
}




sub defaultDependendPath{
  my ($this) = @_;
  my $template = $this->getInstTemplate();
  my $swPath = $template->getSoftwarePath();
  my $sysProgPath = $template->getSystemProgPath();
  my $dbname = $this->getDBName();
  my $portableRoot=  $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'};
  my $depPath ; 
  if($portableRoot){
    $depPath = catfile($portableRoot,$dbname,'db');
  }
  else{
    $depPath = catfile($sysProgPath,'sdb',$dbname);
  }
  return $depPath;
}#defaultDependendPath

sub initializeDependendPath{
  my ($this)=@_;
  my $retVal = $this->defaultDependendPath();
  return $retVal;
}

  

#PARAMETERS:

#DATADRIVE:
sub setDataDrive{
  my ($this,$val)= @_;
  $this->Set('dataDrive',$val,"no can");
  my $dataPathPrefix = $this->getDataPathPrefix();
  my $logPathPrefix = $this->getLogPathPrefix();
  $dataPathPrefix =~ s/^(\w:)/$val:/;
  $logPathPrefix =~ s/^(\w:)/$val:/;
  $this->setDataPathPrefix($dataPathPrefix);
  $this->setLogPathPrefix($logPathPrefix);
}





#VOLSTRUCTSIMPLE:
sub getVolStructSimple{
  my ($this) =  @_;
  return $this->GetVolStructs();
}
sub setVolStructSimple{
  my ($this,$aStruct) =  @_;
  $this->logit("Setting volumes:\n".$this->volStructToString($aStruct));
  my ($dataPath,$logPath) = $this->getLogDataStructValues($aStruct,'Location');
  my $dbname = $this->getDBName();
  $dataPath =~ s/$dbname/\$\{DBNAME\}/g;
  $logPath =~ s/$dbname/\$\{DBNAME\}/g;
  $dataPath and $this->setDataPath($dataPath);
  $logPath and $this->setLogPath($logPath);
}
sub volumePathConvention{
  my ($this,$aStruct,$aVal) =  @_;
  my $art = $aStruct->{'Art'};
  my $prefix = $art eq 'DATA' ? $this->getDataPathPrefix() : $this->getLogPathPrefix();
  my $postfix = $art eq 'DATA' ? $this->getDataPathPostfix() : $this->getLogPathPostfix();
  my $prefixDefault = $art eq 'DATA' ? $this->GetDefaultResolved('dataPathPrefix') : $this->GetDefaultResolved('logPathPrefix');
  my $newPrefix;
  my $newLocation;
  if($^O =~ /mswin/i){
    my ($oldDrive) = ($prefixDefault =~ /^\s*(\w):/);
    $oldDrive = uc $oldDrive;
    my $newDrive = $aVal;
    $newDrive =~ /^\s*(\w):/ and  $newDrive = $1;
    $newDrive = uc $newDrive;
    if($newDrive eq $oldDrive){
      $newPrefix = $prefixDefault;
    }
    else{
      $newPrefix = catfile($newDrive.':','sdb','data');
    }
  }
  else{
    $newPrefix = $aVal;
  }
  $newLocation = catfile($newPrefix,$postfix);
  return $newLocation;
}



#DBUSER:
sub getDBUser{
  my $this = shift;
  my $str = $this->Get('dBUser','no can');
  my $retVal = {};
  my ($name,$ps,$cps) = $this->splitGlobalUser($str);
  $retVal->{'Description'}='Database System Administrator';
  $retVal->{'Role'}="globaluser";
  $retVal->{'Name'}=$name;
  $retVal->{'Password'}=$ps;
  $retVal->{'ConfirmPassword'}=$cps;
  return $retVal;
}

sub setDBUser{
  my ($this,$val) = @_;
  my($name,$ps,$cps); 
  if(ref $val){
    ($name,$ps,$cps) = ($val->{'Name'},$val->{'Password'},$val->{'ConfirmPassword'});
    my $newVal = join(",",($name,$ps,$cps));
    $this->Set('dBUser',$newVal,'no can');
  }
  else{
    ($name,$ps,$cps) = $this->splitGlobalUser($val);
    return $this->Set('dBUser',$val,'no can');
  }
  $this->setDBAUser($name);
  $this->setDBAPass($ps);
  $this->setDBMPass($ps);
}
sub formatDBUser{
  my ($this,$key,$val ) = @_;
  my $retVal='';

  if(ref $val){
    $retVal = join (",", ($val->{'Name'},$val->{'Password'}))
  }
  else{
    $val =~ /^\s*(\w+)\s*,\s*(\w*)/ and $retVal = "$1,$2";
  }
#  print "Unformatted: '$val'\n";
#  print "Formatted: '$retVal'\n";
  $retVal = (uc $key) . "=" . $retVal;
  return $retVal;
}

sub splitGlobalUser{
  my ($this,$str) = @_;
  my ($name,$ps) = split(/,/,$str);
  $ps or $ps = ''; 
  return ($name,$ps,$ps);
}


sub guiValidateDBUser{
  my ($this,$val) = @_;

  my @retVal = (1);
  my @errorRets = (0,"Database user missing", 
                   "Enter data for user ");
  my @errorRets2 = (0,"Database user name missing", 
                    "Specify name of database user");
  
  if (not defined $val) {
    @retVal = @errorRets;
  } else {
    my $name = $val->{"Name"};
    my $psw = $val->{"Password"};
    my $cpsw = $val->{"ConfirmPassword"};
    my $dbm = $this->getDBMUser();
    unless(defined $name and not "" eq $name){
      @retVal = @errorRets2;
    }
    if ($retVal[0]) {
      @retVal = $this->confirmPasswordValidatorLabeled($psw,$cpsw,$name);
    }
    if ($retVal[0] ) {
      if ( (uc $dbm) eq  (uc $name)) {
        @retVal = (0,"Invalid user name", 
                   "User name $name not allowed, choose another name");
      }
    }
  }
  return @retVal;
}
#INDEPPROG:
sub validateIndepProg{
  my ($this,$val) = @_;
  $val or return "Cannot set path to an empty value";
  my ($indepData,$indepProg) = readIndepPath();
  $indepProg and $indepProg =~ /\S/ or return 0;
  if(not pathsEqualIgnoreCase($indepProg,$val)){
    return "Independend Programs path already set to '$indepProg'. Cannot change to '$val'";
  }
  return 0;
}



package SAPDB::Install::Templates2::InstanceCustom;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use strict;

our @ISA = qw (SAPDB::Install::Templates2::DatabaseInstance);

my $customData = << "XMLDATA";
<ParameterDefinitions>
  <Parameter id="dBSize" unit="pages">
    <Type>DATASIZE</Type>
  </Parameter>
  <Parameter id="cPUCount"  persistent="YES">
    <Description>Number of CPUs to be used by the database. (Default is system dependend)</Description>
  </Parameter>
  <Parameter id="parallelConnections" persistent="YES">
    <Description>Number of parallel connections</Description>
  </Parameter>
  <Parameter id="autoRestart" persistent="YES:win">
    <Description>Flag indicating automatic start after os reboot</Description>
  </Parameter>
  <Parameter id="autoUpdateStatistics" persistent="YES">
    <Description>Flag indicating wether statistics should be updated automatically</Description>
  </Parameter>
  <Parameter id="autoExtend" persistent="YES">
    <Description>Flag indicating if number of volumes is automatically extended</Description>
  </Parameter>
  <Parameter id="initialBackupDir" persistent="YES">
    <Description>Directory for initial backup</Description>
  </Parameter>
  <Parameter id="logSaveDir" persistent="YES">
    <Description>Directory for automatic log backups</Description>
  </Parameter>
  <Parameter id="logStrategy" persistent="YES">
    <Description>
Log Strategy. Possible value are:
\tAUTO Automatically save log 
\tNONE Log will be overwritten
\tMANUAL log backup must be  done manually
</Description>
  </Parameter>
  <Parameter id="adminUserStruct" >
    <Type>USERLIST.COMPOUND</Type>
  </Parameter>
  <Parameter id="dBAdmin" persistent="YES">
    <Description>Database System Administrator (USERNAME,PASSWORD)</Description>
    <Default>DBADMIN,SECRET</Default>
  </Parameter>
  <Parameter id="dBManager"  persistent="YES">
    <Description>Database Manager Operator (USERNAME,PASSWORD)</Description>
    <Default>DBM,DBM</Default>
  </Parameter>
  <Parameter id="dependendPath"  >
    <Default>\${SOFTWAREPATH}/\${DBNAME}</Default>
  </Parameter>
  <Parameter id="dBVolume"  persistent="YES">
    <Description>
Database Volumes
You can specify an arbitrary number of data and log volumes. 
An example for a log volume would be:
\tDBVOLUME 1;DATA;\${INDEPDATA}/\${DBNAME}/data/DISKD0001;F;131072
An example for a data volume would be:
\tDBVOLUME 1;LOG;\${INDEPDATA}/\${DBNAME}/log/DISKL0001;F;6400
The format is "DBVOLUME num;type;path;F;size"
\t'num':  Sequence number (data and log volumes seperately)
\t'type': DATA or LOG
\t'path': The full path of the volume
\t'size': Size in 8K pages
   
</Description>
  </Parameter>
  <Parameter id="softwarePath" >
    <Type>PATH</Type>
  </Parameter>
  <Parameter id="volstructLog" >
    <Type>VOLUMELIST.COMPOUND</Type>
  </Parameter>
  <Parameter id="volstructData" >
    <Type>VOLUMELIST.COMPOUND</Type>
  </Parameter>
  <Parameter id="loadSampleSchema" persistent="YES">
    <Default></Default>
  </Parameter>
</ParameterDefinitions>
<ParameterData>
DBNAME MAXDB1
PARALLELCONNECTIONS 20
AUTOSAVELOG  YES
LOADSAMPLESCHEMA NO
AUTOEXTEND YES
AUTOUPDATESTATISTICS NO
#DBVOLUME 1;DATA;\${INDEPDATA}/\${DBNAME}/data/DISKD0001;F;131072
#DBVOLUME 1;LOG;\${INDEPDATA}/\${DBNAME}/log/DISKL0001;F;6400
LOGGINGSTRUCT  AUTO[\${INDEPDATA}/\${DBNAME}/backup/data;\${INDEPDATA}/\${DBNAME}/backup/log]
#LOGGINGSTRUCT  MANUAL
#LOGGINGSTRUCT  NONE
DBPARAM CACHE_SIZE 2500
DBPARAM MAXDATAVOLUMES   64
DBPARAM _UNICODE   YES
#DBPARAM _SERVERDB_FOR_SAP   NO
</ParameterData>
XMLDATA
my @customXmlData = split(/\n/,$customData);

#OVERWRITTEN METHODS:

sub initializeDBVolume{
  return defaultDBVolume();
}

sub defaultDBVolume{
  my $retVal;
  my $portableRoot=  $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'};
  if($portableRoot){
  $retVal=
    [
     "1;DATA;DISKD0001;F;131072",
     "1;LOG;DISKL0001;F;6400"
    ];
  }
  else{
  $retVal=
    [
     "1;DATA;\${INDEPDATA}/\${DBNAME}/data/DISKD0001;F;131072",
     "1;LOG;\${INDEPDATA}/\${DBNAME}/log/DISKL0001;F;6400"
    ];
  }
  return $retVal;
  }



sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@customXmlData);
  return \@newData;
}




#ATTRIBUTES:
my $dbmDesc = "Database Manager Operator";
my $dbaDesc = "Database System Administrator";

#PARAMETERS:
sub getSoftwarePath{
  my $this =  shift;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->getSoftwarePath();
}
sub setSoftwarePath{
  my ($this,$val)  =  @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->setSoftwarePath($val);
}


#ADMINUSERSTRUCT:
sub defaultAdminUserStruct{
  my ($this) = @_;
  my $retVal = [];
  my $dbm  = {
              "Description" => $dbmDesc,
              "Name" => $this->getDBMUser(),
              "Password" => "",
              "ConfirmPassword" => "",
              "Role" => "dbm"
             };
  my $dba  = {
              "Description" => $dbaDesc,
              "Name" => $this->getDBAUser(),
              "Password" => "",
              "ConfirmPassword" => "",
              "Role" => "dba"
             };
  push @$retVal,($dbm,$dba);
  return $retVal;
}


sub setAdminUserStruct{
  my ($this,$aVal) = @_;
  $this->Set('adminUserStruct',$aVal,'no can');
  my $dbm   =undef;
  my $dba  = undef;
  foreach my $user (@$aVal) {
    my $role = $user->{"Role"};
    my $desc = $user->{"Description"};
    if ($dbaDesc eq $desc) {
      $dba = $user;
    } elsif ($dbmDesc eq $desc) {
      $dbm  = $user;
    }
  }
  $this->setDBMUser($dbm->{"Name"});
  $this->setDBMPass($dbm->{"Password"});
  $this->setDBAUser($dba->{"Name"});
  $this->setDBAPass($dba->{"Password"});
}

sub guiValidateAdminUserStruct{
  my ($this,$aVal) = @_;
  my $dbm   =undef;
  my $dba  = undef;
  my @retVal = (1);
  my @errorRets = (0,"DBM-User missing", 
                   "Enter data for dbm");
  my @errorRets2 = (0,"DBA-User missing", 
                    "Enter data for dba");
  my @errorRets3 = (0,"DBA-User name missing", 
                    "Enter name of dba user");
  my @errorRets4 = (0,"DBA-User name missing", 
                    "Enter name of dba user");
  my @errorRets5 = (0,"User names must not be equal", 
                    "Enter different  user names for DBM and DBA");
  foreach my $user (@$aVal) {
    my $desc = $user->{"Description"};
    if ($dbaDesc eq $desc) {
      $dba = $user;
    } elsif ($dbmDesc eq $desc) {
      $dbm  = $user;
    }
  }
  if (not defined $dbm) {
    @retVal = @errorRets;
  } elsif (not defined $dba) {
    @retVal = @errorRets2;
  } else {
    my $dbmName = $dbm->{"Name"};
    my $dbmPass = $dbm->{"Password"};
    my $dbmConfirm = $dbm->{"ConfirmPassword"};
    unless(defined $dbmName and not "" eq $dbmName){
      @retVal = @errorRets4;
    }
    if ($retVal[0]) {
      @retVal = $this->confirmPasswordValidatorLabeled($dbmPass,$dbmConfirm,$dbmName);
    }
    if ($retVal[0]) {
      my $dbaName = $dba->{"Name"};
      my $dbaPass = $dba->{"Password"};
      my $dbaConfirm = $dba->{"ConfirmPassword"};
      unless(defined $dbaName and not "" eq $dbaName){
        @retVal = @errorRets3;
      }
      if ($retVal[0]) {
        @retVal = $this->confirmPasswordValidatorLabeled($dbaPass,$dbaConfirm,$dbaName);
      }
    }
    if ($retVal[0]) {
      my $dbmName = $dbm->{"Name"};
      my $dbaName = $dba->{"Name"};
      if ((uc $dbmName) eq (uc $dbaName)) {
        @retVal = @errorRets5;
      }
    }
  }
  return @retVal;
}#guiValidateAdminUserStruct

#VOLSTRUCTLOG:
sub setVolstructLog{
  my ($this,$aStruct) = @_;
  $this->dirtyFillupVolstruct($aStruct,'LOG');
  $this->replaceVolumesByStruct($aStruct,'LOG','DATA');
}
sub getVolstructLog{
  my ($this) = @_;
  my $aStruct = $this->GetVolStructs("LOG");
  return $aStruct;
}
sub guiValidateVolstructLog{
  my ($this,$aStruct) = @_;
  $this->dirtyFillupVolstruct($aStruct,'LOG');
  my @retVal =  $this->guiValidateVolstructArray($aStruct,'LOG');
  unless($retVal[0]){
    my $err = $retVal[1];
    $this->logit("Validation error occured: '$err'\n",$this->volStructToString($aStruct));
  }
  return @retVal;
}

#VOLSTRUCTDATA:
sub setVolstructData{
  my ($this,$aStruct) = @_;
  $this->dirtyFillupVolstruct($aStruct,'DATA');
  $this->replaceVolumesByStruct($aStruct,'DATA','LOG');
}
sub getVolstructData{
  my ($this) = @_;
  my $aStruct = $this->GetVolStructs("DATA");
  return $aStruct;
}

sub guiValidateVolstructData{
  my ($this,$aStruct) = @_;
  $this->dirtyFillupVolstruct($aStruct,'DATA');
  my @retVal =  $this->guiValidateVolstructArray($aStruct,'DATA');
  unless($retVal[0]){
    my $err = $retVal[1];
    $this->logit("Validation error occured: '$err'\n",$this->volStructToString($aStruct));
  }
  return @retVal;
}



#DBSIZE:

sub defaultDBSize{
  my $this = shift;
  my $vols = $this->getDBVolume();
  my $asize = 0;
  foreach my $volume (@$vols){
    my ($num,$art,$path,$type,$size )=split(/;/,$volume);
    $art eq 'DATA' and $asize +=$size;
  }
  return $asize;
}




package SAPDB::Install::Templates2::InstanceSAPDW;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::InstanceCustom);

my $instancesapdwdata = << "INSTANCESAPDWDATA";
<ParameterDefinitions>
  <Parameter id="softwarePath"  />
  <Parameter id="dependendPath"  persistent="YES">
    <Default>\${SOFTWAREPATH}/\${DBNAME}/db</Default>
  </Parameter>
  <Parameter id="dBName"  persistent="YES">
   <Default>J2E</Default>
  </Parameter>
  <Parameter id="dataSize" persistent="YES" >
    <Type>DATASIZE</Type>
   <Default>131072</Default>
    <Description>Size of the default data volume</Description>
  </Parameter>
  <Parameter id="logSize" persistent="YES" >
    <Type>DATASIZE</Type>
   <Default>12800</Default>
    <Description>Size of the default log volume</Description>
  </Parameter>
  <Parameter id="dBAdmin">
   <Default>SUPERDBA,admin</Default>
  </Parameter>
  <Parameter id="dBManager">
   <Default>CONTROL,control</Default>
  </Parameter>
  <Parameter id="dBMemory"  persistent="YES" unit="pages">
    <Type>DATASIZE</Type>
    <Description>Memory consumption in 8K pages.</Description>
  </Parameter>
  <Parameter id="sQLUser"  persistent="YES" >
  </Parameter>
  <Parameter id="xUserKey"  persistent="YES" >
  </Parameter>
</ParameterDefinitions>
<ParameterData>
DBNAME J2E 
#DBVOLUME 
#DBVOLUME 1;DATA;\${SOFTWAREPATH}/\${DBNAME}/sapdata/DISKD0001;F;\${DATASIZE}
#DBVOLUME 1;LOG;\${SOFTWAREPATH}/\${DBNAME}/saplog/DISKL0001;F;\${LOGSIZE}
AUTOUPDATESTATISTICS YES
AUTOEXTEND YES
LOGSTRATEGY NONE
AUTOSAVELOG NO
AUTOOVERWRITELOG YES
PARALLELCONNECTIONS 5
#SQLUSER sapr3,sap,UNICODE
#SQLUSER sap\${DBNAME}DB,sap,ASCII
#XUSERKEY sapr3,sap,c_J2EE
#
DBPARAM 
DBPARAM _UNICODE YES
DBPARAM DEFAULT_CODE         ASCII
DBPARAM MAXUSERTASKS         5
DBPARAM CACHE_SIZE           1280
DBPARAM _PACKET_SIZE         131072
DBPARAM LOG_IO_QUEUE         66
DBPARAM MAXLOCKS             3000
DBPARAM CAT_CACHE_SUPPLY     3000
DBPARAM JOIN_MAXTAB_LEVEL4   64
DBPARAM CAT_CACHE_SUPPLY     704
DBPARAM FORMAT_DATAVOLUME    NO
</ParameterData>
INSTANCESAPDWDATA

#OVERWRITTEN METHODS:



sub initializeDBVolume{
  return defaultDBVolume();
}


sub defaultDBVolume{
  my $retVal;
  my $portableRoot=  $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'};
  if($portableRoot){
    $retVal=
      [
       "1;DATA;DISKD0001;F;\${DATASIZE}",
       "1;LOG;DISKL0001;F;\${LOGSIZE}"
      ];
  }
  else{
  $retVal=
    [
     "1;DATA;\${SOFTWAREPATH}/\${DBNAME}/sapdata/DISKD0001;F;\${DATASIZE}",
     "1;LOG;\${SOFTWAREPATH}/\${DBNAME}/saplog/DISKL0001;F;\${LOGSIZE}"
    ];
  }
  return $retVal;
}








sub GetXmlData{
  my $this = shift;
  return   $this->MergeData($instancesapdwdata,$this->SUPER::GetXmlData());
}









#PARAMETERS

#DBMEMORY
sub maxDBMemory{
  my ($this) = @_;
  return 10000;
}

sub minDBMemory{
  return 896;
}
sub defaultDBMemory{
  return 896;
}




 






#SOFTWAREPATH

sub getSoftwarePath{
  my ($this) = @_;
  my $instConfig = $this->getInstTemplate();
  my $swpath = $instConfig->getSoftwarePath();
  return $swpath;
}#getSoftwarePath




package SAPDB::Install::Templates2::InstanceMobileClient;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Untgz;
use SAPDB::Install::BackupPipe;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::InstanceCustom);

my $mobileClientData = << "MOBILECLIENTDATA";
<ParameterDefinitions>
  <Parameter id="dependendPath"  persistent="NO"/>
  <Parameter id="dBName" persistent="YES">
   <Default>MIDB</Default>
  </Parameter>
  <Parameter id="dataSize" persistent="YES" >
    <Type>DATASIZE</Type>
   <Default>393216</Default>
    <Description>Size of the database in 8K pages</Description>
  </Parameter>
  <Parameter id="logSize" persistent="YES" >
    <Type>DATASIZE</Type>
   <Default>25600</Default>
    <Description>Size of the log volume in 8K pages</Description>
  </Parameter>
  <Parameter id="dBAdmin"  persistent="NO">
    <Default>DBA,dba</Default>
  </Parameter>
  <Parameter id="initialBackupDir"  persistent="NO"/>
  <Parameter id="dBVolume"  persistent="NO"/>
  <Parameter id="loadSampleSchema"  persistent="NO"/>
  <Parameter id="dBManager"  persistent="NO"/>
  <Parameter id="autoUpdateStatistics"  persistent="NO"/>
  <Parameter id="autoRestart"  persistent="NO"/>
  <Parameter id="logSaveDir"  persistent="NO"/>
  <Parameter id="logStrategy"  persistent="NO"/>
  <Parameter id="cPUCount"  persistent="NO"/>
  <Parameter id="autoExtend"  persistent="NO"/>
</ParameterDefinitions>
<ParameterData>
DBNAME MIDB
AUTOUPDATESTATISTICS YES
AUTOEXTEND YES
#Log strategy:
LOGSTRATEGY NONE
AUTOSAVELOG NO
AUTOOVERWRITELOG YES
#DBVOLUME 
#DBVOLUME 1;DATA;\${SOFTWAREPATH}/\${DBNAME}/data/DISKD0001;F;\${DATASIZE}
#DBVOLUME 1;LOG;\${SOFTWAREPATH}/\${DBNAME}/log/DISKL0001;F;\${LOGSIZE}
#DBVOLUME 1;DATA;data/DISKD0001;F;\${DATASIZE}
#DBVOLUME 1;LOG;log/DISKL0001;F;\${LOGSIZE}
DBMEMORY 2500
DBPARAM 
DBPARAM MAXUSERTASKS   20
DBPARAM SESSION_TIMEOUT 0 
DBPARAM _UNICODE   YES
DBPARAM CACHE_SIZE 2500
LOGSTRATEGY NONE
  <ParameterValue id="statusList">
$SAPDB::Install::Templates2::DatabaseInstance::DEFAULTSTATUSLIST
instanceArchive|Loading backup|PENDING||
  </ParameterValue>
</ParameterData>
MOBILECLIENTDATA
my @mobileClientData = split(/\n/,$mobileClientData);




sub initializeDBVolume{
  return defaultDBVolume();
}


sub defaultDBVolume{
  my $retVal;
  my $portableRoot=  $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'};
  if($portableRoot){
  $retVal=
    [
     "1;DATA;DISKD0001;F;\${DATASIZE}",
     "1;LOG;DISKL0001;F;\${LOGSIZE}"
    ];
  }
  else{
  $retVal=
    [
     "1;DATA;\${SOFTWAREPATH}/\${DBNAME}/data/DISKD0001;F;\${DATASIZE}",
     "1;LOG;\${SOFTWAREPATH}/\${DBNAME}/log/DISKL0001;F;\${LOGSIZE}"
    ];
  }
  return $retVal;
}



#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@mobileClientData);
  return \@newData;
}


sub DoReset{
  my ($this ) = @_;
  $this->logit('Erasing the database contents.');
  #Create the DBM-Object and  log in
  my ($retVal,$dbm) = $this->CreateDBMCmdObject();
  $retVal and return   $retVal;
  #Erase everything by calling 'db_activate' 
  $retVal = $this->changeDBState($dbm,"db_admin") and return $retVal;
  $retVal = $this->DoDBActivate($dbm) and return $retVal;
  $dbm->release();
  return 0;
}



sub DoInstanceArchive{
  my ($this ) = @_;
  #Initialize the UNTGZ-Object from which to read the backup
  my $untgz; 
  my $retVal;
  $retVal = $this->DoInitializeUntgz(\$untgz) and return $this->creationError('instanceArchive',$retVal);
  

  #Create the DBM-Object and  log in
  my $dbm = $this->CreateDBMCmdObject();
  $retVal and return  $this->creationError('instanceArchive', $retVal);


  #Recover from the backup and reload system tables:
  $retVal = $this->changeDBState($dbm,"db_admin") and return $this->creationError('instanceArchive',$retVal);
  $retVal = $this->DoRecoverFromArchive($dbm,$untgz)and return $this->creationError('instanceArchive',$retVal);
  $retVal = $this->changeDBState($dbm,"db_online") and return $this->creationError('instanceArchive',$retVal);
  $retVal = $this->DoLoadSystab($dbm) and return $this->creationError('instanceArchive',$retVal);


  #Close the DBM-Connection
  $dbm->release ();
  return $retVal;
}#doInstanceArchive







sub DoRecoverFromArchive{
  my ($this,$dbm,$untgz) = @_;
  my $retVal ;
  #Put the medium:
  my $pipeFile = $^O =~ /mswin/i ? "\\\\.\\pipe\\SDBINSTPIPE" : "/opt/sdb/SDBINSTPIPE"; 
  $retVal = $this->dbmcmd($dbm,"medium_put SDBINSTPIPE  $pipeFile PIPE DATA 0 0 YES") and return $retVal;

  #Create pipe
  $this->logit("Creating pipe '$pipeFile'");
  my $bup = SAPDB::Install::BackupPipe->new ($pipeFile, 'w');
  defined $bup or return "Could not create pipe $pipeFile";

  #Start a util session
  $retVal = $this->dbmcmd($dbm,"util_connect") and return $retVal;  

  #Clear the logs. (This command is not documented (i.e. hidden)
  #If we did not do that, we could not get the database online
  #after the recovery, because we have done a db_activate before
  $retVal = $this->dbmcmd($dbm,"util_execute CLEAR LOG") and return $retVal;  

  # Start db_activate RECOVER. This starts reading from the pipe: 
  my $recoverCommand = "recover_start SDBINSTPIPE" ;
  $this->logit ("Sending '$recoverCommand' to DBM-Server.");
  my $activateId = $dbm->send ($recoverCommand);

  $retVal = writeToPipeFromTgz($this,$untgz,$bup,$dbm,$activateId,$pipeFile,$recoverCommand)
    and $this->returnDoRecoverFromArchive($dbm,$retVal);

  #Get Result of db_activate
  my $activateResult  = $dbm->recv ($activateId);
  my @activateLines =  split ('\n',$activateResult );
  my $activateSuccess = $activateLines[0];
  if ($activateSuccess =~ /^OK/) {
    $retVal=0;
    $this->logit("Result of \"$recoverCommand\":\n $activateResult\n");
  } else {
    $retVal = $activateResult;
  }
  return $this->returnDoRecoverFromArchive($dbm,$retVal);
}#DoRecoverFromArchive



sub DoRecoverFromBackupPipe{
  my ($this,$dbm,$file) = @_;
  my $retVal ;
  #Put the medium:
  my $pipeFile = $^O =~ /mswin/i ? "\\\\.\\pipe\\SDBINSTPIPE" : "/opt/sdb/SDBINSTPIPE"; 
  $retVal = $this->dbmcmd($dbm,"medium_put SDBINSTPIPE  $pipeFile PIPE DATA 0 0 YES") and return $retVal;

  #Create pipe
  $this->logit("Creating pipe '$pipeFile'");
  my $bup = SAPDB::Install::BackupPipe->new ($pipeFile, 'w');
  unless(defined $bup){
    $this->dbmcmd($dbm,"medium_delete SDBINSTPIPE");
    return "Could not create pipe $pipeFile";
  }

  #Start a util session
  if($retVal = $this->dbmcmd($dbm,"util_connect")){
    $this->dbmcmd($dbm,"medium_delete SDBINSTPIPE");
    return $retVal;
  }

  #Clear the logs. (This command is not documented (i.e. hidden)
  #If we did not do that, we could not get the database online
  #after the recovery, because we have done a db_activate before
#  if($retVal = $this->dbmcmd($dbm,"util_execute CLEAR LOG")){
#    $this->dbmcmd($dbm,"medium_delete SDBINSTPIPE");
#    $this->dbmcmd($dbm,"util_release"); 
#    return $retVal;
#  }

  # Start db_activate RECOVER. This starts reading from the pipe: 
#  my $recoverCommand = "recover_start SDBINSTPIPE" ;
  my $recoverCommand = "db_activate RECOVER SDBINSTPIPE DATA" ;
  $this->logit ("Sending '$recoverCommand' to DBM-Server.");
  my $activateId = $dbm->send ($recoverCommand);

  #Start writing into the pipe:
  if($retVal = writeToPipeFromFile($this,$bup,$dbm,$activateId,$pipeFile,$recoverCommand,$file)){
    $this->dbmcmd($dbm,"medium_delete SDBINSTPIPE");
    $this->dbmcmd($dbm,"util_release"); 
    return $retVal;
  }

  #Get Result of db_activate
  my $activateResult  = $dbm->recv ($activateId);
  my @activateLines =  split ('\n',$activateResult );
  my $activateSuccess = $activateLines[0];
  if ($activateSuccess =~ /^OK/) {
    $retVal=0;
    $this->logit("Result of \"$recoverCommand\":\n $activateResult\n");
  } else {
    $retVal = $activateResult;
  }
  $this->dbmcmd($dbm,"medium_delete SDBINSTPIPE");
  $this->dbmcmd($dbm,"util_release"); 
  return $retVal;
}#DoRecoverFromBackupPipe





sub writeToPipeFromFile{
  my ($this,$bup,$dbm,$activateId,$pipeFile,$recoverCommand,$file) = @_;
  my $retVal=0;
  #Open pipe
  $this->logit("Trying to open pipe '$pipeFile'");
  defined $bup->Open() or return "ERROR: Could not open pipe $pipeFile";
  $this->logit("Pipe '$pipeFile' opened");
  open (AFILE, $file) or return "Could not open file $file: $!";

  #Read the backup from the archive and write it to the pipe
  for (;;) {
    my ($got, $want, $buff);
    $want = 32 * 1024;
    $got = sysread(AFILE,$buff,$want);
    unless (defined $got) {
      $retVal = "ERROR: Could not read from file $file: $!\n";
      last;
    }
    if ($got <= 0) {
      last;
    }

    $want = $got;
    $got = $bup->Write ($buff, $want);
    if ($activateId == $dbm->select (0)) {
      $retVal = "ERROR: '$recoverCommand' terminated too early \"\n";
      last;
    }
  }
  #Close the pipe:
  $this->logit("Closing pipe '$pipeFile'");
  $bup->Close();
  close AFILE;
  return $retVal;
}



















sub writeToPipeFromTgz{
  my ($this,$untgz,$bup,$dbm,$activateId,$pipeFile,$recoverCommand) = @_;
  my $retVal=0;
  #Open pipe
  $this->logit("Trying to open pipe '$pipeFile'");
  defined $bup->Open() or return "ERROR: Could not open pipe $pipeFile";
  $this->logit("Pipe '$pipeFile' opened");
  #Read the backup from the archive and write it to the pipe
  for (;;) {
    my ($got, $want, $buff);
    $want = 32 * 1024;
    $got = $untgz->ExtractRead ($buff, $want);
    unless (defined $got) {
      $retVal = "ERROR: Could not read from recovery archive.\n";
      last;
    }
    if ($got <= 0) {
      last;
    }

    $want = $got;
    $got = $bup->Write ($buff, $want);
    if ($got != $want) {
      $retVal = "ERROR: Write failed to $pipeFile.(Wanted : $want, got $got)!\n";
      last;
    } else {
    #        $this->logit ("Wrote $want bytes\n");
    }

    if ($activateId == $dbm->select (0)) {
      $retVal = "ERROR: '$recoverCommand' terminated too early \"\n";
      last;
    }
  }
  #Close the pipe:
  $this->logit("Closing pipe '$pipeFile'");
  $bup->Close();
  return $retVal;
}




#sysread

sub returnDoRecoverFromArchive{
  my ($this,$dbm,$retVal) = @_;
  my $res ;
  $res = $this->dbmcmd($dbm,"medium_delete SDBINSTPIPE");
  $res and  $this->logit($res);
  $res = $this->dbmcmd($dbm,"util_release"); 
  $res and  $this->logit($res);
  return $retVal;
}



sub DoRecoverFromBackup{
  my ($this,$dbm,$file) = @_;
  my $retVal ;
  #Put the medium:
  $retVal = $this->dbmcmd($dbm,"medium_put SDBINSTFILE  \"$file\" FILE DATA 0 8 YES") and return $retVal;

  #Start a util session
  $retVal = $this->dbmcmd($dbm,"util_connect") and return $retVal;  

  #Clear the logs. (This command is not documented (i.e. hidden)
  #If we did not do that, we could not get the database online
  #after the recovery, because we have done a db_activate before
  $retVal = $this->dbmcmd($dbm,"util_execute CLEAR LOG")
    and return $this->returnDoRecoverFromBackup($dbm,$retVal);  

  # Start recover_start. This starts reading from the pipe: 
  my $recoverCommand = "recover_start SDBINSTFILE" ;
  $retVal = $this->dbmcmd($dbm,"recover_start SDBINSTFILE")
    and return $this->returnDoRecoverFromBackup($dbm,$retVal);  
  return $this->returnDoRecoverFromBackup($dbm,$retVal);
}#DoRecoverFromBackup








sub DoInitializeUntgz{
  my($this,$auntgz) = @_;
  my $retVal;

  my $instanceArchive = $this->getInstTemplate()->getInstanceArchive();
  -f $instanceArchive or return "Instance archive not found: '$instanceArchive'";
  #Open archive
  my $untgz = SAPDB::Install::Untgz::new ();
  my $archiveSuccess = $untgz->Open($instanceArchive);
  $retVal = $untgz->GetErr() and return "Cannot open '$instanceArchive': $retVal";

  #Find backup in Archive:
  my $bn = $this->getInstTemplate()->getInstanceArchiveBackup();
  my $aBackup;
  my @hrd = $untgz->Next();
  while (1) {
    unless(defined $hrd[0]){
      last;
    }
    my $bnn = $hrd[3]; 
    if ( (uc $bn) eq (uc $bnn)) {
      $aBackup = $bnn;
      last;
    }
    @hrd = $untgz->Next();
  }
  $aBackup or return "Backup '$bn' not found in archive '$instanceArchive'";
  $$auntgz = $untgz;
  return 0;
}


sub returnDoRecoverFromBackup{
  my ($this,$dbm,$retVal) = @_;
  my $res ;
  $res = $this->dbmcmd($dbm,"medium_delete SDBINSTFILE");
  $res and  $this->logit($res);
  $res = $this->dbmcmd($dbm,"util_release"); 
  $res and  $this->logit($res);
  return $retVal;
}



#PARAMETERS:
#DEPENDENDPATH:
sub defaultDependendPath{
  my ($this) = @_;
  my $instConfig = $this->getInstTemplate();
  my $swPath = $instConfig->getSoftwarePath();
  my $depPath = catfile($swPath,'${DBNAME}','db');
  return $depPath;
}#defaultDependendPath


my $prefMemory = 896;
#DBMEMORY:
sub minDBMemory{
  my ($this) = @_;
  my $maxm = $this->maxDBMemory();
  return $maxm <=  $prefMemory ? $maxm :  $prefMemory;
}
sub defaultDBMemory{
  my ($this) = @_;
  my $min = $this->minDBMemory();
  return $min >= $prefMemory ? $min : $prefMemory;
}












package SAPDB::Install::Templates2::InstanceMobileClientArchive;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Untgz;
use SAPDB::Install::BackupPipe;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::InstanceMobileClient);

my $mobileClientArchive = << "MOBILECLIENTARCHIVE";
<ParameterData>
</ParameterData>
MOBILECLIENTARCHIVE
my @mobileClientArchive = split(/\n/,$mobileClientArchive);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@mobileClientArchive);
  return \@newData;
}


sub createInstance{
  my ($this) = @_;
  my ($result,$retVal,$dbm) = (undef,0,undef);
  my $dbname = $this->getDBName();
  #Check if instance exists:
  $this->instanceExists() and  return "Database $dbname already exists (Abort)";
  #Prepare load of backup
  $retVal = $this->DoPrepareLoadBackup() and return $retVal;
  #Create the instance:
  $retVal = $this->SUPER::createInstance() and return $retVal;
  return $retVal;
}#createInstance()



sub DoPrepareLoadBackup{
  my ($this) = @_;
  my ($retVal);

  my $dataSize = $this->getDataSize();
  my $backupFile = $this->getInstTemplate()->getInstanceBackup();
  $retVal = $this->checkBackup($backupFile,$dataSize);
  if(not $retVal){
    $this->logit("Using backup '$backupFile' ...\n");
  }#if
  else{
    my $msg = "ERROR: Backup check for  '$backupFile' failed:\n".$retVal;
    $this->logit($msg);
    return  $msg;
  }#else
  return $retVal;
}#DoPrepareLoadBackup





#DoDBActivate
# sub DoDBActivate{
#   my ($this,$dbm) = @_;
#   my $retVal = 0;
#   my $admin = $this->getDBAdmin();
#   my $backupFile = $this->getInstTemplate()->getInstanceBackup();
#   $retVal = $this->dbmcmd($dbm,"medium_put SDBINSTFILE  \"$backupFile\" FILE DATA 0 8 YES")
#       and return $retVal;
#   #Start a util session
#   $retVal = $this->dbmcmd($dbm,"util_connect") and  return $this->returnDoRecoverFromBackup($dbm,$retVal);
#   $retVal = $this->async_dbmcmd($dbm,"db_activate RECOVER  SDBINSTFILE  DATA",1)
#     and return $this->returnDoRecoverFromBackup($dbm,$retVal);
#   $retVal = $this->dbmcmd($dbm,"util_release") and return $this->returnDoRecoverFromBackup($dbm,$retVal);
#   $retVal = $this->changeDBState($dbm,"db_online")
#     and return $this->returnDoRecoverFromBackup($dbm,$retVal);
#   return $this->returnDoRecoverFromBackup($dbm,$retVal);
#}#DoDBActivate

 sub DoDBActivate{
   my ($this,$dbm) = @_;
   my $retVal = 0;
   my $admin = $this->getDBAdmin();
   my $backupFile = $this->getInstTemplate()->getInstanceBackup();
   $retVal = $this->DoRecoverFromBackupPipe($dbm,$backupFile) and return $retVal;
   $retVal = $this->changeDBState($dbm,"db_online") and return $retVal;
   return $retVal;
}#DoDBActivate




 












package SAPDB::Install::Templates2::InstanceMobileClientReuse;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::InstanceMobileClient);

my $mobileClientReuseData = << "MOBILECLIENTREUSEDATA";
<ParameterData>
  <ParameterValue id="statusList">
doReset|Resetting database|PENDING||
instanceArchive|Loading backup|PENDING||
  </ParameterValue>
</ParameterData>
MOBILECLIENTREUSEDATA
my @mobileClientReuseData = split(/\n/,$mobileClientReuseData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@mobileClientReuseData);
  return \@newData;
}


sub createInstance{
  my ($this) = @_;
  my ($result,$retVal,$dbm) = (undef,0,undef);
  my $dbname = $this->getDBName();

  #Check if instance exists:
  $this->instanceExists() or  return "Database $dbname must exist to be reused!";

  #Reset the database
  $this->startStatus('doReset');
  $retVal = $this->DoReset() and return $this->creationError('doReset',$retVal);
  $this->endStatusSuccess('doReset');

  #Load  backup from Archive:
  $this->startStatus('instanceArchive');
  $retVal = $this->DoInstanceArchive() and return  $this->creationError('instanceArchive',$retVal);
  $this->endStatusSuccess('instanceArchive');

  return $retVal;
}#createInstance


#PARAMETERS:
#DBNAME:
sub guiValidateDBName{
  my ($this,$val) = @_;
  my @retVal = (1);
#  print "Gotcha ....\n";
  unless($this->instanceWithNameExists($val)){
    @retVal = (0,"Database $val does not exist", "Database $val must already exist in this template");
  }
  return @retVal;
}


#Create a MobileClient  database without using a backup archive
package SAPDB::Install::Templates2::InstanceMobileClientEmpty;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::InstanceMobileClient);

my $mobileClientEmptyData = << "MOBILECLIENTREUSEDATA";
<ParameterData>
  <ParameterValue id="statusList">
$SAPDB::Install::Templates2::DatabaseInstance::DEFAULTSTATUSLIST
  </ParameterValue>
</ParameterData>
MOBILECLIENTREUSEDATA
my @mobileClientEmptyData = split(/\n/,$mobileClientEmptyData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@mobileClientEmptyData);
  return \@newData;
}


#Create a MobileClient  database without using a backup archive
package SAPDB::Install::Templates2::InstanceSAPSBS;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::InstanceMobileClient);

my $DEFAULTDATASIZE_IS_500MB=500*128;
my $DEFAULTLOGSIZE_IS_200MB=200*128;

my $sapsbsData = << "SAPSBSDATA";
<ParameterDefinitions>
  <Parameter id="dBName" persistent="YES">
   <Default>SBS</Default>
  </Parameter>
  <Parameter id="dataSize" persistent="YES" >
    <Type>DATASIZE</Type>
   <Default>$DEFAULTDATASIZE_IS_500MB</Default>
  </Parameter>
  <Parameter id="logSize" persistent="YES" >
    <Type>DATASIZE</Type>
   <Default>$DEFAULTLOGSIZE_IS_200MB</Default>
  </Parameter>
  <Parameter id="parallelConnections" unit="number" persistent="YES">
    <Description>Number of parallel connections</Description>
    <Default>50</Default>
  </Parameter>
</ParameterDefinitions>
<ParameterData>
DBNAME SBS
PARALLELCONNECTIONS 50
AUTOEXTENDSIZE 100%
AUTORESTART   
DBMEMORY 80 MB
SQLUSER  SBOUSER,DBA
DBPARAM MAXUSERTASKS  50
DBPARAM CAT_CACHE_SUPPLY 400
DBPARAM MAXBACKUPDEVS 1
DBPARAM MAXSERVERTASKS 20
DBPARAM XP_MAXPAGER 2
DBPARAM SHAREDSQL YES
DBPARAM MAXDATAVOLUMES 4
DBPARAM _WORKDATA_SIZE 16384
DBPARAM DEFAULT_CODE UNICODE
  <ParameterValue id="statusList">
$SAPDB::Install::Templates2::DatabaseInstance::DEFAULTSTATUSLIST
  </ParameterValue>
</ParameterData>
SAPSBSDATA
my @sapsbsData = split(/\n/,$sapsbsData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@sapsbsData);
  return \@newData;
}




package SAPDB::Install::Templates2::InstanceMobileClientReset;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use strict;
our @ISA = qw (SAPDB::Install::Templates2::InstanceMobileClient);

my $mobileClientResetData = << "MOBILECLIENTRESETDATA";
<ParameterDefinitions>
</ParameterDefinitions>
<ParameterData>
  <ParameterValue id="statusList">
doReset|Resetting database|PENDING||
  </ParameterValue>
</ParameterData>
MOBILECLIENTRESETDATA
my @mobileClientResetData = split(/\n/,$mobileClientResetData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@mobileClientResetData);
  return \@newData;
}#GetXmlData


sub createInstance{
  my ($this) = @_;
  my ($result,$retVal,$dbm) = (undef,0,undef);
  my $dbname = $this->getDBName();

  #Check if instance exists:
  $this->instanceExists() or  return "Database $dbname must exist to be reused!";

  #Reset the database
  $this->startStatus('doReset');
  $retVal = $this->DoReset() and return $this->creationError('doReset',$retVal);
  $this->endStatusSuccess('doReset');


  return $retVal;
}#createInstance




#PARAMETERS:
#DBNAME:
sub guiValidateDBName{
  my ($this,$val) = @_;
  my @retVal = (1);
  unless($this->instanceWithNameExists($val)){
    @retVal = (0,"Database $val does not exist", "Database $val must already exist in this template");
  }
  return @retVal;
}



package SAPDB::Install::Templates2::PackageDBKernel;
use SAPDB::Install::System;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Instance;
use SAPDB::Install::MD5Sum;
unless($^O =~ /mswin/i){
  use SAPDB::Install::System::Unix;
  use SAPDB::Install::System::Unix::Dir;
}
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
sub myKeys{
  return {
          releaseId =>{
                       'DEFAULT' => sub{
                         my ($this) = @_;
                         my $aRelease  = $this->getVersion();
                         if($aRelease){
                           $aRelease =~ s/\.//g;
                           $aRelease =~ s/\d\d$//;
                         }
                         return $aRelease;
                       }
                      },
          mainPath =>{
                      'DEFAULT' => '${systemProgPath}/sdb/${releaseId}'
                     },
          kernelPackage => {},
          oldKernelPackage => {}
         };
}
my %deppath;
$deppath{'name'} = 'dependent path';
$deppath{'list'} = 'kernel.lst';
$deppath{'opt'} = 'depend=s';
$deppath{'mode'}=0775;
$deppath{'update'}=0;
sub GetPaths{ return (\%deppath); }



sub SetInstTemplate{
  my ($this,$value) = @_;
  $this->SUPER::SetInstTemplate($value);
  if($value){
    my $dbName = $value->getDBName();
    $this->setMainPath("\${SYSTEMPROGPATH}/sdb/$dbName");
  }
}



sub instanceErrorChecker{
  my ($this, $instance,$successStates) = @_;
  my $errorState = $instance->get_errorstate;
  my $msgtxt = $instance->{msgtxt};
  my $retVal = $errorState . ":\n" . $msgtxt;
  foreach (@$successStates) {
    $errorState eq $_ and return 0
  }
  return $retVal;
}


sub postprepare{
  my ($this,$oldPackage) = @_;
  my $retVal=0;
  $retVal = $this->SUPER::postprepare($oldPackage) and return $retVal;
  my $version = $this->getVersion();
  my $instConfig = $this->getInstTemplate();
  my $indep_data = $instConfig->getIndepData();
  my $mainPath = $this->getMainPath();
  my $user = $this->getUser();
  my $grp = $this->getGroup();
  my $uid = $this->GetUID();
  my $gid = $this->GetGRID();
  unless($^O =~ /mswin/i or $this->crelease2num($version) < $this->crelease2num('7.5.00.00')){
    my @nodes = (
                 {
                  'path' => catfile($indep_data,"config"),
                  'access' => 'rwx'
                 },
                 {
                  'path' => catfile($indep_data,"config","Databases.ini"),
                  'access' => 'rw'
                 },
                 {
                  'path' => catfile($indep_data,"config","Installations.ini"),
                  'access' => 'r'
                 }

		);
		
    foreach my $ref_node (@nodes) {
      my %node = %$ref_node; 
      my $aPath = $node{'path'};
      my $aAcc= $node{'access'};
      -l $aPath and $aPath = readlink($aPath);
      -e $aPath or next;
      $retVal=$this->testaccess ($aPath,$user,$aAcc) and return $retVal; 
    }
  }

  unless ($^O =~ /mswin/i) {
    chown ($uid, $gid, $mainPath) == 1 or
      return ("Cannot chown '$mainPath' to user $user and group $grp"); 
    chmod (0555, $mainPath) == 1 or return  ("cannot chmod '$mainPath' to 0555\n") ;
    # check access of parent directories
    unless (uaccess ($user, $mainPath) == 0) {
      my @ancestors =	SAPDB::Install::System::Unix::Dir::getancestors ($mainPath);
      foreach my $dir (@ancestors) {
        next if (-l $dir);
        my ($dev, $ino, $mode) = stat ($dir);
        next if (($mode & 0111) == 0111);
        chmod (($mode & 07777) | 00111, $dir);
      }
    }
    uaccess ($user, $mainPath) == 0 or return ("user $user cannot access directory $mainPath\n");
  }
  return $retVal;
}# kernel postprepare

#get internal parameters for the upgrade;
sub GetUpgradeParams{
  my ($this,$oldPackage) = @_;
  my $instConfig = $this->getInstTemplate();
  my $packageName = $this->getName();
  my $dbname = $instConfig->getDBName();
  my $path = $this->getMainPath();
  my $dbmuser = $instConfig->getDBMUser();
  my $version = $this->getVersion();
  my ( $apoTargetRelease,$kernelPackage,$oldKernelPackage,$targetRelease) = $this->GetInternalParams($oldPackage);
  my $oldVersion = $oldPackage ? $oldPackage->getVersion() : undef;
  my $upgradeFileName = $instConfig->getUpgradeFile();
  my $upgradeFile = SAPDB::Install::Templates2::UpgradeFile::new();
  return ( $instConfig , $packageName , $kernelPackage,$oldKernelPackage, $dbname ,
           $path , $dbmuser , $version, $apoTargetRelease , $targetRelease , $oldVersion ,
           $upgradeFileName , $upgradeFile); 
}


#get internal parameters for the upgrade;
#kernel specific
sub GetInternalParams{
  my ($this,$oldPackage) = @_;
  my $apoTargetRelease = undef;
  my ($kernelPackage,$oldKernelPackage)  = ($this,$oldPackage); 
  my $targetRelease =  $this->getVersion;
  return ($apoTargetRelease,$kernelPackage,$oldKernelPackage,$targetRelease);
}


sub isUpgrade{
  my ($this,$oldPackage)  = @_;
  unless($this->instanceExists ) {
    $this->warnit("Database instance not found\n");
    return (0,0);
  }
  my ($isResponsible,$retVal) = $this->isResponsible();
  $retVal and return (undef,"Could not check instance type:\n".$retVal);
  not $isResponsible and return (0,0);
  unless($oldPackage){
    $this->logit("No instance check necessary (in 'framestart')\n");
    return (0,0);
  }
  return (1,0);
}




# kernel framestart:
sub framestart{
  my ($this,$oldPackage) = @_;
  $this->SUPER::framestart($oldPackage);
  my ($retVal,$isUpgrade)=(0,0);
  my ( $instConfig , $packageName , $kernelPackage,$oldKernelPackage, $dbname ,
       $path , $dbmuser , $version, $apoTargetRelease , $targetRelease , $oldVersion ,
       $upgradeFileName , $upgradeFile)  = $this->GetUpgradeParams($oldPackage);
  -f $upgradeFileName and $retVal = $upgradeFile->parseFile($upgradeFileName) and return $retVal;
  if($upgradeFile->getInstanceCheckOK){
    $this->logit("Instance check was successfull last time\n");
    return 0;
  }
  $retVal=$this->call_x_server("start",['18641','19720']) and return $retVal;
  ($isUpgrade,$retVal) = $this->isUpgrade($oldPackage);
  $retVal and return $retVal;
  not $isUpgrade and return 0;
  $this->debugit(1,"Checking for multiple instances");
  $this->findInstances > 1 and 
    return "Multiple instance update not supported -  multiple instances on \"".$this->getMainPath . "\"\n";

  $this->debugit(1,"Creating upgrade object");
  my  $instance =$this->createUpgradeObject();
  my %sessiondata =  ( 'apo_target_release' => $apoTargetRelease,
                       'dbname' => $dbname,
                       'userpwd' => $dbmuser ,
                       'target_release' => $targetRelease,
                       'additional' => undef,
                       'logpos' => $upgradeFile->getLogPos,
                       'backup_logpos' => $upgradeFile->getBackupLogPos);
  $this->debugit(1,"Creating session object from upgrade object");
  my $dbm =$instance->session (%sessiondata);
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $this->debugit(1,"Starting instance preparation now");
  $instance->prepare();
  #save instance date here ....
  $upgradeFile->setApoStartingRelease($instance->{'apo_starting_release'});
  $upgradeFile->setStartingRelease($oldKernelPackage->getBuildString());
  $upgradeFile->setInstanceCheckOK($instance->get_errorstate () eq 'OK' ? 1 : 0);
  $upgradeFile->setLogPos($instance->{'logpos'});
  $upgradeFile->setBackupLogPos($instance->{'backup_logpos'});
  $upgradeFile->setMigrationStrategy($instance->{'migration_strategy'});
  $upgradeFile->setLCOwner($instance->{'lcowner'});
  $upgradeFile->setDefaultCode($instance->{'default_code'});
  $upgradeFile->setMCOD($instance->{'mcod'});

  
  $retVal = $upgradeFile->saveToFile($upgradeFileName) and return "Could not save upgrade status: '" . $retVal . "'";
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  #TODO
  
  $instance->shutdown();
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $instance->disconnect();
  $instance = undef;
  $retVal=$this->call_x_server("stop",['18640']) and return $retVal;
  return $retVal;
}

#kernel framestop
sub framestop{
  my ($this,$oldPackage) = @_;
  $this->SUPER::framestop($oldPackage);
  my ($retVal,$isUpgrade)=(0,0);
  $retVal=$this->call_x_server("start",['18641','19720']) and return $retVal;
  ($isUpgrade,$retVal) = $this->isUpgrade($oldPackage);
  $retVal and return $retVal;
  not $isUpgrade and return 0;
  my ( $instConfig , $packageName , $kernelPackage,$oldKernelPackage, $dbname ,
       $path , $dbmuser , $version, $apoTargetRelease , $targetRelease , $oldVersion ,
       $upgradeFileName , $upgradeFile)  = $this->GetUpgradeParams($oldPackage);
  not -f $upgradeFileName and return "Upgrade data file '$upgradeFileName' not found.";
  $retVal = $upgradeFile->parseFile($upgradeFileName) and return "Could not read data in upgrade file '$upgradeFile':\n$retVal";
  #get instance data from registry
  my  $instance = $this->createUpgradeObject();
  $instance->chvolperm($dbname, $upgradeFile->getStartingRelease(), $targetRelease);
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  my %sessionData =  	 (
                          'apo_target_release' => $apoTargetRelease,
                          'apo_starting_release' => $upgradeFile->getApoStartingRelease,
                          'default_code' => $upgradeFile->getDefaultCode,
                          'lcowner' => $upgradeFile->getLCOwner,
                          'dbname' => $dbname, 
                          'userpwd' => $dbmuser,
                          'target_release' => $targetRelease,
#                          'instancetype' => 'LCA',
                          'starting_release' => $upgradeFile->getStartingRelease,
                          'mcod' => $upgradeFile->getMCOD,
                          'migration_strategy' => $upgradeFile->getMigrationStrategy);
  my $dbm =$instance->session (%sessionData);
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $instance->finalize();
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $instance->disconnect();
  if ($instance->get_keep_instance_data) {
  } else {
    unlink $upgradeFileName or return "Could not delete upgrade file '$upgradeFileName': $!";  
  }
  return $retVal;
}


sub isResponsible{
  my ($this) = @_;
  my ($isLiveCache,$retVal) = $this->isLiveCache;
  $retVal and return (undef,$retVal);
  if($isLiveCache){
    $this->logit("Kernel package ignores liveCache upgrade\n");
    return (0,$retVal);
  }
  return (1,$retVal);
}




sub  createUpgradeObject{
  my ($this) = @_;
  my $instConfig = $this->getInstTemplate();
  my  $instance = SAPDB::Install::Instance->new (); 
  my $lt = $instConfig->GetListeners('log');
  $instance->{msg0} = sub { my $msg = join "\n",@_ ; foreach(@$lt){&$_($msg);}};
  $instance->{msg1} = sub { my $msg = join "\n",@_ ; foreach(@$lt){&$_($msg);}};

  my $statusListeners = $instance->{status_listeners};
  $statusListeners or $statusListeners = [];
  my @listeners = @$statusListeners;
  push @listeners, (sub { my $msg = join "\n",@_ ; foreach(@$lt){&$_($msg);}});
  $instance->{status_listeners} = $statusListeners;
  return $instance;
}



sub preuninstall{
  my $this = shift;
  $this->SUPER::preuninstall();
  my %instances = SAPDB::Install::Registry::getInstances();  	
  foreach my $instance_name (keys(%instances)){
    my $dbroot = normalizeAPath($instances{$instance_name});
    my $mainPath = normalizeAPath($this->getMainPath());
    if($dbroot eq $mainPath){
      return "Found instance $instance_name in '$dbroot'\n";
    }
  }
  return 0;
}
sub register{
  my $this = shift;
  $this->SUPER::register();
  my ($retVal,$cmd,$release)=(0,undef,$this->GetRelease());
  $release gt '7.3' and $retVal = $this->do_dbmsrvctl() and return $retVal;
  $retVal = $this->do_instreg() and return $retVal;
  $release le '7.3' and $retVal = $this->do_xregcomp('-c') and return $retVal;
  return $retVal;
}


sub do_xregcomp{
  my ($this,$opt) = @_;
  my ($retVal,$cmd) = (0,undef);
  my $mainPath = $this->getMainPath();
  my $success=0;
  my $checker = sub { my $text=shift;
                      $this->debugit(1,$text);
                      $text =~ /Registration\s+done/ and $success=1;		
                      $text =~ /Registration\s+cleared/ and $success=1; };
  my $cmd1 = "$mainPath/bin/xregcomp $opt \\\"$mainPath/pgm/dbpinstall\\\"";
  my $cmd2 = "$mainPath/bin/xregcomp $opt \\\"$mainPath/sap/sphsapdb\\\"";
  if( $^O =~ /mswin/i){
    $cmd1 =~ s/\//\\/g;
    $cmd2 =~ s/\//\\/g;
  } 
  $this->logit("[$cmd1]");
  $retVal = callSystem($cmd1,$checker) and return "Error in xregcomp: $retVal";
  $success or return "Error in xregcomp\n";
  $success=0;
  $this->logit("[$cmd2]");
  $retVal = callSystem($cmd2,$checker) and return "Error in xregcomp: $retVal";
  $success or return "Error in xregcomp\n";
  return $retVal;
}


sub do_instreg{
  my ($this) = @_;
  my ($retVal,$cmd) = (0,undef);
  my $mainPath = $this->getMainPath();
  if( $^O =~ /mswin/i){
    $mainPath=~s/\//\\/g;
    $cmd= " -R \"" . $mainPath . "\" inst_reg -k \\\"" . $mainPath . "\\\"";
  }
  else{
    $cmd= " -R \"" . $mainPath . "\" inst_reg -k \"" . $mainPath . "\"";
  }
  $retVal = $this->call_dbmcli($cmd) and return $retVal;
  return $retVal;
}
sub do_dbmsrvctl{
  my $this = shift;
  my $retVal = 0;
  my $dbmsrvctl = catfile( $this->getMainPath(),"bin","dbmsrvctl");
  my $cmd = $dbmsrvctl . " start";
  $retVal = callSystem($cmd, sub{ }) and return "Error in '$cmd'\nReturn code $retVal\n";
  return $retVal;
}


sub unregister{
  my $this = shift;
  $this->SUPER::unregister();
  my ($release,$retVal) = ($this->GetRelease(),0);
  $release le '7.3' and $retVal = $this->do_xregcomp('-u') and return $retVal;
  my $mainPath = $this->getMainPath();
  my $cmd= " -R \"" . $mainPath . "\" inst_unreg";
  $retVal = $this->call_dbmcli($cmd) and return $retVal;
  return $retVal;
}

sub postinstall{
  my $this = shift;
  my $retVal=0;
  $retVal = $this->SUPER::postinstall(@_) and return $retVal;
  my $instConfig = $this->getInstTemplate();
  my $indepData = $instConfig->getIndepData();
  my $mainPath = $this->getMainPath();
  my $grid = $this->GetGRID();
  my $uid = $this->GetUID();
  my $version = $this->getVersion();
  #SKIPPED: remove instance data from earlier installations

  my @demofiles;                # files which have to be modified
  if ($^O=~/mswin/i) {
    @demofiles = ('misc/create_demo_db.cmd','misc/drop_demo_db.cmd');
  } else {
    @demofiles = ('misc/create_demo_db.sh','misc/drop_demo_db.sh'
                  ,'doc/FirstSteps/index.html', 'doc/FirstSteps/sapdbenv.sh','doc/FirstSteps/Perl/HowToPerl.html',
                  'doc/FirstSteps/Python/HowToPython.html','doc/FirstSteps/ODBC/HowToODBC.html',
                  'doc/FirstSteps/Java/HowToJava.html'
                 );
  }
  my $regmd5sums = $this->getFileList();
  foreach my $file (@demofiles) {
    my $longfile=catfile($mainPath,$file);
    if (-f $longfile) {
      my @statbuf = stat($longfile);
      open(FD,$longfile) or $this->warnit("Cannot open demo database script file \"$longfile\"\n") and next;
      my @newfile;              # array of newlines for file
      while (my $line = <FD>) {
        chomp($line);
        my $path=$mainPath;
        $^O=~/mswin/i and $path=~s/\//\\/g;
        $line=~s/\$DEP/$path/g;
        $path=$mainPath;
        $^O=~/mswin/i and $path=~s/\//\\/g;
        $line=~s/\$IND/$path/g;
        push(@newfile,$line);
      }
      close(FD);
      open(FD,">$longfile") or $this->warnit("Cannot create new demo database script file \"$longfile\"\n") and next;
      foreach my $line (@newfile) {
        print FD "$line\n";
      }
      close(FD);
      utime($statbuf[9],$statbuf[9],$longfile);
      foreach (keys %$regmd5sums) {
        if ($_ eq $file) {
          $regmd5sums->{$_} = MD5Sum($longfile);
        }
      }
    } else {
      $this->warnit("Cannot modify \"$longfile\"\n");
    }
  }
  #SKIPPED: Deletion of unsupported files.

  unless($^O=~/mswin/i){
    my $file =catfile($mainPath,'pgm/dbmverifypw');
    chown(0,$grid,$file) or $this->warnit("Cannot change owner to \"root\" for file \"$file\" - please doit manually\n");
    chmod(04550,$file) or $this->warnit("Cannot chmod file \"$file\"  to 04550- please doit manually\n");
    $file = catfile($mainPath,'pgm/starter');
    chmod (0500, $file) or $this->warnit("Cannot chmod \"$file\ to 0500 - please doit manually\n");;
    $file = catfile($mainPath,'pgm/starter/sdbstarter');
    chown (0, $grid, $file)or $this->warnit("Cannot chown \"$file\ to root," . $this->getGroup() . " - please doit manually\n");
    chmod (04510, $file) or $this->warnit("Cannot chmod \"$file\ to 04510 - please doit manually\n");
  }

  if ($^O =~ /mswin/i) {
    #old loader code:
    my $p1 = catfile($mainPath,'/env/LOADER.py');
    my $p2 = catfile($mainPath,'/env/LOADER.pyc');
    foreach ($p1,$p2) {
      -f $_ and (unlink $_ or $this->warnit("Cannot delete $_, please do it manually"));
    }
  }

  foreach my $file ('lcinit.log','lcinit.his','adatmp','adatmp2') {
    my $fullpath = catfile($mainPath,$file); 
    -f $fullpath and unlink($fullpath);
  }
  #SKIPPED: Migration registrations
  # 	migrateDBRegistrations($deppath{'value'}) unless($^O =~ /mswin/i);
  return $retVal;
}                               #postinstall





sub dirtyInstanceCheck{
  my $this = shift;
  my $retVal=0;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  if($this->instanceExists ) {
    $this->warnit("Database $dbname already exists\n");
    return 0;
  }
  $retVal=$this->call_x_server("start",['18641','19720']) and return $retVal;
  return $retVal;
}



sub isLiveCache{
  my ($this) = @_;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my ($retVal,$result,$instanceType,$isLiveCache)=(0,[],undef,0);

  $retVal = $this->call_dbmcli(" -u $dbmuser -d $dbname param_directget INSTANCE_TYPE",$result)
    and return (undef,$retVal);
  foreach(@$result){
    /^\s*INSTANCE_TYPE\s+(\w+)\s*$/ and $instanceType = $1;
  }
  $instanceType or return (undef,"Could not determine instance type:\n" . join("\n",@$result));
  if ('LVC' eq $instanceType) {
    $isLiveCache =1;
  }
  return ($isLiveCache,$retVal);
}


# creates an oltp instance
sub createInstance{
  my $this = shift;
  my $retVal=0;
  $retVal= $this->dirtyInstanceCheck() and return  $retVal;
  my $release = $this->GetRelease();
  $release le '7.3' and return $this->createInstance73();
  my $instConfig = $this->getInstTemplate();
  my $depPath = $this->getMainPath();
  $depPath =~ s/\//\\/g if $^O =~ /mswin/i;
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my $dbauser = $instConfig->getDBAUser();
  $retVal =  $this->call_dbmcli(" -R \"$depPath\" db_create  $dbname $dbmuser") and return $retVal; 
  my $dbm = SAPDB::Install::DBMCmd->new ('', $dbname) or return "DBMCmd not initialized!";
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmuser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_startsession") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_init") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_commitsession") and return $retVal;
  my $params = $instConfig->getDBParams();
  foreach(keys %$params){
    $retVal = $this->dbmcmd($dbm,"param_directput " . $_ . " " . $params->{$_}) and return $retVal;
  }
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->addVolumes($dbm) and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_admin") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_activate $dbauser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"load_systab -u $dbauser -ud domain") and return $retVal;
#  $retVal = $this->dbmcmd($dbm,"load_systab -u $dbauser") and return $retVal;
  return $retVal;
}

sub createInstance73{
  my ($this) = @_;
  my ($retVal) = (0);

  my $instConfig = $this->getInstTemplate();
  my $depPath = $this->getMainPath();
  $depPath =~ s/\//\\/g if $^O =~ /mswin/i;
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my $dbauser = $instConfig->getDBAUser();

  #do db_create
  $retVal =  $this->call_dbmcli(" -R \"$depPath\" db_create  $dbname $dbmuser") and return $retVal; 
  #Generate script:
  my $script = << "SCRIPTEND";
user_logon $dbmuser
param_startsession
param_init
param_checkall
param_commitsession
SCRIPTEND

  #Add database parameters to script:
  my $params = $instConfig->getDBParams();
  foreach (keys %$params) {
    $script .= "param_directput " . $_ . " " . $params->{$_} . "\n";
  }
  #Add a param_checkall if necessary
  keys %$params and $script .= "param_checkall\n";
  
  #Add the volumes:
  my $vols = $instConfig->getDBVolumes();
  foreach(@$vols){
    my ($num,$type,$path,$kind,$size)= split/;/; 
    $path =~ s/\\/\//g unless $^O =~ /mswin/i;
    $path =~ s/\//\\/g if $^O =~ /mswin/i;
    $retVal = $this->pmakedir(dirname($path),0777) and return $retVal;
    $script .= "param_adddevspace $num $type $path $kind $size\n";
  }

  $script .= << "SCRIPTEND2";
db_cold
util_connect $dbmuser
util_execute init config
util_activate  $dbauser 
util_release
load_systab -u $dbauser -ud domain
db_warm
SCRIPTEND2

  #Execute the script:
   $retVal =  $this->call_dbmcli_script($script,$dbmuser,$dbname) and return $retVal; 
  return $retVal;
}




sub addVolumes{
  my($this,$dbm) = @_;
  my $retVal = 0;
  my $instConfig = $this->getInstTemplate();
  my $vols = $instConfig->getDBVolumes();
  foreach(@$vols){
    my ($num,$type,$path,$kind,$size)= split/;/; 
    $path =~ s/\\/\//g unless $^O =~ /mswin/i;
    $path =~ s/\//\\/g if $^O =~ /mswin/i;
    $retVal = $this->pmakedir(dirname($path),0777) and return $retVal;
    $retVal = $this->dbmcmd($dbm,"param_addvolume $num $type \"$path\" $kind $size") and return $retVal;
  }
  return $retVal;
}


sub deleteInstance{
  my $this = shift;
  my $retVal=0;
  $this->GetRelease() le '7.3' and return $this->deleteInstance73();
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  unless($this->instanceExists ) {
    $this->warnit("Database $dbname not found\n");
    return 0;
  }
  my $dbmuser = $instConfig->getDBMUser();
  my $dbm = SAPDB::Install::DBMCmd->new ('', $dbname) or return "DBMCmd not initialized!";
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmuser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_offline") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_drop") and return $retVal;
  return $retVal;
}

sub deleteInstance73{
  my $this = shift;
  my $retVal=0;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  unless($this->instanceExists ) {
    $this->warnit("Database $dbname not found\n");
    return 0;
  }
  my $dbmuser = $instConfig->getDBMUser();
  $this->call_dbmcli_script("db_offline\ndb_drop\n",$dbmuser,$dbname);
  return $retVal;
}

sub instanceExists{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  return $this->instanceWithNameExists2($dbname);
}

sub instanceWithNameExists2{
  my ($this,$dbname)  = @_;
  my @instances = $this->findInstances();
  my $f=0;
  foreach(@instances){
    if((uc $_) eq (uc $dbname)){
      $f=1;
      last;
    }
  }
  return $f;
}




sub findInstances{
  my $this = shift;
  my $mainPath = normalizeAPath($this->getMainPath());
  my %instances = SAPDB::Install::Registry::getInstances();  	
  my @retVal;
  foreach my $instance_name (keys(%instances)){
    my $dbroot = normalizeAPath($instances{$instance_name});
    if($dbroot eq $mainPath ){
      push(@retVal,($instance_name));
    }
  }
  return @retVal;
}








package   SAPDB::Install::Templates2::PackageSQLS;
our @ISA = qw (SAPDB::Install::Templates2::PackageDBM);

sub myKeys{
my $instScript = <<"INSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-DlgOrder]
Dlg0=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcome-0
Count=5
Dlg1=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdAskDestPath-0
Dlg2=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdSelectFolder-0
Dlg3=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdStartCopy-0
Dlg4=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcome-0]
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdAskDestPath-0]
szDir=\${winInstPath}
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdSelectFolder-0]
szFolder=MySQL MaxDB
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdStartCopy-0]
Result=1
[Application]
Name=SQL Studio
Version=\${version}
Company=SAP AG
Lang=0009
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{BDF33DF1-74DD-42F7-B1E4-5FB009F322CC\}-DlgOrder]
Count=0
[\{40314691-03B7-4446-BD07-455BB5065FB8\}-DlgOrder]
Count=0
[\{1266F848-EDF7-482A-AC08-8572932505A1\}-DlgOrder]
Count=0
[\{3002A6A5-63C2-41ED-85DE-3438A737AF4C\}-DlgOrder]
Count=0
[\{F2AEC4C5-45F1-46D6-9E04-28C59D5CAA30\}-DlgOrder]
Count=0
[\{B75C64BC-0006-4437-9F51-641B18904F8B\}-DlgOrder]
Count=0
[\{05F72A77-4A2A-4B9E-B5E7-E53AE156FDF3\}-DlgOrder]
Count=0
INSTFILE
  my $uninstScript = <<"UNINSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-DlgOrder]
Dlg0=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcomeMaint-0
Count=3
Dlg1=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SprintfBox-0
Dlg2=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcomeMaint-0]
Result=303
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SprintfBox-0]
Result=1
[Application]
Name=SQL Studio
Version=\${version}
Company=SAP AG
Lang=0009
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{BDF33DF1-74DD-42F7-B1E4-5FB009F322CC\}-DlgOrder]
Count=0
[\{40314691-03B7-4446-BD07-455BB5065FB8\}-DlgOrder]
Count=0
[\{1266F848-EDF7-482A-AC08-8572932505A1\}-DlgOrder]
Count=0
[\{3002A6A5-63C2-41ED-85DE-3438A737AF4C\}-DlgOrder]
Count=0
[\{F2AEC4C5-45F1-46D6-9E04-28C59D5CAA30\}-DlgOrder]
Count=0
[\{B75C64BC-0006-4437-9F51-641B18904F8B\}-DlgOrder]
Count=0
[\{05F72A77-4A2A-4B9E-B5E7-E53AE156FDF3\}-DlgOrder]
Count=0
UNINSTFILE

  my $updateScript = <<"UPDATEFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-DlgOrder]
Dlg0={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdWelcome-0
Count=3
Dlg1={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdStartCopy-0
Dlg2={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdFinish-0
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdWelcome-0]
Result=1
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdStartCopy-0]
Result=1
[Application]
Name=SQL Studio
Version=\${version}
Company=SAP AG
Lang=0009
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[{CDEE62EA-7780-438C-BEE2-BEFF1C4E0D90}-DlgOrder]
Count=0
[{C5813181-443D-4786-8E7A-8942D357B7F0}-DlgOrder]
Count=0
[{74D357CA-ACF2-4BBD-AD30-9F873A8C4E6B}-DlgOrder]
Count=0
[{28488D74-4BBF-463F-9F83-10E38E5EFB10}-DlgOrder]
Count=0
[{E5DDC44F-C6A7-421B-AEC1-0AD3C883EB64}-DlgOrder]
Count=0
[{D6BA7E1B-4B0B-48C8-ACB4-4E5CC3754376}-DlgOrder]
Count=0
[{19CD7770-1B50-49BA-9025-72586042A74D}-DlgOrder]
Count=0
UPDATEFILE
  return {
          winInstPath =>{ 'DEFAULT' => '${indepProg}\SQLStudio' },
          winExePath =>{ 'DEFAULT' => '${indepProg}\misc' },
          mainPath =>{ 'DEFAULT' => '${indepProg}' },
          instScript =>{ 'DEFAULT' => $instScript },
          uninstScript =>{ 'DEFAULT' => $uninstScript },
          updateScript =>{ 'DEFAULT' => $updateScript },
          exePrefix =>{ 'DEFAULT' => 'SQLSTD' },
          versionKey => { 'DEFAULT' => 'SOFTWARE\SAP AG\SQL Studio\7.6'},
          classID => {},# must be read from registry
          uninstKey => { 'DEFAULT' => 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\${classID}'}
         };
}







package SAPDB::Install::Templates2::PackageCPCBase;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackageDBAnalyzer;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackageDBConnect;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackageJDBC;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackageLCSDK;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackagePCR;
use SAPDB::Install::SetUser;
use SAPDB::Install::Templates2::Util;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);



sub GetPCRVersion{
  my $this = shift;
  my $name = $this->getName();
  return $this->getName() =~ /^PCR\s* (\d+)$/ ? $1 : undef; 
}



sub registerWin{
  my $this = shift;
  my $mainPath = $this->getMainPath();
  $mainPath=~s/\//\\/g;
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
  my $callstring =
    ($pcrVersion >= 7500) ?
      "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -g -i -p \"$mainPath\\runtime\\$pcrVersion\"" :
        "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -i -p \"$mainPath\\runtime\\$pcrVersion\"";
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  return $retVal;
}

sub registerUX{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $indepData = $instConfig->getIndepData();
  my $inifile = catfile($indepData,'/config','Runtimes.ini');
  my $uid = $this->GetUID();
  my $grid = $this->GetGRID();
  my $mainPath = $this->getMainPath();
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
   if (-f $inifile) {
     chmod (0444, $inifile);
     chown ($uid,$grid,$inifile);
   }
  SetUser($uid) == -1 and return "Cannot set effective uid and gid\n";
#   callSystem('id',sub{return 1;});
   my $callstring =
     ($pcrVersion >= 7404) ?
       "$mainPath/runtime/$pcrVersion/bin/irconf -g -i -p $mainPath/runtime/$pcrVersion" :
         "$mainPath/runtime/$pcrVersion/bin/irconf -i -p $mainPath/runtime/$pcrVersion";
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  SetUser() == -1 and return "cannot set effective uid and gid back to root\n";
  return $retVal;
}

sub register{
  my $this = shift;
  $^O =~ /^MsWin/i ? $this->registerWin() : $this->registerUX();
}

sub unregister{
  my $this = shift;
  $^O =~ /^MsWin/i ? $this->unregisterWin() : $this->unregisterUX();
}

sub unregisterWin{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $mainPath = $this->getMainPath();
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
  my $callstring =
    ($pcrVersion >= 7404) ?
      "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -g -r -p \"$mainPath\\runtime\\$pcrVersion\"" :
        "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -r -p \"$mainPath\\runtime\\$pcrVersion\"";
  $callstring=~s/\//\\/g;
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  return $retVal;

}
sub unregisterUX{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $mainPath = $this->getMainPath();
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
  my $callstring =
    ($pcrVersion >= 7404) ?
      "$mainPath/runtime/$pcrVersion/bin/irconf -g -r -p $mainPath/runtime/$pcrVersion" :
        "$mainPath/runtime/$pcrVersion/bin/irconf -r -p $mainPath/runtime/$pcrVersion";
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  return $retVal;
}


package SAPDB::Install::Templates2::PackageCPC;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackageDBAna;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackagePython;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackageWebtools;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);
package SAPDB::Install::Templates2::PackageXML;
our @ISA = qw (SAPDB::Install::Templates2::InstUnit);

package SAPDB::Install::Templates2::PackageAPO;
use SAPDB::Install::Templates2::Util;
our @ISA = qw (SAPDB::Install::Templates2::PackageDBKernel);
sub myKeys{
  my $this = shift;
  my $retVal = $this->SUPER::myKeys();
  $retVal->{'mainPath'} = {
                           'DEFAULT' => '${systemProgPath}/sdb/${releaseId}/sap'
                          };
  return $retVal;
}
sub preinstall {return 0;}
sub preuninstall {return 0;}
sub preunpack {return 0;}
sub postprepare {return 0;}
sub postinstall {return 0;}
sub postuninstall {return 0;}
sub register {return 0;}
sub unregister {return 0;}

sub SetInstTemplate{
  my ($this,$value) = @_;
  $this->SUPER::SetInstTemplate($value);
  $this->setMainPath(catfile($this->SUPER::getMainPath(),'sap'));
}



sub isResponsible{
  my ($this) = @_;
  my ($isLiveCache,$retVal) = $this->isLiveCache;
  $retVal and return (undef,$retVal);
  if(not $isLiveCache){
    $this->logit("APO package ignores oltp upgrade\n");
    return (0,$retVal);
  }
  return (1,$retVal);
}






#get internal parameters for the upgrade;
#apo specific:
sub GetInternalParams{
  my ($this,$oldPackage) = @_;
  my $apoTargetRelease = $this->getVersion;
  $apoTargetRelease =~ s/\.(\d+)$/\ Build\ $1/; 
  my ($kernelPackage,$oldKernelPackage) = ($this->getKernelPackage,$this->getOldKernelPackage); 
  my $targetRelease =  $kernelPackage->getBuildString; 
  return ( $apoTargetRelease,$kernelPackage,$oldKernelPackage,$targetRelease);
}




#create a livecache instance
sub createInstance{
  my $this = shift;
  my $retVal=0;
  $retVal= $this->dirtyInstanceCheck() and return  $retVal;
  my $instConfig = $this->getInstTemplate();
  my $depPath = $this->getMainPath();
  $depPath =~ s/\//\\/g if $^O =~ /mswin/i;
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my $dbauser = $instConfig->getDBAUser();
  my $lcauser = $instConfig->getLCAUser();
  my $lcaencoding = $instConfig->getLCAEncoding();
  $retVal =  $this->call_dbmcli(" -R \"$depPath\" db_create  $dbname $dbmuser") and return $retVal; 
  my $dbm = SAPDB::Install::DBMCmd->new ('', $dbname) or return "DBMCmd not initialized!";
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmuser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_startsession") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_init lvc") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_commitsession") and return $retVal;
  my $params = $instConfig->getDBParams();
  foreach(keys %$params){
    $retVal = $this->dbmcmd($dbm,"param_directput " . $_ . " " . $params->{$_}) and return $retVal;
  }
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->addVolumes($dbm) and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_admin") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_activate $dbauser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"load_systab -u $dbauser -ud domain") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_offline") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"exec_lcinit init -uSQL $lcauser -e $lcaencoding") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_connect  $lcauser") and return $retVal;
#  $retVal = $this->dbmcmd($dbm,"sql_execute enable write transaction") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_execute create table \"__\$__LCFLAG\" " .
                          "(flag CHAR(1),begin CHAR(23), end CHAR(23),version CHAR(12))") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_execute insert into \"__\$__LCFLAG\" values ('X','nix', 'nix','nix')") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_execute grant select on \"__\$__LCFLAG\" to public") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_release") and return $retVal;
  return $retVal;
}


package SAPDB::Install::Templates2::UpgradeFile;
our @ISA = qw (SAPDB::Install::Templates2::Properties);


sub new(){
  my $this = { 'KEYS' =>{
                         'apoStartingRelease'  => { },
                         'startingRelease'  => { },
                         'instanceCheckOK'  => { },
                         'logPos'  => { },
                         'backupLogPos'  => { },
                         'mCOD'  => { },
                         #default character encoding of liveCache-Owner
                         'defaultCode'  => { },
                         #liveCache-Owner
                         'lCOwner'  => { },
                         'migrationStrategy'  => { }
                        }};
  bless $this ;
  $this->SUPER::initData();
  return  $this;
}

1;


# Local Variables: ***
# eval:   (setq my-own  (sdb-get-current-own)) ***
# eval:   (setq my-sdbrun (format "%s\\sys\\wrk\\fast\\install\\tmp\\sdbrun" my-own)) ***
# eval:   (setq my-src (format "%s\\sys\\src\\install\\perl" my-own)) ***
# eval:   (setq perl-dbg-flags (format "-w -Ic:\\aw\\patch -I%s -I%s\\readkey -I%s" my-sdbrun my-sdbrun  my-src))***
# End: ***
